/*
 *  File : bp.c
 *
 *  History
 *     Date       Author               Description
 *     -------    -----------------    --------------------------------------
 *     08/23/91   Andrew H. Fagg       Original Release
 *
 *
 */

#include "nsl_include.h"
#include <stream.h>
#include <math.h>

#include "bp_parms.h"
//#include "alib.h"

#define nl << "\n"
#define sp << " "


/*
 *  nsl_vector mat_mult_col_vec(nsl_matrix& mat, nsl_vector& vec)
 *
 *    Computes C = M * V
 *
 *      Where C and V are both column vectors and M is a matrix.
 *
 */

nsl_vector mat_mult_col_vec(nsl_matrix& mat, nsl_vector& vec)

{
  int xn;
  int yn;
  int vxn;
  int i, j;

  xn = mat.get_xn();
  yn = mat.get_yn();
  vxn = vec.get_xn();

  if(xn != vxn)
    {
      cout << "mat_mult_col_vec : dimension mismatch.\n";
      cout << xn sp << yn sp << vxn nl;
      exit(0);
    }

  nsl_vector out(yn);
  for(i = 0; i < yn; ++i)
    {
      out.elem(i) = 0.0;
      for(j = 0; j < xn; ++j)
	out.elem(i) += vec.elem(j) * mat.elem(j,i);
    }
      
  return out;
}
/*
 *  void extract_row_vector(nsl_matrix& mat, int row, nsl_vector& vec)
 *
 *   Copy the <row>th row of <mat> into <vec>.
 *
 */

/*
 *  nsl_vector mat_trans_mult_col_vec(nsl_matrix& mat, nsl_vector& vec)
 *
 *                  T
 *    Computes C = M * V
 *
 *    Where C and V are column vectors.
 *
 */

nsl_vector mat_trans_mult_col_vec(nsl_matrix& mat, nsl_vector& vec)

{
  int xn;
  int yn;
  int vxn;
  int i, j;

  xn = mat.get_xn();
  yn = mat.get_yn();
  vxn = vec.get_xn();

  if(yn != vxn)
    {
      cout << "mat_mult_col_vec : dimension mismatch.\n";
      cout << xn sp << yn sp << vxn nl;
      exit(0);
    }

  nsl_vector out(xn);
  for(i = 0; i < xn; ++i)
    {
      out.elem(i) = 0.0;
      for(j = 0; j < yn; ++j)
	out.elem(i) += vec.elem(j) * mat.elem(i,j);
    }
      
  return out;
}

void extract_row_vector(nsl_matrix& mat, int row, nsl_vector& vec)
{
  int col;
  int xn, yn;

  xn = vec.get_xn();
  yn = mat.get_yn();
  if(xn != yn)
    {
      cout << "Error\n";
      exit(0);
    }
  for(col = 0; col < xn; ++col)
    {
      vec.elem(col) = mat.elem(row, col);
    }
}

/*
 *  nsl_matrix vec_mult_vec_trans(nsl_vector& vecA, nsl_vector& vecB)
 *
 *                  T
 *    Return = A * B
 *
 */

nsl_matrix vec_mult_vec_trans(nsl_vector& vecA, nsl_vector& vecB)

{
  int An, Bn;
  int i, j;

  An = vecA.get_xn();
  Bn = vecB.get_xn();

  nsl_matrix out(Bn, An);

  for(i = 0; i < An; ++i)
    for(j = 0; j < Bn; ++j)
      out.elem(j, i) = vecA.elem(i) * vecB.elem(j);
  return out;
}

/*
 *  from: funcs.c
 *
 *   Andrew H. Fagg
 *   5/10/91
 */

/*
 * float random_value()
 *
 *  Return = a random number between [-1.0, 1.0].
 *
 */

float random_value()
{
  return(((float)(rand() % 30001)/ 15000.0) - 1.0);
}

/*
 * float random()
 *
 *  Return = a random number between [0.0, 1.0).
 *
 */

// float randomv()
// {
//   return((float)(rand() % 30000)/ 30000.0);
// }

/*
 *   nsl_vector square(nsl_vector& a)
 *
 *  Square each element in a vector.
 *
 */

// nsl_vector square(nsl_vector& a)
// {
//   int i;
//   int j;

//   i = a.get_xn();

//   nsl_vector out(i);

//   for(j = 0; j < i; ++j)
//     {
//       out.elem(j) = a.elem(j) * a.elem(j);
//     }
//   return(out);
// }

/*
 *  nsl_vector sqrt(nsl_vector& a)
 *
 *    Take the square-root of each element in a vector.
 *
 */

// nsl_vector sqrt(nsl_vector& a)
// {
//   int i;
//   int j;

//   i = a.get_xn();

//   nsl_vector out(i);

//   for(j = 0; j < i; ++j)
//     {
//       out.elem(j) = sqrt(a.elem(j));
//     }
//   return(out);
// }

/*
 *  nsl_vector abs(nsl_vector& a)
 *
 *   Take the absolute value of each element of a vector.
 */

// nsl_vector abs(nsl_vector& a)
// {
//   int i;
//   int len;

//   len = a.get_xn();

//   nsl_vector out(len);

//   for(i = 0; i < len; ++i)
//     out.elem(i) = fabs(a.elem(i));
//   return(out);
// }

/*
 *  nsl_vector apply_func(nsl_vector& a, double (*func)(double))
 *
 *    Apply a function to each element of a vector.  Returns 
 *  a new vector.
 */

// nsl_vector apply_func(nsl_vector& a, double (*func)(double))

// {
//   int i;
//   int len;

//   len = a.get_xn();

//   nsl_vector out(len);

//   for(i = 0; i < len; ++i)
//     out.elem(i) = (func)(a.elem(i));
//   return(out);
// }


/*
 *  nsl_matrix apply_func(nsl_matrix& a, double (*func)(double))
 *
 *    Apply a function to each element of a matrix.  Returns a
 *  new matrix.
 */

// nsl_matrix apply_func(nsl_matrix& a, double (*func)(double))

// {
//   int i, j;
//   int len1;
//   int len2;
  
//   len1 = a.get_xn();
//   len2 = a.get_yn();

//   nsl_matrix out(len1, len2);

//   for(i = 0; i < len1; ++i)
//     for(j = 0; j < len2; ++j)
//       out.elem(i,j) = (func)(a.elem(i,j));
//   return(out);
// }

/*
 *  nsl_vector sigmoid(nsl_vector& vec)
 *
 *   In place, apply the sigmoid function to each element of <vec>.
 *
 *  Return = a pointer to the modified vector.
 *
 */

nsl_vector sigmoid(nsl_vector& vec)

{
  int i;
  int xn;

  xn = vec.get_xn();
  nsl_vector out(xn);

  for(i = 0; i < xn; ++i)
    {
      out.elem(i) = (1.0 / (1.0 + exp(-vec.elem(i))));
    }
  return out;
}

/*
 *  void randomize(nsl_vector& vec)
 *
 *    Set the values of <vec> to random numbers in the range [-1.0, 1.0].
 *
 */

void randomize(nsl_vector& vec)

{
  int i;
  int init;
  int limit;

  init = vec.get_x0();
  limit = vec.get_x1();
  for(i = init; i <= limit; ++i)
    {
      vec.elem(i) = random_value();
    }
}

/*
 *  void randomize(nsl_matrix& mat)
 *
 *    Set the values of <mat> to random numbers in the range [-1.0, 1.0].
 *
 */

void randomize(nsl_matrix& mat)

{
  int i, j;
  int x0, x1;
  int y0, y1;

  x0 = mat.get_x0();
  x1 = mat.get_x1();
  y0 = mat.get_y0();
  y1 = mat.get_y1();

  for(i = x0; i <= x1; ++i)
    for(j = y0; j <= y1; ++j)
      {
	mat.elem(i,j) = random_value();
      }
}


NETWORK(BP);

#define BUF_LEN 100

                            /*  Input vector.  */
VECTOR(A_Input, NUM_INPUT);

                               /*  Vector used to display activations in a   */
                               /*    vertical format.                        */
#ifdef DISPLAY_ACTIVATIONS_FLAG
MATRIX(A_Input_R, NUM_INPUT, 1);
#endif

                                   /*  Required hidden unit information.  */
VECTOR(A_Hidden, NUM_HIDDEN);      /*   Activation. */
VECTOR(D_Hidden, NUM_HIDDEN);      /*   Delta.      */
VECTOR(T_Hidden, NUM_HIDDEN);      /*   Threshold.  */
VECTOR(DT_Hidden, NUM_HIDDEN);     /*   Change in threshold.  */
VECTOR(OLD_DT_Hidden, NUM_HIDDEN); /*   Last change in threshold.  */


                               /*  Vector used to display activations in a   */
                               /*    vertical format.                        */
#ifdef DISPLAY_ACTIVATIONS_FLAG
MATRIX(A_Hidden_R, NUM_HIDDEN, 1);
#endif

                                   /*  Required output unit info.   */
VECTOR(A_Output, NUM_OUTPUT);
VECTOR(D_Output, NUM_OUTPUT);
VECTOR(T_Output, NUM_OUTPUT);
VECTOR(DT_Output, NUM_OUTPUT);
VECTOR(OLD_DT_Output, NUM_OUTPUT);
VECTOR(Desired_Output, NUM_OUTPUT);

/* Yuri: output file var. */
#ifdef WRITE_INPUT_OUTPUT_FLAG
fstream fout;
#endif

                               /*  Vector used to display activations in a   */
                               /*    vertical format.                        */
#ifdef DISPLAY_ACTIVATIONS_FLAG
MATRIX(A_Output_R, NUM_OUTPUT, 1);
MATRIX(Desired_Output_R, NUM_OUTPUT, 1);
#endif

VECTOR(E_Output, NUM_OUTPUT);      /*     Error at output.  */

                                   /*  Weights.  */

MATRIX(W_I_H, NUM_INPUT, NUM_HIDDEN);   /*  Input to hidden.  */
MATRIX(W_H_O, NUM_HIDDEN, NUM_OUTPUT);  /*  Hidden to output. */

                                   /*  Change in weights.  */

MATRIX(DW_I_H, NUM_INPUT, NUM_HIDDEN);
MATRIX(DW_H_O, NUM_HIDDEN, NUM_OUTPUT);

                                   /*  Last change in weights.  */
MATRIX(OLD_DW_I_H, NUM_INPUT, NUM_HIDDEN);
MATRIX(OLD_DW_H_O, NUM_HIDDEN, NUM_OUTPUT);

                               /*  Patterns.    */
MATRIX(P_Input, MAX_PATTERNS, NUM_INPUT);
MATRIX(P_Output, MAX_PATTERNS, NUM_OUTPUT);


                               /*  Input to output weights.  */
#ifdef INPUT_TO_OUTPUT_WEIGHTS
MATRIX(W_I_O, NUM_INPUT, NUM_OUTPUT);
MATRIX(DW_I_O, NUM_INPUT, NUM_OUTPUT);
MATRIX(OLD_DW_I_O, NUM_INPUT, NUM_OUTPUT);
#endif

                               /*  Last hidden values are entered into */
                               /* the net. */
#ifdef LAST_HIDDEN_FLAG
VECTOR(A_Last_Hidden, NUM_HIDDEN);
MATRIX(W_LH_H, NUM_HIDDEN, NUM_HIDDEN);
MATRIX(DW_LH_H, NUM_HIDDEN, NUM_HIDDEN);
MATRIX(OLD_DW_LH_H, NUM_HIDDEN, NUM_HIDDEN);

                               /*  Flag read from file which signals  */
                               /* a refresh of the LH layer.          */
VECTOR(LH_Restart, MAX_PATTERNS);

DATA(pat_flag);   /*  Determines if the pattern is taken from the file */
                  /* during playback (0.0).  Otherwise, the input pattern */
                  /* is copied from the output (1.0).                     */

#ifdef LAST_HIDDEN_TO_OUTPUT_WEIGHTS
MATRIX(W_LH_O, NUM_HIDDEN, NUM_OUTPUT);
MATRIX(DW_LH_O, NUM_HIDDEN, NUM_OUTPUT);
MATRIX(OLD_DW_LH_O, NUM_HIDDEN, NUM_OUTPUT);
#endif

                               /*  Weight mask from last hidden to hidden.  */
#ifdef LH_H_WEIGHT_MASK
MATRIX(MASK_LH_H, NUM_HIDDEN, NUM_HIDDEN);
#endif
#endif

                               /*  Operation parameters.  */
DATA(lrate);
DATA(momentum);
DATA(tss_temp);
DATA(tss);

#ifdef GRADIENT_BIAS_FLAG
DATA(gradient_bias);
#endif

int num_pats = 0;
int current_pat;
int epoch;
int nsl_epoch;
int epoch_step;
int current_epoch;

                              /*  Weight file name.  */
char wfname[BUF_LEN] = "";

                              /*  Patterns.          */
int patterns_loaded_flag = 0;


                              /*  Auto-parameters storage. */
#ifdef AUTO_PARAMETERS_FLAG
DATA(ap_auto_parameters);
DATA(ap_start_lrate);
DATA(ap_max_momentum);
DATA(ap_start_momentum);
DATA(ap_settle_count);
DATA(ap_patience_count);
DATA(ap_counter);
DATA(ap_lrate_factor);
DATA(ap_lrate_growth_factor);
DATA(ap_momentum_step);
DATA(ap_max_derror);
DATA(ap_mode);
DATA(old_tss);
DATA(ap_up_tolerance);
DATA(ap_momentum_gain);

DATA(dtss);

#endif

fstream log_file;
int log_flag;



/*
 *  void initial_setup()
 *
 *    Initializes variables when program is started up.
 *      (not when init is called).
 */

void initial_setup()
{
#ifdef AUTO_PARAMETERS_FLAG
  ap_auto_parameters = AP_DEFAULT_AUTO_PARAMETERS;
  ap_start_lrate = AP_DEFAULT_LRATE;
  ap_start_momentum = AP_DEFAULT_MOMENTUM;
  ap_max_momentum = AP_DEFAULT_MAX_MOMENTUM;
  ap_settle_count = AP_DEFAULT_SETTLE_COUNT;
  ap_patience_count = AP_DEFAULT_PATIENCE_COUNT;
  ap_lrate_factor = AP_DEFAULT_LRATE_FACTOR;
  ap_lrate_growth_factor = AP_DEFAULT_LRATE_GROWTH_FACTOR;
  ap_momentum_step = AP_DEFAULT_MOMENTUM_STEP;
  ap_max_derror = AP_DEFAULT_MAX_DERROR;
  ap_up_tolerance = AP_DEFAULT_UP_TOLERANCE;
  ap_momentum_gain = AP_DEFAULT_MOMENTUM_GAIN;
#endif
  log_flag = FALSE;
}


/*
 *  void closelog_cmd()
 *
 *    Called when the "user closelog" command is issued.
 *
 */

void closelog_cmd()
{
  if(log_flag)
    {
      log_flag = FALSE;
      log_file.close();
      cout << "Closed.\n" nl;
    }
  else
    {
      cout << "No log file to close." nl;
    }
  
}


/*
 *  void openlog_cmd()
 *
 *   Open a log file - close the old one if it is already open.
 *
 */

void openlog_cmd()
{
  char wfname[BUF_LEN];

                          /*  Close old log if already open.  */
  if(log_flag)
    {
      cout << "Closing current log." nl;
      closelog_cmd();
    }

  cout << "File name => ";
  cin >> wfname;

                             /*  Open file.  */
  log_file.open(wfname, output);
  if(!log_file)
    {
      cerr << "Error opening file.\n";
      log_flag = FALSE;
      return;
    }
  else
    {
      log_flag = TRUE;
      cout << "Opened." nl;
    }
}


/*
 *  int check_file(fstream& file)
 *
 *    Checks <file> to make sure that no errors have occurred.
 *
 *  Return = TRUE if file status is _eof or _good.
 *           FALSE if status is _fail or _bad.
 */

int check_file(fstream& file)

{
  switch(file.rdstate())
    {
    case _fail :
    case _bad :
      return FALSE;
    default:
      return TRUE;
    }
}


/*
 *  void get_mask_cmd()
 *
 *    Prompt for a file name of a mask for the lh_h weights.  The
 *  mask header must state 'MASK_LH_H x'.  Where x is the number
 *  of hidden units.
 *
 */
#ifdef LH_H_WEIGHT_MASK

void get_mask_cmd()
{
  char buf[BUF_LEN];
  fstream file;
  int temp;

      /*  Get file name  */

  cout << "File name => ";
  cin >> buf;
                         /*  Open the file.  */

  file.open(buf, input);
  if(!file)
    {
      cerr << "Error opening file.\n";
    }
  else
    {
                         /*  File open, check the header.  */

      file >> buf;
      if(strncmp(buf, "MASK_LH_H", BUF_LEN))
	{
	  cerr << "Bad header, expecting ""MASK_LH_H""." nl;
	  file.close();
	  return;
	}
      file >> temp;
      if(temp != NUM_HIDDEN)
	{
	  cerr << "Wrong number of hidden units, expecting " << NUM_HIDDEN
	    << "." nl;
	  file.close();
	  return;
	}
      file >> MASK_LH_H;
      if(!check_file(file))
	{
	  cerr << "Not enough data." nl;
	  file.close();
	  return;
	}
      file >> buf;
      if(strncmp(buf, "end", BUF_LEN))
	{
	  cerr << "Do not find end marker (" << buf<<")." nl;
	  file.close();
	  return;
	}

      cout << "MASK_LH_H read." nl;
      file.close();
    }
}


/*
 *  void init_mask_cmd()
 *
 *    Set all elements of the MASK_LH_H mask to 1.
 *
 */

void init_mask_cmd()
{
  MASK_LH_H = 1.0;
}
#endif   /*  ifdef above get_mask_cmd().   */


/*
 *  void read_weights_cmd()
 *
 *    Read a set of weights from a user-specified file.  If a
 *  file name has already been used, then present it as a
 *  default.  The loaded weight file must match in terms of
 *  configuration and layer sizes.  Also, the "epoch" value
 *  is loaded, which is the total number of learning epochs
 *  to date.
 */

void read_weights_cmd()
{
  char buf[BUF_LEN];
  fstream file;
  int temp;

                         /*  No default?  */

  if(wfname[0] == '\0')
    {
      cout << "File name => ";
      cin >> wfname;
    }
  else
    {
                         /*  Default, if input is "." then use default.  */

      cout << "File name (. = " << wfname <<") => ";
      cin >> buf;
      if(buf[0] != '.' || buf[1] != '\0')
	strcpy(wfname, buf);
    }
                         /*  Open the file.  */

  file.open(wfname, input);
  if(!file)
    {
      cerr << "Error opening file.\n";
    }
  else
    {
                         /*  File open, check the number of input, output */
                         /* and hidden units.                               */
      file >> temp;
      if(temp != NUM_INPUT)
	{
	  cerr << "Wrong number of input units." nl;
	  file.close();
	  return;
	}
      file >> temp;
      if(temp != NUM_OUTPUT)
	{
	  cerr << "Wrong number of output units." nl;
	  file.close();
	  return;
	}
      file >> temp;
      if(temp != NUM_HIDDEN)
	{
	  cerr << "Wrong number of hidden units." nl;
	  file.close();
	  return;
	}
      file >> epoch;

                                /*  Check for appropriate configuration  */
                                /* flags (1 = set, 0 = not set).         */ 
      file >> temp;
#ifdef INPUT_TO_OUTPUT_WEIGHTS
      if(temp == 0)
	{
	  cerr << "Expecting input_to_output weights." nl;
	  file.close();
	  epoch = 0;
	  return;
	}
#else
      if(temp == 1)
	{
	  cerr << "Not expecting input_to_output weights." nl;
	  epoch = 0;
	  file.close();
	  return;
	}
#endif
      file >> temp;	
#ifdef LAST_HIDDEN_FLAG
      if(temp == 0)
	{
	  cerr << "Expecting last hidden configuration." nl;
	  epoch = 0;
	  file.close();
	  return;
	}
#else
      if(temp == 1)
	{
	  cerr << "Not expecting last hidden configuration." nl;
	  epoch = 0;
	  file.close();
	  return;
	}
#endif
      file >> temp;	
#ifdef LAST_HIDDEN_TO_OUTPUT_WEIGHTS
      if(temp == 0)
	{
	  cerr << "Expecting last_hidden_to_output weights." nl;
	  epoch = 0;
	  file.close();
	  return;
	}
#else
      if(temp == 1)
	{
	  cerr << "Not expecting last_hidden_to_output weights." nl;
	  epoch = 0;
	  file.close();
	  return;
	}
#endif
                              /*  Read the weights.  */

      file >> W_I_H;
      file >> W_H_O;
      file >> T_Hidden;
      file >> T_Output;
#ifdef INPUT_TO_OUTPUT_WEIGHTS
      file >> W_I_O;
#endif
#ifdef LAST_HIDDEN_FLAG
      file >> W_LH_H;
#endif
#ifdef LAST_HIDDEN_TO_OUTPUT_WEIGHTS
      file >> W_LH_O;
#endif
      file.close();
    }
}


/*
 *  void read_weights_cmd()
 *
 *    Read a set of weights from a user-specified file.  If a
 *  file name has already been used, then present it as a
 *  default.  The loaded weight file must match in terms of
 *  configuration and layer sizes.  Also, the "epoch" value
 *  is loaded, which is the total number of learning epochs
 *  to date.
 */

void write_weights_cmd()
{
  char buf[BUF_LEN];
  fstream file;
                             /*  No default file name.  */
  if(wfname[0] == '\0')
    {
      cout << "File name => ";
      cin >> wfname;
    }
  else                       /*  Default file name, present as a possibility.*/
    {
      cout << "File name (. = " << wfname <<") => ";
      cin >> buf;
      if(buf[0] != '.' || buf[1] != '\0')
	strcpy(wfname, buf);
    }
                             /*  Open file.  */
  file.open(wfname, output);
  if(!file)
    {
      cerr << "Error opening file.\n";
    }
  else
    {
                             /*  Write layer sizes.  */
      file << NUM_INPUT nl;
      file << NUM_OUTPUT nl;
      file << NUM_HIDDEN nl;
                             /*  Number of epocs.  */

      file << epoch nl;
                             /*  Write network configuration.  */

#ifdef INPUT_TO_OUTPUT_WEIGHTS
      file << (int) 1 nl;
#else
      file << (int) 0 nl;
#endif
#ifdef LAST_HIDDEN_FLAG
      file << (int) 1 nl;
#else
      file << (int) 0 nl;
#endif
#ifdef LAST_HIDDEN_TO_OUTPUT_WEIGHTS
      file << (int) 1 nl;
#else
      file << (int) 0 nl;
#endif
                    /*  Weights.  */

      file << W_I_H nl;
      file << W_H_O nl;
      file << T_Hidden nl;
      file << T_Output nl;
#ifdef INPUT_TO_OUTPUT_WEIGHTS
      file << W_I_O nl;
#endif
#ifdef LAST_HIDDEN_FLAG
      file << W_LH_H nl;
#endif
#ifdef LAST_HIDDEN_TO_OUTPUT_WEIGHTS
      file << W_LH_O nl;
#endif
      file.close();

    }
}


/*
 *  INIT_MODULE(bp_parms)
 *
 *   Initialize parameters.
 *
 */

INIT_MODULE(bp_parms)
{


/*  lrate = 0.4; */
  current_pat = 0;
  tss_temp = 0.0;
  tss = 0.0;
/*  momentum = 0.6;  */
  epoch = 0;
  nsl_epoch = 0;
#ifdef GRADIENT_BIAS_FLAG
  gradient_bias = 0.1;
#endif
  epoch_step = DEFAULT_EPOCH_STEP;
  current_epoch = 0;

#ifdef AUTO_PARAMETERS_FLAG
  if(ap_auto_parameters.elem() == 1.0)
    {
      lrate = ap_start_lrate;
      momentum = ap_start_momentum;
      ap_mode = AP_WAIT;
      old_tss = -1.0;
      ap_counter = 0.0;
    }
#endif
}


/*
 *  read_patterns_cmd()
 *
 *   Initialize the pattern matrix.  If patterns have not already been
 *  loaded, then prompt for a file name, otherwise prompt to see if
 *  the user wants to load another file.
 *
 */

void read_patterns_cmd()
{
  int i;
  int pat;
  char fname[100];
  int temp;

  fstream file;

  if(patterns_loaded_flag)
    {
      cout << "Load patterns (" << num_pats << " already loaded) => ";
      cin >> fname;
      if(fname[0] != 'y' && fname[0] != 'Y')
	return;
    }
  cout << "Pattern file name => ";
  cin >> fname;
  file.open(fname, input);
  if(!file)
    {
      cerr << "Error opening file.\n";
      file.close();
      return;
    }

  file >> num_pats;
  cout << "num_pats = " << num_pats nl;
  if(num_pats > MAX_PATTERNS)
    {
      cout << "Too many patterns." nl;
      num_pats = 0;
      file.close();
      return;
    }
  file >> temp;
  if(temp != NUM_INPUT)
    {
      cerr << "Wrong number of input units." nl;
      num_pats = 0;
      file.close();
      return;
    }
  file >> temp;
  if(temp != NUM_OUTPUT)
    {
      cerr << "Wrong number of output units." nl;
      num_pats = 0;
      file.close();
      return;
    }
  for(pat = 0; pat < num_pats; ++pat)
    {
#ifdef LAST_HIDDEN_FLAG
      file >> LH_Restart.elem(pat);
#endif
      for(i = 0; i < NUM_INPUT; ++i)
	{
	  file >> P_Input.elem(pat, i);
	}
      for(i = 0; i < NUM_OUTPUT; ++i)
	{
	  file >> P_Output.elem(pat, i);
	}
    }
  cout << "read " << num_pats << " patterns\n";
  patterns_loaded_flag = 1;
  file.close();
}


/*
 *  void weight_update()
 *
 *    Update the temporary collections of weight changes for the
 *  current pattern set.
 *
 */

void weight_update() 
{
                            /*  Weights.  */

  DW_I_H = DW_I_H + vec_mult_vec_trans(D_Hidden, A_Input);
  DW_H_O = DW_H_O + vec_mult_vec_trans(D_Output, A_Hidden);
#ifdef INPUT_TO_OUTPUT_WEIGHTS
  DW_I_O = DW_I_O + vec_mult_vec_trans(D_Output, A_Input);
#endif

#ifdef LAST_HIDDEN_FLAG
  DW_LH_H = DW_LH_H + vec_mult_vec_trans(D_Hidden, A_Last_Hidden);
#ifdef LAST_HIDDEN_TO_OUTPUT_WEIGHTS
  DW_LH_O = DW_LH_O + vec_mult_vec_trans(D_Output, A_Last_Hidden);
#endif
#endif

                            /*  Thresholds.  */
  DT_Hidden = DT_Hidden + D_Hidden;
  DT_Output = DT_Output + D_Output;
}


/*
 *  INIT_MODULE(randomize_weights)
 *
 *    Initialize the network weights and activations.
 *
 */

INIT_MODULE(randomize_weights)
{
#ifdef WRITE_INPUT_OUTPUT_FLAG
  /* Yuri: Added to write results to the file */
  /* open file */
  fout.open(WRITE_INPUT_OUTPUT_FILE, output);
#endif

  randomize(W_I_H);
  randomize(W_H_O);
  randomize(T_Hidden);
  randomize(T_Output);
#ifdef INPUT_TO_OUTPUT_WEIGHTS
  randomize(W_I_O);
#endif
#ifdef LAST_HIDDEN_FLAG
  randomize(W_LH_H);
#ifdef LH_H_WEIGHT_MASK
  W_LH_H = W_LH_H ^ MASK_LH_H;
#endif

#ifdef LAST_HIDDEN_TO_OUTPUT_WEIGHTS
  randomize(W_LH_O);
#endif
#endif
  DW_I_H = 0.0;
  DW_H_O = 0.0;
  DT_Hidden = 0.0;
  DT_Output = 0.0;
  OLD_DW_I_H = 0.0;
  OLD_DW_H_O = 0.0;
  OLD_DT_Hidden = 0.0;
  OLD_DT_Output = 0.0;
#ifdef INPUT_TO_OUTPUT_WEIGHTS
  DW_I_O = 0.0;
  OLD_DW_I_O = 0.0;
#endif
#ifdef LAST_HIDDEN_FLAG
  DW_LH_H = 0.0;
  OLD_DW_LH_H = 0.0;
  A_Last_Hidden = 0.0;
#ifdef LAST_HIDDEN_TO_OUTPUT_WEIGHTS
  DW_LH_O = 0.0;
  OLD_DW_LH_O = 0.0;
#endif
/*
#ifdef LH_H_WEIGHT_MASK
  MASK_LH_H = 1.0;
#endif
*/
#endif
}


/*
 *  void forward_pass()
 *
 *   Pass the activation forward.
 *
 */

void forward_pass()
{
  A_Hidden = sigmoid(mat_mult_col_vec(W_I_H, A_Input) + T_Hidden
#ifdef LAST_HIDDEN_FLAG
		     + mat_mult_col_vec(W_LH_H, A_Last_Hidden)
#endif
		     );
  A_Output =
#ifndef LINEAR_OUTPUTS
    sigmoid(
#endif
	    mat_mult_col_vec(W_H_O, A_Hidden) + T_Output
#ifdef INPUT_TO_OUTPUT_WEIGHTS
		     + mat_mult_col_vec(W_I_O, A_Input)
#endif
#ifdef LAST_HIDDEN_FLAG
#ifdef LAST_HIDDEN_TO_OUTPUT_WEIGHTS
		     + mat_mult_col_vec(W_LH_O, A_Last_Hidden)
#endif
#endif

#ifndef LINEAR_OUTPUTS
		     )
#endif
      ;
}


/*
 *  void propagate_errors()
 *
 *   Pass the error back through the network.
 *
 */

void propagate_errors()

{
  D_Output = E_Output
#ifndef LINEAR_OUTPUTS
    ^ ((A_Output ^ (1.0 - A_Output))
#ifdef GRADIENT_BIAS_FLAG
       + gradient_bias
#endif
       )
#endif
    ;
  D_Hidden = ((A_Hidden ^ (1.0 - A_Hidden))
#ifdef GRADIENT_BIAS_FLAG
			 + gradient_bias
#endif
			 )
    ^ mat_trans_mult_col_vec(W_H_O, D_Output);
}


/*
 *  update_actual_weights()
 *
 *   At this point, all the weight changes for the current set
 * of weights (over the entire pattern set) have been collected.
 * These changes are then applied to the actual network weights
 * and the temporary variables are cleared.
 *
 */

void update_actual_weights()
{
                        /*  Update weights and old values.  */

  OLD_DW_I_H = lrate * DW_I_H + momentum * OLD_DW_I_H;
  W_I_H = W_I_H + OLD_DW_I_H;
  OLD_DW_H_O = lrate * DW_H_O + momentum * OLD_DW_H_O;
  W_H_O = W_H_O + OLD_DW_H_O;
  OLD_DT_Hidden = lrate * DT_Hidden + momentum * OLD_DT_Hidden;
  T_Hidden = T_Hidden + OLD_DT_Hidden;
  OLD_DT_Output = lrate * DT_Output + momentum * OLD_DT_Output;
  T_Output = T_Output + OLD_DT_Output;

#ifdef INPUT_TO_OUTPUT_WEIGHTS
  OLD_DW_I_O = lrate * DW_I_O + momentum * OLD_DW_I_O;
  W_I_O = W_I_O + OLD_DW_I_O;
#endif

#ifdef LAST_HIDDEN_FLAG
  OLD_DW_LH_H = lrate * DW_LH_H + momentum * OLD_DW_LH_H;
  W_LH_H = (W_LH_H + OLD_DW_LH_H);
#ifdef LH_H_WEIGHT_MASK
  W_LH_H = W_LH_H ^ MASK_LH_H;
#endif
#ifdef LAST_HIDDEN_TO_OUTPUT_WEIGHTS
  OLD_DW_LH_O = lrate * DW_LH_O + momentum * OLD_DW_LH_O;
  W_LH_O = W_LH_O + OLD_DW_LH_O;
#endif
#endif
                        /*  Reset collection values.  */
  DW_I_H = 0.0;
  DW_H_O = 0.0;
  DT_Hidden = 0.0;
  DT_Output = 0.0;

#ifdef INPUT_TO_OUTPUT_WEIGHTS
  DW_I_O = 0.0;
#endif

#ifdef LAST_HIDDEN_FLAG
  DW_LH_H = 0.0;
#ifdef LAST_HIDDEN_TO_OUTPUT_WEIGHTS
  DW_LH_O = 0.0;
#endif
#endif
}


#ifdef LAST_HIDDEN_FLAG
void set_last_hidden()
{
  if(((int) LH_Restart.elem(current_pat)) != 0)
    {
      A_Last_Hidden = 0.0;
    }
  else
    {
      A_Last_Hidden = A_Hidden;
    }
}
#endif


nsl_data calc_error(nsl_vector& vec)
{

  int n;
  int i;
  nsl_data result;

  n = vec.get_xn();

  for(result = 0.0, i = 0; i < n; ++i)
    result = result + vec.elem(i) * vec.elem(i);
  return result;
}


#ifdef AUTO_PARAMETERS_FLAG
void process_parameters()
{
#ifdef DISPLAY_AUTO_PARAMETERS_DATA
  cout << "Mode = " << ap_mode nl;
  cout << "Count = " << ap_counter nl;
#endif

  dtss = tss - old_tss;

#ifdef DISPLAY_AUTO_PARAMETERS_DATA
  cout << "DT = " << dtss nl;
#endif

                         /*  Wait mode.  */
  if(ap_mode.elem() == AP_WAIT)
    {
                         /*  Negative change.  */
      if(dtss.elem() <= 0.0)
	{
                         /*  Too much negative change. */
	  if(dtss.elem() < - ap_max_derror.elem())
	    {
	      ap_counter = 0.0;
	    }
	  else           /*  Just right negative change.  */
	    {
	      ap_counter = ap_counter + 1.0;
	      if(ap_counter.elem() == ap_settle_count.elem())
		{
                         /*  Enough settling.  */

		  ap_mode = AP_RUN;
		  momentum = ap_max_momentum;
		  ap_counter = 0.0;
		  cout << "** Transition to RUN MODE." nl;
		}
	    }
	}
      else               /*  Positive change.  */
	{
	  ap_counter = ap_counter + 1.0;
	  if(ap_counter.elem() > ap_up_tolerance.elem())
	    {
                         /*  Still going up after too many epochs.  */
	      momentum = 0.0;
	      ap_counter = 0.0;
	      lrate = lrate / ap_lrate_factor;
	      cout << "** Drop LRATE." nl;
	    }
	}
    }
  else
    {
                         /*  Run mode.  */
      if(ap_mode.elem() == AP_RUN)
	{
	  if(dtss.elem() > 0.0)
	    {
	                 /*  Up - transition to wait_mode.  */
	      momentum = 0.0;
	      ap_mode = AP_WAIT;
	      ap_max_momentum = ap_max_momentum - ap_momentum_step;
	      if(ap_max_momentum.elem() < 0.0)
		ap_max_momentum = 0.0;
	      ap_counter = 0.0;
	      lrate = lrate / ap_lrate_factor;
	      cout << "** Transition to WAIT MODE." nl;
	    }
	  else
	    {
	                 /*  Down - check amount down.  */
	      if(dtss.elem() < - ap_max_derror.elem())
		{
		         /*  Moving too fast down.  */
		  ap_counter = 0.0;
		}
	      else
		{
		         /*  Check counter.  */

		  ap_counter = ap_counter + 1.0;
		  if(ap_counter.elem() == ap_patience_count.elem())
		    {
		          /*  Speed up descent.  */
		      ap_counter = 0.0;
		      ap_max_momentum = ap_max_momentum + ap_momentum_gain *
			(AP_DEFAULT_MAX_MOMENTUM - ap_max_momentum);
		      momentum = ap_max_momentum;
		      lrate = lrate * ap_lrate_growth_factor;
		      cout << "** Increase LRATE." nl;
		    }
		}
	    }
	}
    }
}

#endif


/*
 *  RUN_MODULE(bp_run)
 *
 *   Main loop : executed once for each pattern.  Obtains the
 * current pattern, passes the activation forward, computes the
 * network error and passes it back through the weights.  Updates
 * the temporary weight changs.  If all patterns have been processed,
 * the actual weights are updated (including lrate and momentum), and
 * the next pattern is set to start at the beginning.
 *
 */

RUN_MODULE(bp_run)
{
  int i;
  int dif;

#ifdef WRITE_INPUT_OUTPUT_FLAG
  int ii, imax;
#endif

  if(num_pats > 0)
    {
                /*  Copy needed pattern into Input.  */
#ifdef LAST_HIDDEN_FLAG
      if(pat_flag.elem() == 0.0 || LH_Restart.elem(current_pat) == 1.0)
	{

	  if(LH_Restart.elem(current_pat) == 1.0)
	    cout << "New starting position." nl;

	  extract_row_vector(P_Input, current_pat, A_Input);
	}
      else
        {
	     /*  Copy output to input.  */
	  for(dif = i = A_Input.get_xn() - A_Output.get_xn();
	      i < A_Input.get_xn(); ++i)
	    {
	      A_Input.elem(i) = A_Output.elem(i - dif);
	    }
	}
#else
      extract_row_vector(P_Input, current_pat, A_Input);
#endif
      extract_row_vector(P_Output, current_pat, Desired_Output);

#ifdef LAST_HIDDEN_FLAG
      set_last_hidden();
#endif
      
      forward_pass();

#ifdef DISPLAY_ACTIVATIONS_FLAG
      A_Input_R = NSLcol_vec_to_mat(A_Input_R, A_Input);
      A_Output_R = NSLcol_vec_to_mat(A_Output_R, A_Output);
      A_Hidden_R = NSLcol_vec_to_mat(A_Hidden_R, A_Hidden);
      Desired_Output_R = NSLcol_vec_to_mat(Desired_Output_R, Desired_Output);
#endif
#ifdef WRITE_INPUT_OUTPUT_FLAG
      /* Yuri: write results to the file */
      imax = A_Input.get_imax();
      for (ii = 0; ii < imax; ii++)
	  fout << A_Input.v[ii] << " ";
      imax = A_Output.get_imax();
      for (ii = 0; ii < imax; ii++)
	  fout << A_Output.v[ii] << " ";
      fout nl;
#endif
      
                /*  Compute the network error. */

      E_Output = Desired_Output - A_Output;
      
      tss_temp = tss_temp + calc_error(E_Output);

      propagate_errors();

      weight_update();

      if(++current_pat >= num_pats)
	{
                /*  Update control variables.  */

	  tss = tss_temp;
	  tss_temp = 0.0;
	  epoch = epoch + 1;
	  nsl_epoch = nsl_epoch + 1;
	  current_epoch = current_epoch + 1;
	  if(current_epoch == epoch_step)
	    {
                /*  Report temporary results.  */

	      current_epoch = 0;
	      cout << "Epoch " << epoch << "(" << nsl_epoch <<
		"), tss = " << tss nl;
	    }
	  current_pat = 0;

	  update_actual_weights();

#ifdef AUTO_PARAMETERS_FLAG
	  if(ap_auto_parameters.elem() == 1.0 && old_tss.elem() != -1.0)
	    process_parameters();
	  old_tss = tss;
#endif
	  if(log_flag)
	    {
	      log_file << tss << " " << lrate << " " << momentum;
#ifdef AUTO_PARAMETERS_FLAG

	      log_file << " " << dtss;
#endif
	      log_file nl;
	    }
	  
	}
    }
  else
    cout << "No patterns to process." nl;
}

