/****************************************************************/
/*                                                           	*/
/*                       cue_interaction.c                      */
/*                                                            	*/
/****************************************************************/

# include "nsl_include.h"

extern void view_to_right_retina(nsl_vector&,nsl_matrix&,
	nsl_data&,nsl_data&,nsl_data&);
extern void view_to_left_retina(nsl_vector&,nsl_matrix&,
	nsl_data&,nsl_data&,nsl_data&);
extern void retina_to_accommodation(nsl_matrix&,nsl_matrix&,nsl_vector&,
	nsl_data&,nsl_data&,nsl_data&,nsl_data&,nsl_data&);
extern void retina_to_disparity(nsl_matrix&,nsl_vector&,nsl_vector&);

NETWORK(CUE_INTERACTION);

MATRIX(IN,81,81);	// network input 
VECTOR(RL,161); 	// left retina
VECTOR(RR,161); 	// right retina

MATRIX(A,81,81); 	// accommodation layer
MATRIX(D,81,81); 	// disparity layer

MATRIX(AOUT,81,81); 	// accommodation output layer
MATRIX(DOUT,81,81); 	// disparity output layer
MATRIX(OUT,81,81); 	// output layer

MATRIX(m,81,81); 	// membrane potential layer
MATRIX(s,81,81); 	// membrane potential layer
MATRIX(M,81,81); 	// firing rate layer
MATRIX(S,81,81); 	// firing rate layer

VECTOR(u,81); 		// membrane potential layer
VECTOR(U,81); 		// firing rate layer
VECTOR(v,81); 		// membrane potential layer
VECTOR(V,81); 		// firing rate layer

DATA(u_tm);
DATA(v_tm);
DATA(s_tm);
DATA(m_tm);

DATA(w); 	// = 3.0;	1/2 of interpupilary distance (cm)
DATA(yf);	// = -10.0;	intersection of optical axes (0,yf) (cm)
DATA(l);	// = 22.0;	distance of interpupilary line from origin (cm)

DATA(dmax);	// = 0.25;	// maximum disparity
DATA(sigma);	// = 0.25;	// spread parameter

VECTOR(Ws,3);
VECTOR(Wm,3);

DATA(Kmu);
DATA(Ksv);
DATA(Kds);
DATA(Kvs);
DATA(Kms);
DATA(Kam);
DATA(Kum);
DATA(Ksm);

DATA(x1);
DATA(x2);

INIT_MODULE(cue_init)
{
	m = 0;
	s = 0;
	u = 0;
	v = 0;
	M = 0;
	S = 0;
	U = 0;
	V = 0;
	RR = 0;
	RL = 0;
	A = 0;
	D = 0;
}
INIT_MODULE(cue_RR)
{
	view_to_right_retina(RR,IN,w,yf,l);
}
INIT_MODULE(cue_RL)
{
	view_to_left_retina(RL,IN,w,yf,l);
}
INIT_MODULE(cue_A)
{
	retina_to_accommodation(A,IN,RR,w,yf,l,dmax,sigma);
}
INIT_MODULE(cue_D)
{
 	retina_to_disparity(D,RR,RL);
}

RUN_MODULE(cue_M)
{
	DIFF.eq(m,m_tm) = -m + Kam*A - Kum*NSLrow_vec_to_mat(U,81)+Ksm*S+Wm*M;
	M = NSLsigma(m,x1,x2);
}
RUN_MODULE(cue_S)
{
	DIFF.eq(s,s_tm) = -s + Kds*D - Kvs*NSLrow_vec_to_mat(V,81)+Kms*M + Ws*S;
	S = NSLsigma(s,x1,x2);
}
RUN_MODULE(cue_U)
{
	DIFF.eq(u,u_tm) = -u + Kmu*NSLmat_to_row_vec(M);
	U = NSLramp(u);
}
RUN_MODULE(cue_V)
{
	DIFF.eq(v,v_tm) = -v + Ksv*NSLmat_to_row_vec(S);
	V = NSLramp(v);
}


/*
END_MODULE(cue_AOUT)
{
	k = 0.5;	// threshold
	AOUT = retina_to_view(A,yf,w,l,k);
}
END_MODULE(cue_DOUT)
{
	k = 0.5;	// threshold
	DOUT = retina_to_view(D,yf,w,l,k);
}
END_MODULE(cue_OUT)
{
	k = 0.5;
	OUT = retina_to_view(M,k);
}
*/
