// 
//		NSL RETINA MODEL
//

# include "nsl_include.h"

NETWORK(RETINA);

INPUT_MAT(r,13,13);

MATRIX(axh,13,13);
MATRIX(ath1,13,13);
MATRIX(ath2,13,13);
MATRIX(ath,13,13);
MATRIX(axd,13,13);
MATRIX(atd1,13,13);
MATRIX(atd2,13,13);
MATRIX(atd,13,13);

MATRIX(r2,13,13);
MATRIX(R2,13,13);
MATRIX(R2f,13,13);
MATRIX(r3,13,13);
MATRIX(R3,13,13);
MATRIX(R3f,13,13);
MATRIX(r4,13,13);
MATRIX(R4,13,13);
MATRIX(R4f,13,13);

// parameters

DATA(axh_tm);
DATA(axd_tm);

// masks

MATRIX(r2_erf,13,13);
MATRIX(r2_irf,13,13);
MATRIX(r3_erf,13,13);
MATRIX(r3_irf,13,13);
MATRIX(r4_erf,13,13);

DATA(r2_erf_dia);
DATA(r2_irf_dia);
DATA(r2_erf_sig);
DATA(r2_irf_sig);
DATA(r2_erf_wgt);
DATA(r2_irf_wgt);

DATA(r3_erf_dia);
DATA(r3_irf_dia);
DATA(r3_erf_sig);
DATA(r3_irf_sig);
DATA(r3_erf_wgt);
DATA(r3_irf_wgt);

DATA(r4_erf_dia);
DATA(r4_erf_sig);
DATA(r4_erf_wgt);

DATA(r2_k);
DATA(r3_k);
DATA(r4_k);

MATRIX(zero,13,13);

static float dt;

INIT_MODULE(retina_init)
{
	dt = RETINA.get_delta();

	r = 0;
	axh = 0;
	ath1 = 0;
	ath2 = 0;
	ath = 0;
	axd = 0;
	atd1 = 0;
	atd2 = 0;
	atd = 0;
	r2 = 0;
	R2 = 0;
	R2f = 0;
	r3 = 0;
	R3 = 0;
	R3f = 0;
	r4 = 0;
	R4 = 0;
	R4f = 0;

	r2_erf = KERNEL(13,13,r2_erf_dia,r2_erf_sig,r2_erf_wgt);
	r2_irf = KERNEL(13,13,r2_irf_dia,r2_irf_sig,r2_irf_wgt);
	r3_erf = KERNEL(13,13,r3_erf_dia,r3_erf_sig,r3_erf_wgt);
	r3_irf = KERNEL(13,13,r3_irf_dia,r3_irf_sig,r3_irf_wgt);
	r4_erf = KERNEL(13,13,r4_erf_dia,r4_erf_sig,r4_erf_wgt);
}

MODULE(retina_r)
{
	r.run();
}
RUN_MODULE(retina_axh)
{
	DIFF.eq(axh,axh_tm) = -axh + r;
}
RUN_MODULE(retina_axd)
{
	DIFF.eq(axd,axd_tm) = -axd - r;
}
RUN_MODULE(retina_ath)
{
	ath1 = 5*(r - axh);
	ath2 = exp(-dt/0.3)*ath;

	ath = NSLstep(ath1,zero,ath1,ath2);
}
RUN_MODULE(retina_atd)
{
	atd1 = -5*(r + axd);
	atd2 = exp(-dt/0.3)*atd;

	atd = NSLstep(atd1,zero,atd1,atd2);
}

RUN_MODULE(retina_R2)
{
	r2 = ath*(r2_erf - r2_irf); // change convolution output size

	R2 = NSLramp(r2);

	R2f = r2_k*R2;
}
RUN_MODULE(retina_R3)
{
	r3 = ath*(r3_erf - r3_irf); // change convolution output size

	R3 = NSLramp(r3);

	R3f = r3_k*R3;

}
RUN_MODULE(retina_R4)
{
	r4 = r4_erf*(ath - atd); // change convolution output size

	R4 = NSLramp(r4);

	R4f = r4_k*R4/(R4+0.2);
}
