//
//	display_frame.c
//

# include "nsl.h"
# include "nsl_headers.h"
# include "nsl_winds.h"

extern "C" int fork();

extern Frame create_frame(Frame,int,int,int,int,int);
extern void set_frame_label(Frame,char*);
extern void set_frame_icon(Frame,char*);
extern void show_frame(Frame);

extern int destroy_frame(Frame);
extern int resize_frame(Frame,int,int,int,int);
extern int get_frame_width(Frame);
extern int get_frame_height(Frame);
extern int set_frame_defaults(Frame);

// extern struct singlecolor FRAME_FCOLOR, FRAME_BCOLOR;
// extern int FRAMECOLORFG;
extern int ARGC;
extern char** ARGV;

extern generic_window* get_window_obj(gen_window_list*,int);
extern generic_window* get_window_obj(gen_window_list*,char*);

// display_frame

display_frame::display_frame(display_frame* fr,char* str) : (str,0,0,0,0,0)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_frame::display_frame(" << str << ");\n";
	    cout.flush();
	}

	if (fr != NULL)
		parent_frame = fr->get_frame();	   // sunview parent frame
	else
		parent_frame = 0;
	frame = 0;			   // NULL sunview frame

	window_total = 0;
	window_list = 0; 

	init_fg = 0;

	done_fg = 0;
//	resize_fg = 1; 
}

display_frame::~display_frame()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_frame::~display_frame()");

	if (window_list != NULL)
	{
	    gen_window_iterator next(*window_list);
	    generic_window* win;
	    while (win = next()) 
	    {
//		cmd_out("DELETE WIN: ",win->get_index());
		delete (display_window*) win;
	    }
	}

	window_total = 0;
	window_list = 0; 

	done_fg = 1; // dead display frame

	destroy_frame(frame);
}

int display_frame::add_window(display_window* win)
{ 
	if (DEBUG_FG == 1)
	{
	    cout << "// display_frame::add_window(";
	    cout << win->get_name() << ");\n";
	    cout.flush();
	}

	if (win == NULL || window_list == NULL)
	{
		cmd_error("*** display_frame::add_window ***");
		cmd_error("Unable to add window to frame list");
		return 0;
	}

	++window_total;

	win->set_index(window_total);
	window_list->append(win); 

	return window_total;
}

void display_frame::clear()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_frame::clear()");

	if (window_list != NULL)
	{
	    gen_window_iterator next(*window_list);
	    generic_window* win;
	    while (win = next()) 
		((display_window*) win)->clear();
	}
}
void display_frame::resize(int x,int y,int w,int h)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_frame::resize(";
	    cout << x << "," << y << "," << w << "," << h << ");\n";
	    cout.flush();
	}

	X0 = x;
	Y0 = y;
	width = w;
	height = h;

	resize_frame(frame,x,y,w,h);
}
void display_frame::resize()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_frame::resize()");

	display_canvas* can;
	int x,y;
	int i;


// make sure there is no division by 0
	if (window_rows <= 0)
		window_rows = window_total;
	if (window_cols <= 0)
		window_cols = 1;

	width = get_frame_width(frame);
	height = get_frame_height(frame);

	int hoff = (height)%window_rows;
	int h = (height - hoff)/window_rows;
	int woff = width%window_cols;
	int w = (width - woff)/window_cols;

	if (window_list != NULL)
	{
	    gen_window_iterator next(*window_list);
	    generic_window* win;
	    while (win = next()) 
	    {
		i = win->get_index();

		x = (w)*((i-1)%window_cols);
		y = (h)*((i-1)/window_cols);

		if ((can = 
		   ((display_window*) win)->get_window_canvas()) != NULL)
		{
			((display_window*) win)->resize(x,y,w,h);
// set_win_sector() = set_win_sector(x,y,w,h) + layer size auto updates
			((display_window*) win)->set_win_sector();
			can->update();
		}
	    }
	}
}
void display_frame::create(int x,int y,int w,int h)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_frame::create(";
	    cout << x << "," << y << "," << w << "," << h << ");\n";
	    cout.flush();
	}

	X0 = x;
	Y0 = y;
	width = w;
	height = h;

	if ((frame = create_frame(parent_frame,x,y,w,h,index)) == 0) 
	{
		cmd_error("*** display_frame::create ***");
		cmd_error("Cannot create Display Frame");
	}
	else
	{
//		reset_frame_label(frame);
		done_fg = 0;	// alive
		set_frame_label(frame,name);
		set_frame_icon(frame,name);
		set_frame_defaults(frame);
//		show_frame(frame);
	}
}
void display_frame::create()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_frame::create()");

	create(X0,Y0,width,height);
}
void display_frame::create(int argc,char** argv)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_frame::create(argc,argv)\n";
	    cout.flush();
	}

	create();
}
void display_frame::reset()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_frame::reset()");

	window_total = 0;		   // initialize total number of windows
	window_list = new gen_window_list; // initialize frame's window list
}
void display_frame::init()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_frame::init()");
}

void display_frame::update()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_frame::update()");

	if (window_list != NULL)
	{
	    gen_window_iterator next(*window_list);
	    generic_window* win;
	    while (win = next()) 
		((display_window*) win)->update();
	}
}
void display_frame::update_from_panel()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_frame::update_from_panel()");

	if (window_list != NULL)
	{
	    gen_window_iterator next(*window_list);
	    generic_window* win;
	    while (win = next()) 
		((display_window*) win)->update_from_panel();
	}
}
void display_frame::display()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_frame::display()");

	if (window_list != NULL)
	{
	    gen_window_iterator next(*window_list);
	    generic_window* win;
	    while (win = next()) 
		win->display();
	}
}
void display_frame::print_status(ostream& out)
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_frame::print_status(out)");

	out << "set frame_name " << name << "\n";
//	out << "set frame_index " << index << "\n";
	out << "set frame_X0 " << X0 << "\n";
	out << "set frame_Y0 " << Y0 << "\n";
	out << "set frame_width " << width << "\n";
	out << "set frame_height " << height << "\n";
	if (resize_fg == 1)
	   out << "set frame_auto_resize ON\n";
	else
	   out << "set frame_auto_resize OFF\n";
	out << "set frame_rows " << window_rows << "\n";
	out << "set frame_cols " << window_cols << "\n";
	out << "// total windows: " << window_total << "\n";
	out << "update display_frame\n";
	out << "\n";
	out.flush();
}
void display_frame::print_all_window_status(ostream& out)
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_frame::print_all_window_status(out)");

	if (window_list != NULL)
	{
	    gen_window_iterator next(*window_list);
	    generic_window* win;
	    while (win = next()) 
		((display_window*) win)->print_status(out);
	}
}
void display_frame::print_window_status(ostream& out,int index)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_frame::print_window_status(out,";
	    cout << index << ");\n";
	    cout.flush();
	}

	display_window* win;

	if ((win = (display_window*) get_window_obj(window_list,index)) != NULL)
		win->print_status(out);
}
void display_frame::print_window_status(ostream& out,char* str)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_frame::print_window_status(out,";
	    cout << str << ");\n";
	    cout.flush();
	}

	display_window* win;

	if ((win = (display_window*) get_window_obj(window_list,str)) != NULL)
		win->print_status(out);
}

display_window* display_frame::get_display_window(char* str)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_frame::get_display_window(";
	    cout << str << ");\n";
	    cout.flush();
	}

	display_window* win;

	if ((win = (display_window*) get_window_obj(window_list,str)) != NULL)
		return (display_window*) win;

	return (display_window*) 0;
}
display_window* display_frame::get_display_window(int index)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// display_frame::get_display_window(";
	    cout << index << ");\n";
	    cout.flush();
	}

	display_window* win;

	if ((win = (display_window*) get_window_obj(window_list,index)) != NULL)
		return (display_window*) win;

	return (display_window*) 0;
}
