//
//	display_window.c
//

# define ENDL "\n"

# include "nsl.h"
# include "nsl_headers.h"
# include "nsl_winds.h"
# include "nsl_system.h"

extern nsl_model* NSL_MODEL; // global model pointer
extern window_interface* NSL_INTERFACE;
extern nsl_system* NSL_SYSTEM;

nsl_data* get_data_obj(data_type_list*,char*);
nsl_data* get_data_obj(data_type_list*,int);

extern nsl_interpreter NSL;

void (*INIT_USER_DISPLAY_WINDOW)(display_window*); // extern graphics

/*

win	read_from_database	write_to_database	display 
----------------------------------------------------------------------
panel	data=>panel		panel=>data		panel display
canvas	data=>canvas					canvas display

*/

// display_window

display_window::display_window(display_frame* fr,char* str) : (str,0,0,0,0,0)
{
	if (DEBUG_FG == 1)
	{
		cout << "// display_window::display_window(" << str << ");\n";
		cout.flush();
	}

	parent_frame = fr;
	fr->add_window(this);

	data_list = 0;
	data_total = 0;

	reset();
	reset_main();

// canvas window
	if ((window_canvas = new display_canvas(parent_frame,this,str)) == NULL)
	{
		cmd_error("*** display_window::create ***");
		cmd_error("Unable to create window canvas");
	}
	else
	{
		window_canvas->set_index(index);
		window_canvas->reset(); // reset_data
	}

	if ((control_panel = new display_panel(parent_frame,this,str)) == NULL)
	{
		cmd_error("*** display_window::create ***");
		cmd_error("Unable to create data panel");
	}
	else
	{
		control_panel->set_index(index);
		control_panel->reset();
		control_panel->reset_main();
	}
}
display_window::~display_window()
{
	if (DEBUG_FG == 1)
		cmd_out("// display_window::~display_window()");

	delete window_canvas;
	delete control_panel;

	data_list = 0;
	data_total = 0;
	main_total = 0;
}

void display_window::resize(int x,int y,int w,int h)
{
	if (DEBUG_FG == 1)
	{
		cout << "// display_window::resize(" << x << ",";
		cout << y << "," << w << "," << h << ");\n";
		cout.flush();
	}

	X0 = x;
	Y0 = y;
	width = w;
	height = h;

	if (window_canvas != NULL) 
	        window_canvas->resize(x,y,w,h);
	else
	{
		cmd_error("*** display_window::resize ***");
		cmd_error("Null window canvas");
	}
}

int panel_X0 = 0;
int panel_Y0 = 120;

void display_window::create(int x,int y,int w,int h)
{
	if (DEBUG_FG == 1)
	{
		cout << "// display_window::create(" << x << ",";
		cout << y << "," << w << "," << h << ");\n";
		cout.flush();
	}

	X0 = x; Y0 = y; width = w; height = h;

	init_main();

	if (window_canvas != NULL)
	{
		window_canvas->create(X0,Y0,width,height);
	}

// control panel
	static int p_width = 240;
	static int p_height = 250; // 150
	static int p_X0 = 0;
	static int p_Y0 = 120;

	if (panel_Y0 >= 700)
	{
		panel_Y0 = p_Y0 + 20;
		panel_X0 = panel_X0 + 20;
	}
	if (panel_Y0 >= 700 && panel_X0 >= 800)
	{
		panel_Y0 = p_Y0;
		panel_X0 = p_X0;
	}

	if  (control_panel != NULL)
	{
		control_panel->create(panel_X0,panel_Y0,p_width-10,p_height);
		control_panel->init_main();
	}
	panel_Y0 = panel_Y0 + p_height;
}
void display_window::create()
{
	if (DEBUG_FG == 1)
		cmd_out("// display_window::create()");

	create(X0,Y0,width,height);
}
void display_window::reset()
{
	if (DEBUG_FG == 1)
		cmd_out("// display_window::reset()");

	data_total = 0;
	data_list = new data_type_list;
}
void display_window::reset_main()
{
	if (DEBUG_FG == 1)
		cmd_out("// display_window::reset_main()");

	main_total = 0;
}
void display_window::reset_data()
{
	if (DEBUG_FG == 1)
		cmd_out("// display_window::reset_data()");

	reset_data_list();
}
void display_window::reset_data_list()
{
	if (DEBUG_FG == 1)
	    cmd_out("// display_window::reset_data_list()");

	if (data_list != NULL)
	{
	    // iterate main_item_elemtns & delete the rest
	    data_list->clear(main_total,data_total-main_total);

	    data_total = main_total; // partial reset
	}
	else
	{
	    cmd_error("*** display_panel::reset_data_list ***");
	    cmd_error("NULL list");
	}
}
void display_window::init()
{
	if (DEBUG_FG == 1)
		cmd_out("// display_window::init()");
}
void display_window::init_main()
{
	if (DEBUG_FG == 1)
		cmd_out("// display_window::init_main()");

	init_general_data("name",3,1);

	init_general_data("GRAPH",3,1);
	init_general_data("LAYER",3,1);
}
void display_window::init_data()
{
	if (DEBUG_FG == 1)
		cmd_out("// display_window::init_data()");

	nsl_data* win_data;

	static float wymin,wymax,ts; // general

	static int i0,i1,j0,j1,imax,jmax,vec_type;

	static char *pulsefg,*hiddenfg,*window_overlay;
	static int elem_i,elem_j,window_step;
	static float t0,t1,val_in,val_out;
	static float sx,sy,sz,ax,by,cz; // spatial 3
	
	if (data_total > main_total && data_list != NULL)
	{
	// main data
	    if ((win_data = get_data_obj(data_list,"wymin")) != NULL)
		wymin = win_data->get_float_value();
	    if ((win_data = get_data_obj(data_list,"wymax")) != NULL)
		wymax = win_data->get_float_value();
	    if ((win_data = get_data_obj(data_list,"ts")) != NULL)
		ts = win_data->get_float_value();
	// layer data
	    if ((win_data = get_data_obj(data_list,"vec_type")) != NULL)
		vec_type = win_data->get_int_value();
	// graph data
	    if ((win_data = get_data_obj(data_list,"val_in")) != NULL)
		val_in = win_data->get_float_value();
	    if ((win_data = get_data_obj(data_list,"t0")) != NULL)
		t0 = win_data->get_float_value();
	    if ((win_data = get_data_obj(data_list,"t1")) != NULL)
		t1 = win_data->get_float_value();
	    if ((win_data = get_data_obj(data_list,"sx")) != NULL)
		sx = win_data->get_float_value();
	    if ((win_data = get_data_obj(data_list,"sy")) != NULL)
		sy = win_data->get_float_value();
	    if ((win_data = get_data_obj(data_list,"sz")) != NULL)
		sz = win_data->get_float_value();
	    if ((win_data = get_data_obj(data_list,"ax")) != NULL)
		ax = win_data->get_float_value();
	    if ((win_data = get_data_obj(data_list,"by")) != NULL)
		by = win_data->get_float_value();
	    if ((win_data = get_data_obj(data_list,"cz")) != NULL)
		cz = win_data->get_float_value();
	    if ((win_data = get_data_obj(data_list,"pulsefg")) != NULL)
		pulsefg = win_data->get_str_value();
	    if ((win_data = get_data_obj(data_list,"hiddenfg")) != NULL)
		hiddenfg = win_data->get_str_value();
	    if ((win_data = get_data_obj(data_list,"window_step")) != NULL)
		window_step = win_data->get_int_value();
            if ((win_data = get_data_obj(data_list,"window_overlay")) != NULL)
	        window_overlay = win_data->get_str_value();
        }
	else
	{
	// main data
		wymin = 0.0;
		wymax = 1.0;
		ts = 0.0;
	// layer data
		vec_type = 0;
		if (layer->get_layer_type() == 2) // only vector
		    vec_type = ((nsl_vector*) layer)->get_vec_type();
	// graph data
		t0 = 0.0;
		t1 = 1.0;
		val_in = 1.0;
		val_out = 0.0;
		elem_i = 0;
		elem_j = 0;
		sx = 1.0;
		sy = 1.0;
		sz = 1.0;
		ax = 1.0;
		by = 1.0;
		cz = 1.0;
		pulsefg = "OFF";
		hiddenfg = "ON";
		window_step = 1;
                window_overlay = "OFF";
        }
	
	reset_data(); // init new data list

	// init main data
	init_general_data("window_step",1);
	init_general_data("window_overlay",3);
	
	init_general_data("wymin",2);
	init_general_data("wymax",2);
	init_general_data("ts",2);

	init_general_data("wx0",1);
	init_general_data("wy0",1);
	init_general_data("wx1",1);
	init_general_data("wy1",1);
	init_general_data("wxs",1);
	init_general_data("wys",1);

	switch (layer_type)
	{
	case 1: // nsl_data
		break;
	case 2: // nsl_vector
		init_general_data("i0",1);
		init_general_data("i1",1);
		init_general_data("imax",1);
//		init_general_data("vec_type",3);
		init_general_data("vec_type",1);
		break;
	case 3: // nsl_matrix
		init_general_data("i0",1);
		init_general_data("i1",1);
		init_general_data("j0",1);
		init_general_data("j1",1);
		init_general_data("imax",1);
		init_general_data("jmax",1);
		break;
	default:
		cmd_error("*** display_window::init_data ***");
		cmd_error_exit("Unknown NSL_LAYER type: ",layer_type);
	}

	switch (graph_type)
	{
	case 1: // area_level_graph
		init_general_data("val_in",2);
		init_general_data("val_out",2);
		if (layer_type == 2 || layer_type == 3)
			init_general_data("elem_i",1);
		if (layer_type == 3)
			init_general_data("elem_j",1);
		break;
	case 2: // spatial_graph_2
		init_general_data("pulsefg",3);
		break;
	case 3: // spatial_graph_3
		init_general_data("hiddenfg",3);
		init_general_data("sx",2);
		init_general_data("sy",2);
		init_general_data("sz",2);
		init_general_data("ax",2);
		init_general_data("by",2);
		init_general_data("cz",2);
		break;
	case 4: // temporal_graph
		init_general_data("t0",2);
		init_general_data("t1",2);
		init_general_data("pulsefg",3);
		break;
	case 5: // user_graph
		if (INIT_USER_DISPLAY_WINDOW != NULL)
			INIT_USER_DISPLAY_WINDOW(this);
		else
		{
			cmd_error("*** display_window::init_graph_data ***");
			cmd_error("Unknow INIT_USER_DISPLAY_WINDOW function");
		}
		break;
	default:
		cmd_error("*** display_window::init_graph_data ***");
		cmd_error_exit("Unknown GRAPH type: ",graph_type);
	}

	// set values

	set_general_data("window_step",window_step);
	set_general_data("window_overlay",window_overlay);

	set_general_data("wymin",wymin);
	set_general_data("wymax",wymax);
	set_general_data("ts",ts);

	switch (layer_type)
	{
	case 1: // nsl_data
		set_win_sector(0,0,0,0,1,1); // update window sector values
		break;
	case 2: // nsl_vector
		i0 = ((nsl_vector*) layer)->get_i0();
		i1 = ((nsl_vector*) layer)->get_i1();
		imax = ((nsl_vector*) layer)->get_imax();
		set_general_data("i0",i0);
		set_general_data("i1",i1);
		set_general_data("imax",imax);
		set_general_data("vec_type",vec_type);
		if (vec_type == 0)  // set window sector
		{
//			set_general_data("vec_type","horizontal");
			set_win_sector(i0,0,i1,0,imax,1);
		}
		else
		{
//			set_general_data("vec_type","vertical");
			set_win_sector(0,i0,0,i1,1,imax);
		}
		break;
	case 3: // nsl_matrix
		i0 = ((nsl_matrix*) layer)->get_i0();
		i1 = ((nsl_matrix*) layer)->get_i1();
		j0 = ((nsl_matrix*) layer)->get_j0();
		j1 = ((nsl_matrix*) layer)->get_j1();
		imax = ((nsl_matrix*) layer)->get_imax();
		jmax = ((nsl_matrix*) layer)->get_jmax();
		set_general_data("i0",i0);
		set_general_data("i1",i1);
		set_general_data("j0",j0);
		set_general_data("j1",j1);
		set_general_data("imax",imax);
		set_general_data("jmax",jmax);
		set_win_sector(i0,j0,i1,j1,imax,jmax); // set window sector
		break;
	default:
		cmd_error("*** display_window::init_data ***");
		cmd_error_exit("Unknown NSL_LAYER type: ",layer_type);
	}

	switch (graph_type)
	{
	case 1: // area_level_graph
		set_general_data("val_in",val_in);
		set_general_data("val_out",val_out);
		if (layer_type == 2 || layer_type == 3)
			set_general_data("elem_i",elem_i);
		if (layer_type == 3)
			set_general_data("elem_j",elem_j);
		break;
	case 2: // spatial_graph_2
		set_general_data("pulsefg",pulsefg);
		break;
	case 3: // spatial_graph_3
		set_general_data("hiddenfg",hiddenfg);
		set_general_data("sx",sx);
		set_general_data("sy",sy);
		set_general_data("sz",sz);
		set_general_data("ax",ax);
		set_general_data("by",by);
		set_general_data("cz",cz);
		break;
	case 4: // temporal_graph
		set_general_data("t1",t0);
		set_general_data("t1",t1);
		set_general_data("pulsefg",pulsefg);
		break;
	case 5: // user_graph
		if (INIT_USER_DISPLAY_WINDOW != NULL)
;//			SET_USER_DISPLAY_WINDOW(this);
		else
		{
			cmd_error("*** display_window::set_graph_data ***");
			cmd_error("Unknow SET_USER_DISPLAY_WINDOW function");
		}
		break;
	default:
		cmd_error("*** display_window::set_graph_data ***");
		cmd_error_exit("Unknown GRAPH type: ",graph_type);
	}
}
void display_window::set_data()
{
	if (DEBUG_FG == 1)
	{
		cout << "// display_window::set_data();\n";
		cout.flush();
	}

	nsl_data* win_data;

	static float wymin,wymax,ts; // general

	static int i0,i1,j0,j1,imax,jmax,vec_type,window_step;

	static char *pulsefg,*hiddenfg,*window_overlay;
	static int elem_i,elem_j;
	static float t0,t1,val_in,val_out;
	static float sx,sy,sz,ax,by,cz; // spatial 3
	
	if (data_total > main_total && data_list != NULL)
	{
	// main data
	    if ((win_data = get_data_obj(data_list,"wymin")) != NULL)
		wymin = win_data->get_float_value();
	    if ((win_data = get_data_obj(data_list,"wymax")) != NULL)
		wymax = win_data->get_float_value();
	    if ((win_data = get_data_obj(data_list,"ts")) != NULL)
		ts = win_data->get_float_value();
	// layer data
	    if ((win_data = get_data_obj(data_list,"vec_type")) != NULL)
		vec_type = win_data->get_int_value();
	// graph data
	    if ((win_data = get_data_obj(data_list,"val_in")) != NULL)
		val_in = win_data->get_float_value();
	    if ((win_data = get_data_obj(data_list,"t0")) != NULL)
		t0 = win_data->get_float_value();
	    if ((win_data = get_data_obj(data_list,"t1")) != NULL)
		t1 = win_data->get_float_value();
	    if ((win_data = get_data_obj(data_list,"sx")) != NULL)
		sx = win_data->get_float_value();
	    if ((win_data = get_data_obj(data_list,"sy")) != NULL)
		sy = win_data->get_float_value();
	    if ((win_data = get_data_obj(data_list,"sz")) != NULL)
		sz = win_data->get_float_value();
	    if ((win_data = get_data_obj(data_list,"ax")) != NULL)
		ax = win_data->get_float_value();
	    if ((win_data = get_data_obj(data_list,"by")) != NULL)
		by = win_data->get_float_value();
	    if ((win_data = get_data_obj(data_list,"cz")) != NULL)
		cz = win_data->get_float_value();
	    if ((win_data = get_data_obj(data_list,"pulsefg")) != NULL)
		pulsefg = win_data->get_str_value();
	    if ((win_data = get_data_obj(data_list,"hiddenfg")) != NULL)
		hiddenfg = win_data->get_str_value();
	    if ((win_data = get_data_obj(data_list,"window_step")) != NULL)
		window_step = win_data->get_int_value();
            if ((win_data = get_data_obj(data_list,"window_overlay")) != NULL)
	        window_overlay = win_data->get_str_value();
        }
	else
	{
	// main data
		wymin = 0.0;
		wymax = 1.0;
		ts = 0.0;
	// layer data
		vec_type = 0;
		if (layer->get_layer_type() == 2) // only vector
		    vec_type = ((nsl_vector*) layer)->get_vec_type();
	// graph data
		t0 = 0.0;
		t1 = 1.0;
		val_in = 1.0;
		val_out = 0.0;
		elem_i = 0;
		elem_j = 0;
		sx = 1.0;
		sy = 1.0;
		sz = 1.0;
		ax = 1.0;
		by = 1.0;
		cz = 1.0;
		pulsefg = "OFF";
		hiddenfg = "ON";
		window_step = 1;
                window_overlay = "OFF";
        }

	set_general_data("window_step",window_step);
	set_general_data("window_overlay",window_overlay);
	
	set_general_data("wymin",wymin);
	set_general_data("wymax",wymax);
	set_general_data("ts",ts);

	switch (layer_type)
	{
	case 1: // nsl_data
		set_win_sector(0,0,0,0,1,1); // update window sector values
		break;
	case 2: // nsl_vector
		i0 = ((nsl_vector*) layer)->get_i0();
		i1 = ((nsl_vector*) layer)->get_i1();
		imax = ((nsl_vector*) layer)->get_imax();
		set_general_data("i0",i0);
		set_general_data("i1",i1);
		set_general_data("imax",imax);
		set_general_data("vec_type",vec_type);
		if (vec_type == 0)  // set window sector
		{
//			set_general_data("vec_type","horizontal");
			set_win_sector(i0,0,i1,0,imax,1);
		}
		else
		{
//			set_general_data("vec_type","vertical");
			set_win_sector(0,i0,0,i1,1,imax);
		}
		break;
	case 3: // nsl_matrix
		i0 = ((nsl_matrix*) layer)->get_i0();
		i1 = ((nsl_matrix*) layer)->get_i1();
		j0 = ((nsl_matrix*) layer)->get_j0();
		j1 = ((nsl_matrix*) layer)->get_j1();
		imax = ((nsl_matrix*) layer)->get_imax();
		jmax = ((nsl_matrix*) layer)->get_jmax();
		set_general_data("i0",i0);
		set_general_data("i1",i1);
		set_general_data("j0",j0);
		set_general_data("j1",j1);
		set_general_data("imax",imax);
		set_general_data("jmax",jmax);
		set_win_sector(i0,j0,i1,j1,imax,jmax); // set window sector
		break;
	default:
		cmd_error("*** display_window::init_data ***");
		cmd_error_exit("Unknown NSL_LAYER type: ",layer_type);
	}

	switch (graph_type)
	{
	case 1: // area_level_graph
		set_general_data("val_in",val_in);
		set_general_data("val_out",val_out);
		if (layer_type == 2 || layer_type == 3)
			set_general_data("elem_i",elem_i);
		if (layer_type == 3)
			set_general_data("elem_j",elem_j);
		break;
	case 2: // spatial_graph_2
		set_general_data("pulsefg",pulsefg);
		break;
	case 3: // spatial_graph_3
		set_general_data("hiddenfg",hiddenfg);
		set_general_data("sx",sx);
		set_general_data("sy",sy);
		set_general_data("sz",sz);
		set_general_data("ax",ax);
		set_general_data("by",by);
		set_general_data("cz",cz);
		break;
	case 4: // temporal_graph
		set_general_data("t1",t0);
		set_general_data("t1",t1);
		set_general_data("pulsefg",pulsefg);
		break;
	case 5: // user_graph
		if (INIT_USER_DISPLAY_WINDOW != NULL)
;//			SET_USER_DISPLAY_WINDOW(this);
		else
		{
			cmd_error("*** display_window::set_graph_data ***");
			cmd_error("Unknow SET_USER_DISPLAY_WINDOW function");
		}
		break;
	default:
		cmd_error("*** display_window::set_graph_data ***");
		cmd_error_exit("Unknown GRAPH type: ",graph_type);
	}
}
void display_window::clear()
{
	if (DEBUG_FG == 1)
		cmd_out("// display_window::clear()");

	char* window_overlay = "OFF";
	nsl_data* win_data;

	if ((win_data = get_data_obj(data_list,"window_overlay")) != NULL)
	    window_overlay = win_data->get_str_value();

	if (strcmp(window_overlay,"ON") != 0)
	    window_canvas->clear();
//	data_panel->clear();
}

void display_window::update()	// read and display from database
{
	if (DEBUG_FG == 1)
		cmd_out("// display_window::update()");

// updating main + data
	window_canvas->update();
	control_panel->update();
}
void display_window::update_window_layer(char* str) // update layer pars
{
	if (DEBUG_FG == 1)
	{
		cout << "// display_window::update_window_layer(";
		cout << str << ");\n";
		cout.flush();
	}

	set_layer_name(str);
	update_window_layer();
}
void display_window::update_window_layer() // update layer pars
{
	if (DEBUG_FG == 1)
		cmd_out("// display_window::update_window_layer()");

	if ((layer = NSL_MODEL->get_layer(layer_name)) != NULL)
	{
		layer_type = layer->get_layer_type();
		set_general_data("LAYER",layer_name);
		control_panel->update("LAYER"); // read_from_database + display
	}
	else
	{
		cmd_error("Unknown Layer: ",layer_name);
		if ((layer = NSL_MODEL->get_layer(1)) != NULL)
		{
			strcpy(layer_name,layer->get_name());
			cmd_error("Using Default Layer: ",layer_name);
			layer_type = layer->get_layer_type();
			set_general_data("LAYER",layer_name);
			control_panel->update("LAYER");
		}
		else
			cmd_error_exit("Couldn't find layer with index 1");
	}
}
void display_window::update_window_graph(char* str) // update graph pars
{
	if (DEBUG_FG == 1)
	{
		cout << "// display_window::update_window_graph(";
		cout << str << ");\n";
		cout.flush();
	}

	set_graph_name(str);
	update_window_graph();
}
void display_window::update_window_graph() // update graph pars
{
	if (DEBUG_FG == 1)
		cmd_out("// display_window::update_window_graph()");

	nsl_data* graph;
	if ((graph = NSL_SYSTEM->get_graph(graph_name)) != NULL)
	{
		graph_type = graph->get_index();
		set_general_data("GRAPH",graph_name);
		control_panel->update("GRAPH"); // read_from_database + display
	}
	else
	{
		cmd_error("Unknown Graphr: ",graph_name);
		if ((graph = NSL_SYSTEM->get_graph(1)) != NULL)
		{
			strcpy(graph_name,graph->get_name());
			cmd_error("Using Default Graph: ",graph_name);
			graph_type = graph->get_index();
			set_general_data("GRAPH",graph_name);
			control_panel->update("GRAPH");
		}
		else
			cmd_error_exit("Couldn't find graph with index 1");
	}
}

void display_window::update_from_panel()
{
	if (DEBUG_FG == 1)
		cmd_out("// display_window::update_from_panel()");

// from panel to database to canvas
	control_panel->write_to_database();

//	NSL << "set display_frame" << parent_frame->get_index() << ENDL;
//	NSL << "set display_window" << index << ENDL;
//	NSL << "status window_item ALL" << ENDL;
//	NSL << "update display_window" << ENDL;

	window_canvas->read_from_database();

	char* window_overlay = "OFF";

	nsl_data* win_data;

	if ((win_data = get_data_obj(data_list,"window_overlay")) != NULL)
	    window_overlay = win_data->get_str_value();

	if (strcmp(window_overlay,"ON") != 0)
	    window_canvas->clear();

	window_canvas->display();
}
void display_window::read_from_database()
{
	if (DEBUG_FG == 1)
		cmd_out("// display_window::read_from_database()");

	control_panel->read_from_database();
	window_canvas->read_from_database(); 
}
void display_window::write_to_database()
{
	if (DEBUG_FG == 1)
		cmd_out("// display_window::write_to_database()");

	control_panel->write_to_database(); 
//	window_canvas->write_to_database();
}
void display_window::display()
{
	if (DEBUG_FG == 1)
		cmd_out("// display_window::display()");

	float ts = NSL_MODEL->get_time();
	int it = NSL_MODEL->get_time_step();
	set_general_data("ts",ts);

	if (ts == 0)
	{
		control_panel->read_from_database();
		control_panel->display();
	}
	
	char* window_overlay = "OFF";
	int display_step = 1;
	nsl_data* win_data;

	if ((win_data = get_data_obj(data_list,"window_step")) != NULL)
		display_step = win_data->get_int_value();

	if ((win_data = get_data_obj(data_list,"window_overlay")) != NULL)
	  window_overlay = win_data->get_str_value();

	if (display_step != 0 && it%display_step == 0)
	{

	    window_canvas->read_from_database();

 	    if (window_canvas->get_clear_fg() == 1 &&
		strcmp(window_overlay,"ON") != 0)
		   window_canvas->clear();

	    window_canvas->display();
	}
}

// init data entry

int display_window::init_general_data(char* name,int type)
{
	if (DEBUG_FG == 1)
	{
		cout << "// display_window::init_general_data(";
		cout << name << "," << type << ");\n";
		cout.flush();
	}

	return init_general_data(name,type,0);
}
int display_window::init_general_data(char* name,int type,int main_fg)
{
	if (DEBUG_FG == 1)
	{
		cout << "// display_window::init_general_data(";
		cout << name << "," << type << "," << main_fg << ");\n";
		cout.flush();
	}

	int index = 0;
	nsl_data* win_data;

	if ((win_data = new nsl_data(name,type)) != NULL)
	{
		win_data->set_flag(main_fg);
		if (main_fg == 1)
			++main_total;
		data_list->append(win_data);
		win_data->set_index(++data_total);
		index = win_data->get_index();
	}
	else
	{
		cmd_error("*** display_window::init_general_data ***");
		cmd_error("Unable to create new database object");
	}

	return index;
}

// set data value

int display_window::set_general_data(char* name,char* sval,int ival)
{
	if (DEBUG_FG == 1)
	{
		cout << "// display_window::set_general_data(";
		cout << name << "," << sval << "," << ival << ");\n";
		cout.flush();
	}

	int index = 0;
	nsl_data* win_data;

	if ((win_data = get_data_obj(data_list,name)) != NULL)
	{
		win_data->set_value(ival); // sets also data type to 1
		win_data->set_value(sval); // sets also data type to 3
		index = win_data->get_index();
	}
	else
	{
		cmd_error("set_general_data");
		cmd_error("Unknown variable: ",name);
	}

	return index;
}
int display_window::set_general_data(char* name,char* val)
{
	if (DEBUG_FG == 1)
	{
		cout << "// display_window::set_general_data(";
		cout << name << "," << val << ");\n";
		cout.flush();
	}

	int index = 0;
	nsl_data* win_data;

	if ((win_data = get_data_obj(data_list,name)) != NULL)
	{
		win_data->set_value(val);
		index = win_data->get_index();
	}
	else
	{
		cmd_error("set_general_data");
		cmd_error("Unknown variable: ",name);
	}

	return index;
}
int display_window::set_general_data(char* name,float val)
{
	if (DEBUG_FG == 1)
	{
		cout << "// display_window::set_general_data(";
		cout << name << "," << val << ");\n";
		cout.flush();
	}

	int index = 0;
	nsl_data* win_data;

	if ((win_data = get_data_obj(data_list,name)) != NULL)
	{
		win_data->set_value(val);
		index = win_data->get_index();
	}
	else
	{
		cmd_error("set_general_data");
		cmd_error("Unknown variable: ",name);
	}

	return index;
}
int display_window::set_general_data(char* name,int val)
{
	if (DEBUG_FG == 1)
	{
		cout << "// display_window::set_general_data(";
		cout << name << "," << val << ");\n";
		cout.flush();
	}

	int index = 0;
	nsl_data* win_data;

	if ((win_data = get_data_obj(data_list,name)) != NULL)
	{
		win_data->set_value(val);
		index = win_data->get_index();
	}
	else
	{
		cmd_error("set_general_data");
		cmd_error("Unknown variable: ",name);
	}

	return index;
}

// get general data

int display_window::get_general_data(char* name,int* val)
{
	if (DEBUG_FG == 1)
	{
		cout << "// display_window::get_general_data(";
		cout << name << ");\n";
		cout.flush();
	}

	int index = 0;
	nsl_data* win_data;

	if ((win_data = get_data_obj(data_list,name)) != NULL)
	{
		*val = win_data->get_int_value();
		index = win_data->get_index();
	}
	else
	{
		cmd_error("get_general_data");
		cmd_error("Unknown variable: ",name);
	}

	if (DEBUG_FG == 1 && index != 0)
	{
		cout << "// \treturn (" << name << ") " << *val << ";\n";
		cout.flush();
	}
	return index;
}
int display_window::get_general_data(char* name,float* val)
{
	if (DEBUG_FG == 1)
	{
		cout << "// display_window::get_general_data(";
		cout << name << ");\n";
		cout.flush();
	}

	int index = 0;
	nsl_data* win_data;

	if ((win_data = get_data_obj(data_list,name)) != NULL)
	{
		*val = win_data->get_float_value();
		index = win_data->get_index();
	}
	else
	{
		cmd_error("get_general_data");
		cmd_error("Unknown variable: ",name);
	}

	if (DEBUG_FG == 1 && index != 0)
	{
		cout << "// \treturn (" << name << ") " << *val << ";\n";
		cout.flush();
	}
	return index;
}
int display_window::get_general_data(char* name,char* val)
{
	if (DEBUG_FG == 1)
	{
		cout << "// display_window::get_general_data(";
		cout << name << ")\n";
		cout.flush();
	}

	int index = 0;
	nsl_data* win_data;

	if ((win_data = get_data_obj(data_list,name)) != NULL)
	{
		strcpy(val,win_data->get_str_value());
		index = win_data->get_index();
	}
	else
	{
		cmd_error("get_general_data");
		cmd_error("Unknown variable: ",name);
	}

	if (DEBUG_FG == 1 && index != 0)
	{
		cout << "// \treturn (" << name << ") " << val << ";\n";
		cout.flush();
	}
	return index;
}
int display_window::get_general_data_type(char* name)
{
	if (DEBUG_FG == 1)
	{
		cout << "// display_window::get_general_data_type(";
		cout << name << ");\n";
		cout.flush();
	}

	int type = 0;
	nsl_data* win_data;

	if ((win_data = get_data_obj(data_list,name)) != NULL)
		type = win_data->get_data_type();
	else
	{
		cmd_error("get_general_data_type");
		cmd_error("Unknown variable: ",name);
	}

	return type;
}
nsl_data* display_window::get_window_data(char* name)
{
	if (DEBUG_FG == 1)
	{
		cout << "// display_window::get_window_data(";
		cout << name << ")\n";
		cout.flush();
	}

	nsl_data* data;

	if ((data = get_data_obj(data_list,name)) != NULL)
		return data;

	return (nsl_data*) 0;
}
nsl_data* display_window::get_window_data(int index)
{
	if (DEBUG_FG == 1)
	{
		cout << "// display_window::get_window_data(";
		cout << index << ")\n";
		cout.flush();
	}

	nsl_data* data;

	if ((data = get_data_obj(data_list,index)) != NULL)
		return data;

	return (nsl_data*) 0;
}

// set window sector

void display_window::set_win_sector() // called only from display_frame
{
	if (DEBUG_FG == 1)
		cmd_out("// display_window::set_window_sector()");

	int i0,i1,imax;
	int j0,j1,jmax;
	int vec_type;

	switch (layer_type)
	{
	case 1: // nsl_data
		set_win_sector(0,0,0,0,1,1); // update window sector values
		break;
	case 2: // nsl_vector
		get_general_data("i0",&i0);
		get_general_data("i1",&i1);
		get_general_data("imax",&imax);
		get_general_data("vec_type",&vec_type);
		if (vec_type == 0) // update window sector
			set_win_sector(0,i0,0,i1,1,imax);
		else
			set_win_sector(i0,0,i1,0,imax,1);
		break;
	case 3: // nsl_matrix
		get_general_data("i0",&i0);
		get_general_data("i1",&i1);
		get_general_data("j0",&j0);
		get_general_data("j1",&j1);
		get_general_data("imax",&imax);
		get_general_data("jmax",&jmax);
		set_win_sector(i0,j0,i1,j1,imax,jmax); // update window sector
		break;
	default:
		cmd_error("*** display_window::set_win_sector ***");
		cmd_error_exit("Unknown NSL_LAYER type: ",layer_type);
	}
}
void display_window::set_win_sector(int i0,int j0,int i1,int j1,
	int imax,int jmax)
{
	if (DEBUG_FG == 1)
	{
		cout << "// display_window::set_win_sector(";
		cout << i0 << "," << j0 << "," << i1 << "," << j1 << ");\n";
		cout.flush();
	}

	int xoff, yoff;

// imax, jmax are crossed
//	int tx0 = (i0 >= 0 && i0 < imax) ? i0 : 0;
//	int ty0 = (j0 >= 0 && j0 < jmax) ? j0 : 0;
//	int ti1 = (i1 >= 0 && i1 >= i0 && i1 < imax) ? i1 : imax - 1;
//	int ty1 = (j1 >= 0 && j1 >= j0 && j1 < jmax) ? j1 : jmax - 1;

	int tj0 = (i0 >= 0 && i0 < imax) ? i0 : 0;
	int ti0 = (j0 >= 0 && j0 < jmax) ? j0 : 0;
	int tj1 = (i1 >= 0 && i1 >= i0 && i1 < imax) ? i1 : imax - 1;
	int ti1 = (j1 >= 0 && j1 >= j0 && j1 < jmax) ? j1 : jmax - 1;

	int dimax = ti1 - ti0 + 1;
	int djmax = tj1 - tj0 + 1;
	
	int ux0 = 20;		
	int ux1 = width - 20;
//	int ux1 = window_canvas->get_width() - 20;	
	int vy0 = 20; 			
	int vy1 = height - 20;
//	int vy1 = window_canvas->get_height() - 20;	

	int width_sector = ux1 - ux0;
	int height_sector = vy1 - vy0;

	int xoff0 = width_sector%dimax;
	int yoff0 = height_sector%djmax;

	if (xoff0%2 == 0)
		xoff = xoff0/2;
	else
		xoff = (xoff0-1)/2;
	if (yoff0%2 == 0)
		yoff = yoff0/2;
	else
		yoff = (yoff0-1)/2;

	int wx0 = ux0 + xoff;
	int wy0 = vy0 + yoff0 - yoff;
	int wx1 = ux1 - (xoff0 - xoff);
	int wy1 = vy1 - yoff;
	int wxs = (width_sector-xoff0)/dimax;
	int wys = (height_sector-yoff0)/djmax;

	set_general_data("wx0",wx0);
	set_general_data("wy0",wy0);
	set_general_data("wx1",wx1);
	set_general_data("wy1",wy1);
	set_general_data("wxs",wxs);
	set_general_data("wys",wys);
}
// mouse input
void display_window::input(int x_pos,int y_pos,int fg)
{
	if (DEBUG_FG == 1)
	{
		cout << "// display_window::output(";
		cout << x_pos << "," << y_pos << "," << fg << ")\n";
		cout.flush();
	}

	window_canvas->input(x_pos,y_pos,fg);
}
void display_window::output(int x_pos,int y_pos)
{
	if (DEBUG_FG == 1)
	{
		cout << "// display_window::output(";
		cout << x_pos << "," << y_pos << ")\n";
		cout.flush();
	}

	window_canvas->output(x_pos,y_pos);
}
// print window data
void display_window::print_status(ostream& out)
{
	if (DEBUG_FG == 1)
		cmd_out("// display_window::print_status(ostream&)");

	out << "set window_name " << name << "\n";
//	out << "set window_index " << index << "\n";
	out << "set window_X0 " << X0 << "\n";
	out << "set window_Y0 " << Y0 << "\n";
	out << "set window_width " << width << "\n";
	out << "set window_height " << height << "\n";
	out << "set window_layer " << layer_name << "\n";
	out << "set window_graph " << graph_name << "\n";
	out << "// total data objects : " << data_total << "\n";
	out << "update display_window\n";
	out << "\n";
	out.flush();
}
void display_window::print_all_item_status(ostream& out)
{
	if (DEBUG_FG == 1)
		cmd_out("// display_window::print_all_item_status(ostream&)");

	if (data_list != NULL)
	{
	   data_type_iterator next(*data_list);
	   nsl_data* data;
	   while (data = next())
	   {
		out << "set window_item_index " << data->get_index() << "\n";
		out << "set window_item_name " << data->get_name() << "\n";
//		out << "set window_item_type " << data->get_type_str() << "\n";
		out << "set window_item_data " << *data; // << "\n";
	        out << "update window_item\n";
	   	out << "\n";
	   }
//	   out << "\nupdate display_window\n";
	   out << "\n";
	   out.flush();
	}
}
void display_window::print_item_status(ostream& out,int index)
{
	if (DEBUG_FG == 1)
	{
		cout << "// display_window::print_item_status(out,";
		cout << index << ")\n";
		cout.flush();
	}

	if (data_list != NULL)
	{
	   data_type_iterator next(*data_list);
	   nsl_data* data;
	   while (data = next())
	   {
	      if (index == data->get_index())
	      {
		out << "set window_item_index " << data->get_index() << "\n";
		out << "set window_item_name " << data->get_name() << "\n";
		out << "// set window_item_type " << data->get_data_type() << "\n";
		out << "set window_item_data " << *data; // << "\n";
	        out << "update window_item\n";
	   	out << "\n";
	      }
	   }
//	   out << "\nupdate display_window\n";
	   out << "\n";
	   out.flush();
	}
}
void display_window::print_item_status(ostream& out,char* str)
{
	if (DEBUG_FG == 1)
	{
		cout << "// display_window::print_item_status(out,";
		cout << str << ")\n";
		cout.flush();
	}

	if (data_list != NULL)
	{
	   data_type_iterator next(*data_list);
	   nsl_data* data;
	   while (data = next())
	   {
	      if (strcmp(str,data->get_name()) == 0)
	      {
		out << "set window_item_index " << data->get_index() << "\n";
		out << "set window_item_name " << data->get_name() << "\n";
		out << "// set window_item_type " << data->get_data_type() << "\n";
		out << "set window_item_data " << *data; // << "\n";
	        out << "update window_item\n";
	   	out << "\n";
	      }
	   }
//	   out << "\nupdate display_window\n";
	   out << "\n";
	   out.flush();
	}
}
void display_window::print_canvas_status(ostream& out)
{
	if (DEBUG_FG == 1)
	{
		cout << "// display_window::print_canvas_status(out)\n";
		cout.flush();
	}

	window_canvas->print_status(out);
}
void display_window::print_panel_status(ostream& out)
{
	if (DEBUG_FG == 1)
	{
		cout << "// display_window::print_panel_status(out)\n";
		cout.flush();
	}

	control_panel->print_status(out);
}
