/****************************************************************/
/*                                                            	*/
/*                       frame_winds.c                        	*/
/*                                                            	*/
/* by Alfredo Weitzenfeld				      	*/
/*                                                            	*/
/****************************************************************/

// static char SccsId[] = "%W%  %G%"; /* SCCS */

# include "nsl.h"
# include "nsl_headers.h"
# include "nsl_winds.h"

# include <xview/icon.h>

extern int BREAKFG;
extern window_interface* NSL_INTERFACE;

extern Notify_value catch_resize(Frame,Event*,Notify_arg,Notify_event_type);
extern Notify_value catch_destroy(Frame,Destroy_status);
extern Notify_value win_urgent_handler(Notify_client,int);

Frame create_frame(Frame frame,int x,int y,int width,int height,int data)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// create_frame(" << frame << "," << x << "," << y <<
	      "," << width << "," << height << "," << data << ")\n";
		cout.flush();
	}

//	cmd_out("frame width ",width);
//	cmd_out("frame height ",height);

	return xv_create(frame, FRAME,
		XV_X, x,
		XV_Y, y,
		XV_WIDTH, width,
		XV_HEIGHT, height,
		WIN_CLIENT_DATA, data,
		XV_KEY_DATA, 1, data,
		XV_SHOW,TRUE,
		0);
}
Frame create_cmd_frame(Frame frame,int x,int y,int width,int height,int data)
{
	if (DEBUG_FG == 1)
	{
	    cout << "// create_cmd_frame(" << frame << "," << x << "," << y <<
	      "," << width << "," << height << "," << data << ")\n";
		cout.flush();
	}

//	cmd_out("cmd frame width ",width);
//	cmd_out("cmd frame height ",height);

	return xv_create(frame, FRAME_CMD,
		XV_X, x,
		XV_Y, y,
		XV_WIDTH, width,
		XV_HEIGHT, height,
		WIN_CLIENT_DATA, data,
		FRAME_CMD_PUSHPIN_IN, TRUE,
		XV_SHOW,TRUE,
		0);
}
void set_frame_label(Frame frame,char* str)
{
	xv_set(frame,XV_LABEL,str,0);
}
void set_frame_no_label(Frame frame,char* str)
{
	xv_set(frame,
		XV_LABEL,str,
		FRAME_SHOW_LABEL, FALSE,
		0);
}

void show_frame(Frame frame)
{
	xv_set(frame,XV_SHOW,TRUE,0);
}
void unshow_frame(Frame frame)
{
	xv_set(frame,FRAME_CMD_PUSHPIN_IN,FALSE,0);				
	xv_set(frame,XV_SHOW,FALSE,0);
	XFlush((Display*)xv_get(frame,XV_DISPLAY));
}
void set_frame_icon(Frame frame,char* str)
{
	Icon icon = xv_create(frame,ICON,
		XV_HEIGHT,64,
		XV_WIDTH,64,
		XV_LABEL,str,
		0);
	xv_set(frame,FRAME_ICON,icon,0);
}
int destroy_frame(Frame frame)
{
        xv_set(frame, FRAME_NO_CONFIRM, TRUE, 0);
	/* xv_destroy(frame); */
        xv_destroy_safe(frame);

	return 1;
}
void set_frame_key_data(Frame frame,int key,int data)
{
	xv_set(frame,XV_KEY_DATA,key,data,0);
}
int get_frame_key_data(Frame frame,int key)
{
	return (int) xv_get(frame,XV_KEY_DATA,key);
}
int resize_frame(Frame frame,int x,int y,int w,int h)
{
	xv_set(frame,
		XV_X,x,
		XV_Y,y,
		XV_WIDTH,w,
		XV_HEIGHT,h,
		XV_SHOW, TRUE, 
		0);

	return 1;
}
int get_frame_height(Frame frame)
{
	return (int) xv_get(frame,XV_HEIGHT);
}
int get_frame_width(Frame frame)
{
	return (int) xv_get(frame,XV_WIDTH);
}

int set_frame_defaults(Frame frame)
{
	xv_set(frame,
//		FRAME_ARGS,ARGC,ARGV,
//	       	FRAME_FOREGROUND_COLOR,&FRAME_FCOLOR,
//        	FRAME_BACKGROUND_COLOR,&FRAME_BCOLOR,
              	FRAME_INHERIT_COLORS, TRUE,
//		WIN_CONSUME_EVENT,WIN_ASCII_EVENTS,
		0);

	notify_interpose_event_func(frame,
		(Notify_func) catch_resize,NOTIFY_SAFE);
	notify_interpose_destroy_func(frame,(Notify_func)catch_destroy);

	return 1;
}

// Notifier (interpose)

Notify_value catch_resize(Frame frame0,Event* event,
	Notify_arg arg,Notify_event_type type)
{
	display_frame* frame;
     	display_panel* panel;
	int resize_fg=0,done_fg=0;    

	/* 95-2-8 ama - Notify_value value = 1111; */
	Notify_value value = (Notify_value) 1111;

	if (event_action(event) == WIN_RESIZE)  // event_id
	{
	    int fr_index = (int) xv_get(frame0,XV_KEY_DATA,1); 
	    if ((frame = NSL_INTERFACE->get_display_frame(fr_index)) != NULL
		&& (resize_fg = frame->get_resize_fg()) == 1
		&& (done_fg = frame->get_done_fg()) == 0)
	    {
		int i = frame->get_init_fg();
//		cmd_out("RESIZE ",i);
//		if (i > 5) // resize after creation only
	    	    frame->resize();
//		else // resize next time
		    frame->set_init_fg(++i);
	    }
//	    else
//		    cmd_error("Cannot resize frame: ",fr_index);
//	    cmd_out("resize ",resize_fg);
//	    cmd_out("done ",done_fg);	
	}
	else
	    value = notify_next_event_func(frame0,
		(Notify_event) event,arg,type);

	return(value); 
}
Notify_value catch_destroy(Frame frame0,Destroy_status status)
{
 	display_frame* frame;
	int i;

        if (status == DESTROY_CHECKING)
        {
	    int fr_index = (int) xv_get(frame0,XV_KEY_DATA,1); 
	    if ((frame = NSL_INTERFACE->get_display_frame(fr_index)) != NULL)
	    {
//		frame->set_done_fg(1);
	    }
        }
        return (notify_next_destroy_func(frame0,status));
}

