/**************************************************************/
/*                                                            */
/*                       nsl_buffer.c 		              */
/*                                                            */
/* by Alfredo Weitzenfeld 				      */
/*                                                            */
/**************************************************************/

//static char SccsId[] = "%W%  %G%"; /* SCCS */

# include "nsl.h"
# include "nsl_headers.h"

extern fstream* LOG_FILE;	// file pointer
extern int LOG_FILE_FG; 	// 1 - log is on
extern int LOG_CMD_FG;		// 1 - log command

nsl_buffer::nsl_buffer()
{
	stream = new strstream;
	sb = stream->rdbuf();
	able_fg = 1;
}
nsl_buffer::~nsl_buffer()
{
}
void nsl_buffer::reset_stream()
{
	rewind_stream();
	reset_all_cnt();	
}
void nsl_buffer::rewind_stream()
{
	sb->seekpos(streampos(0));
//	sb->seekoff(0,ios::beg,ios::in);
}
istream& nsl_buffer::operator>>(char* str)
{
	str_cnt--;
	cmd_cnt--;

	(*stream) >> str;

	return (*stream);
}
istream& nsl_buffer::operator>>(nsl_layer& layer)
{
	return (*stream) >> layer;
}
istream& nsl_buffer::operator>>(nsl_layer* layer)
{
	return (*stream) >> *layer;
}
istream& nsl_buffer::operator>>(float& val)
{
	str_cnt--;

	return (*stream) >> val;
}
istream& nsl_buffer::operator>>(int& val)
{
	str_cnt--;

	return (*stream) >> val;
}
ostream& nsl_buffer::operator<<(char* str)
{
	str_cnt++;
	cmd_cnt++;

	(*stream) << str;
	(*stream) << " "; // keep sapce between strings

	return (*stream);
}
ostream& nsl_buffer::operator<<(nsl_layer* layer)
{
	size_cnt = 1;
	xn_cnt = 1;
	yn_cnt = 1;
	int type = layer->get_layer_type();
	switch (type)
	{
	case 1:
		break;
	case 2:
		xn_cnt = ((nsl_vector*) layer)->get_imax();
		size_cnt = xn_cnt;
		break;
	case 3:
		xn_cnt = ((nsl_matrix*) layer)->get_imax();
		yn_cnt = ((nsl_matrix*) layer)->get_jmax();
		size_cnt = xn_cnt*yn_cnt;
		break;
	default:
		cmd_error("Bad layer type: ",type);
	}

	str_cnt += size_cnt;

	(*stream) << *layer;
	stream->flush();

	return (*stream);
}
ostream& nsl_buffer::operator<<(nsl_layer& layer)
{
	size_cnt = 1;
	xn_cnt = 1;
	yn_cnt = 1;
	int type = layer.get_layer_type();
	switch (type)
	{
	case 1:
		break;
	case 2:
		xn_cnt = ((nsl_vector&) layer).get_imax();
		size_cnt = xn_cnt;
		break;
	case 3:
		xn_cnt = ((nsl_matrix&) layer).get_imax();
		yn_cnt = ((nsl_matrix&) layer).get_jmax();
		size_cnt = xn_cnt*yn_cnt;
		break;
	default:
		cmd_error("Bad layer type: ",type);
	}

	str_cnt += size_cnt;

	(*stream) << layer;
	stream->flush();

	return (*stream);
}
ostream& nsl_buffer::operator<<(float val)
{
	str_cnt++;
	size_cnt = 1;
	xn_cnt = 1;
	yn_cnt = 1;

	return (*stream) << val;
}
ostream& nsl_buffer::operator<<(int val)
{
	str_cnt++;
	size_cnt = 1;
	xn_cnt = 1;
	yn_cnt = 1;

	return (*stream) << val;
}
istream& nsl_buffer::operator>>(ostream& out)
{
	string str;
	
	int i = 0;

	int row;
	int flag = 1;

	int tmp_str_cnt = str_cnt;
	int tmp_cmd_cnt = cmd_cnt;
	int tmp_size_cnt = size_cnt;

	while (tmp_str_cnt > 0)
	{
	    (*stream) >> str;

	    tmp_str_cnt--;

	    if (tmp_cmd_cnt > 0)
	    	tmp_cmd_cnt--;

	    out << str << " ";

	    flag = 1;

	    if (tmp_cmd_cnt <= 0 && xn_cnt > 0 && yn_cnt > 0)
	    {
		if (yn_cnt == 1)
		   row = tmp_size_cnt%xn_cnt;
		else
		   row = tmp_size_cnt%yn_cnt;

		tmp_size_cnt--;
		if (row == 0)
		{
		   out << endl;
		   flag = 0;
		}
	    }
	}

	if (flag == 1)
	{
	   out << endl;
	   out.flush();
	}

	return (*stream);
}
void nsl_buffer::print_status(ostream& out)
{
	out << "\n";
	if (able_fg == 1)
		out << "// stream buffer enabled\n";
	else
		out << "// stream buffer disabled\n";
	out << "// str_cnt " << str_cnt << endl;
	out << "// cmd_cnt " << cmd_cnt << endl;
	out << "// xn_cnt " << xn_cnt << endl;
	out << "// yn_cnt " << yn_cnt << endl;
	out << "// size_cnt " << size_cnt << endl;

	out.flush();
}

