/**************************************************************/
/*                                                            */
/*                   nsl_command_buffer.c                     */
/*                                                            */
/* by Alfredo Weitzenfeld 				      */
/*                                                            */
/**************************************************************/

# include "nsl_command.h"

command_buffer* CMD_BUF;

extern fstream* LOG_FILE;
extern int LOG_FILE_FG;

extern nsl_buffer* NSL_BUFFER;

nsl_data* get_data_obj(data_type_list*,char*);

int nsl_get_on_off(nsl_data* data)
{
	char* str = data->get_str_value();
	if (strcmp(str,"ON") == 0)
		return 1;
	else if (strcmp(str,"OFF") == 0)
		return 0;
	else
		cmd_error("Unknown modifier (only ON or OFF): ",str);

	return 0;
}
command_buffer::command_buffer()
{
	CMD_BUF = this;

	data_total = 0;
	data_list = new data_type_list;

// add type to data depending on model,layer, etc.
	init_general_data("DEBUG_FG",3);
	init_general_data("VERBOSE_FG",3);
	init_general_data("SIM_FG",3);
	init_general_data("SIM_TIME_FG",3);
	set_general_data("DEBUG_FG","OFF");
	set_general_data("VERBOSE_FG","ON");
	set_general_data("SIM_FG","OFF");
	set_general_data("SIM_TIME_FG","OFF");

// model
	init_general_data("network",3); // model
	init_general_data("network_name",3); // model
	init_general_data("network_index",1); // model
	init_general_data("end_time",2);
	init_general_data("current_time",2);
	init_general_data("delta",2);
	init_general_data("integration",3);
//	set_general_data("network_name"); // model
	set_general_data("network_index",0); // model
	set_general_data("end_time",(float) 1.0);
	set_general_data("current_time",(float) 0.0);
	set_general_data("delta",(float) 0.1);
	set_general_data("integration","EULER");

// module
	init_general_data("module",3); // model
	init_general_data("module_name",3); // model
	init_general_data("module_index",1); // model
//	set_general_data("module_name"); // model
	set_general_data("module_index",0); // model

// layer
	init_general_data("layer",3);
	init_general_data("layer_name",3);
	init_general_data("layer_index",1);
	init_general_data("elem_i",1);
	init_general_data("elem_j",1);
//	set_general_data("layer_name");
	set_general_data("layer_index",0);
	set_general_data("elem_i",(int) 0);
	set_general_data("elem_j",(int) 0);

// input_layer
	init_general_data("input_layer",3);
	init_general_data("input_layer_name",3);
	init_general_data("input_layer_index",1);
	init_general_data("input_xz",1);
	init_general_data("input_yz",1);
	init_general_data("input_dx",2);
	init_general_data("input_dy",2);
//	set_general_data("input_layer_name");
	set_general_data("input_layer_index",0);
	set_general_data("input_xz",(int) 0);
	set_general_data("input_yz",(int) 0);
	set_general_data("input_dx",(float) 1.0);
	set_general_data("input_dy",(float) 1.0);

// stim_obj
	init_general_data("stim",1); // index
//	init_general_data("stim_name",3); // name
	init_general_data("stim_index",1); // index
	init_general_data("stim_xc",2);
	init_general_data("stim_yc",2);
	init_general_data("stim_x0",2);
	init_general_data("stim_y0",2);
	init_general_data("stim_val",2);
	init_general_data("stim_dx",2);
	init_general_data("stim_dy",2);
	init_general_data("stim_vx",2);
	init_general_data("stim_vy",2);
	init_general_data("stim_t0",2);
	init_general_data("stim_t1",2);
	init_general_data("stim_spec",3); // name
	init_general_data("stim_type",3); // name
	set_general_data("stim",0);
//	set_general_data("stim_name");
	set_general_data("stim_index",0);
	set_general_data("stim_xc",(float) 0.0);
	set_general_data("stim_yc",(float) 0.0);
	set_general_data("stim_x0",(float) 0.0);
	set_general_data("stim_y0",(float) 0.0);
	set_general_data("stim_val",(float) 1.0);
	set_general_data("stim_dx",(float) 0.0);
	set_general_data("stim_dy",(float) 0.0);
	set_general_data("stim_t0",(float) 0.0);
	set_general_data("stim_t1",(float) 1.0);
	set_general_data("stim_spec","CORNER"); // CENTER or CORNER
	set_general_data("stim_type","BLOCK");  // BLOCK or ICON

// frame
	init_general_data("display_frame",3);
	init_general_data("frame_name",3);
	init_general_data("frame_index",1);
	init_general_data("frame_X0",1);
	init_general_data("frame_Y0",1);
	init_general_data("frame_width",1);
	init_general_data("frame_height",1);
	init_general_data("frame_auto_resize",3);// 1
	init_general_data("frame_cols",1);
	init_general_data("frame_rows",1);
//	set_general_data("frame_name");
	set_general_data("frame_index",0);
	set_general_data("frame_X0",(int) 0);
	set_general_data("frame_Y0",(int) 0);
	set_general_data("frame_width",(int) 500); 
	set_general_data("frame_height",(int) 500); 
	set_general_data("frame_auto_resize","ON");
	set_general_data("frame_cols",(int) 1);
	set_general_data("frame_rows",(int) 1);

// window
	init_general_data("display_window",3);
	init_general_data("window_name",3);
	init_general_data("window_index",1);
	init_general_data("window_X0",1);
	init_general_data("window_Y0",1);
	init_general_data("window_width",1);
	init_general_data("window_height",1);
	init_general_data("window_step",1);
	init_general_data("window_overlay",3);
//	set_general_data("window_name");
	set_general_data("window_index",0);
	set_general_data("window_X0",(int) 0);
	set_general_data("window_Y0",(int) 0);
	set_general_data("window_width",(int) 0);
	set_general_data("window_height",(int) 0);
	set_general_data("window_step",(int) 1);
	set_general_data("window_overlay","OFF");
// window item
	init_general_data("window_layer",3);
	init_general_data("window_graph",3);
	init_general_data("window_wymin",2);
	init_general_data("window_wymax",2);
	init_general_data("window_val_in",2);
	init_general_data("window_layer_i0",1); // x0
	init_general_data("window_layer_i1",1); // x1
	init_general_data("window_layer_j0",1); // y0
	init_general_data("window_layer_j1",1); // y1
	init_general_data("window_t0",2);
	init_general_data("window_t1",2);
	init_general_data("window_ax",2);
	init_general_data("window_by",2);
	init_general_data("window_cz",2);
	init_general_data("window_sx",2);
	init_general_data("window_sy",2);
	init_general_data("window_sz",2);
	init_general_data("window_pulse",3);
	init_general_data("window_hidden_line",3);
	init_general_data("window_vec_type",3);
//	set_general_data("window_layer");
	set_general_data("window_graph","area_level_graph");
	set_general_data("window_wymin",(float) 0.0);
	set_general_data("window_wymax",(float) 1.0);
	set_general_data("window_val_in",(float) 2.0);
	set_general_data("window_layer_i0",(int) 0);
	set_general_data("window_layer_i1",(int) 0);
	set_general_data("window_layer_j0",(int) 0);
	set_general_data("window_layer_j1",(int) 0);
	set_general_data("window_t0",(float) 0.0);
	set_general_data("window_t1",(float) 1.0);
	set_general_data("window_ax",(float) 1.0);
	set_general_data("window_by",(float) 1.0);
	set_general_data("window_cz",(float) 1.0);
	set_general_data("window_sx",(float) 1.0);
	set_general_data("window_sy",(float) 1.0);
	set_general_data("window_sz",(float) 1.0);
	// spatial_graph_2, temporal_graph
	set_general_data("window_pulse","OFF");
	set_general_data("window_hidden_line","ON");
	set_general_data("window_vec_type","HORIZONTAL"); // or VERTICAL

// window item
	init_general_data("window_item_index",1);
	init_general_data("window_item_name",3);
	init_general_data("window_item_type",3);
	init_general_data("window_item_data",3);
	set_general_data("window_item_index",(int) 0);

// canvas
	init_general_data("canvas_name",3);
	init_general_data("canvas_index",1);
	set_general_data("canvas_index",(int) 0);
//	set_general_data("canvas_name");

// canvas item
	init_general_data("canvas_item_name",3);
	init_general_data("canvas_item_index",1);
	init_general_data("canvas_item_enable",3);
	init_general_data("canvas_item_wx",1);
	init_general_data("canvas_item_wy",1);
	set_general_data("canvas_item_index",(int) 0);
	set_general_data("canvas_item_enable","ON");
	set_general_data("canvas_item_wx",(int) 0);
	set_general_data("canvas_item_wy",(int) 0);

// panel
	init_general_data("panel_name",3);
	init_general_data("panel_index",1);
	set_general_data("panel_index",(int) 0);
//	set_general_data("panel_name");

// panel item
	init_general_data("panel_item_name",3);
	init_general_data("panel_item_index",1);
	init_general_data("panel_item_enable",3);
	init_general_data("panel_item_wx",1);
	init_general_data("panel_item_wy",1);
	set_general_data("panel_item_index",(int) 0);
	set_general_data("panel_item_enable","ON");
	set_general_data("panel_item_wx",(int) 0);
	set_general_data("panel_item_wy",(int) 0);

// external files
	init_general_data("log_file",3);
	set_general_data("log_file","COUT");

	init_general_data("file_auto_name",3);
	init_general_data("file_name",3);
	init_general_data("file_index",1);
	init_general_data("file_type",3);
	set_general_data("file_auto_name","ON");
//	set_general_data("file_name");
	set_general_data("file_index",(int) 0);
	set_general_data("file_type","OUTPUT"); // INPUT,OUTPUT

	init_general_data("dump_auto_name",3);
	init_general_data("dump_file",3);
	init_general_data("dump_ras_file",3);
	init_general_data("dump_ps_file",3);
//	init_general_data("dump_index",1);
	init_general_data("dump_ras_index",1);
	init_general_data("dump_ps_index",1);
	init_general_data("dump_type",3);
	init_general_data("dump_text",3);
	set_general_data("dump_auto_name","ON");
//	set_general_data("dump_file");
//	set_general_data("dump_ras_file");
//	set_general_data("dump_ps_file");
//	set_general_data("dump_index",(int) 0);
	set_general_data("dump_ras_index",(int) 0);
	set_general_data("dump_ps_index",(int) 0);
	set_general_data("dump_type","PRINTER"); // PRINTER,PS,RAS
}
command_buffer::~command_buffer()
{
}
// update
void command_buffer::update_layer_buffer(char* str)
{
	if (DEBUG_FG == 1)
	{
		cout << "// command_buffer::update_layer_buffer(";
		cout << str << ");\n";
		cout.flush();
	}

	nsl_layer* layer;
	nsl_data* data;

	if ((layer = NSL_MODEL->get_layer(str)) != NULL)
	{
		int type = layer->get_layer_type();
		switch (type)
		{
		case 1:
		   if ((data = get_data("window_layer_i0")) != NULL)
 			data->set_value(0);
		   if ((data = get_data("window_layer_i1")) != NULL)
			data->set_value(0);
 		   if ((data = get_data("window_layer_j0")) != NULL)
			data->set_value(0);
 		   if ((data = get_data("window_layer_j1")) != NULL)
			data->set_value(0);
		   break;
 		case 2:
		   if ((data = get_data("window_layer_i0")) != NULL)
 			data->set_value(0);
		   if ((data = get_data("window_layer_i1")) != NULL)
			data->set_value(((nsl_vector*) layer)->get_imax()-1);
 		   if ((data = get_data("window_layer_j0")) != NULL)
			data->set_value(0);
 		   if ((data = get_data("window_layer_j1")) != NULL)
			data->set_value(0);
 		   break;
		case 3:
		   if ((data = get_data("window_layer_i0")) != NULL)
			data->set_value(0);
 		   if ((data = get_data("window_layer_i1")) != NULL)
			data->set_value(((nsl_matrix*) layer)->get_imax()-1);
 		   if ((data = get_data("window_layer_j0")) != NULL)
			data->set_value(0);
 		   if ((data = get_data("window_layer_j1")) != NULL)
			data->set_value(((nsl_matrix*) layer)->get_jmax()-1);
		   break;
		default:
		   cmd_error("Unknown layer type: ",type);
		}
	}
	else
	{
		cmd_error("*** command_buffer::update_layer_buffer ***");
		cmd_error("NULL layer");
	}
}
void command_buffer::update_graph_buffer(char* str)
{
	if (DEBUG_FG == 1)
	{
		cout << "// command_buffer::update_graph_buffer(";
		cout << str << ");\n";
		cout.flush();
	}
}

// layer

void command_buffer::set_layer_elem_data_value(istream& in_file,
	nsl_layer* layer,int i,int j)
{
	num_type val;
	nsl_data dval;

	if (layer != NULL)
	{
		in_file >> dval;

		if (!in_file) // not working
			return;

		val = dval.elem();

       	 	int type = layer->get_layer_type();

  	      	switch (type)
  	      	{
 	      	case 1:
                	*((nsl_data*) layer) = val;
                	break;
        	case 2:
                	((nsl_vector*) layer)->elem(i) = val;
                	break;
        	case 3:
                	((nsl_matrix*) layer)->elem(i,j) = val;
                	break;
        	default:
                	cmd_error("*** set_layer_elem_data_value ***");
                	cmd_error("Bad layer type",type);
        	}
	}
	else
	{
		cmd_out("*** command_buffer::set_layer_elem_data_value ***");
		cmd_out("Null layer");
	}
}
void command_buffer::set_layer_all_data_value(istream& in_file,nsl_layer* layer)
{
	num_type val;
	nsl_data dval;

	if (layer != NULL)
	{
		in_file >> dval;

		if (!in_file) // not working
			return;

		(*layer) = dval;
	}
	else
	{
		cmd_out("*** command_buffer::set_layer_all_data_value ***");
		cmd_out("Null layer");
	}
}
void command_buffer::set_layer_data_value(istream& in_file,nsl_layer* layer)
{
	if (layer != NULL)
	{
		in_file >> *layer;
	}
	else
	{
		cmd_out("*** command_buffer::set_layer_data_value ***");
		cmd_out("Null layer");
	}
}

// init data entry

int command_buffer::init_general_data(char* name,int type)
{
	int index = 0;
	nsl_data* data;

	if (data_list != NULL)
	{
	    if ((data = new nsl_data(name,type)) != NULL)
	    {
		data_list->append(data);
		data->set_index(data_total);
		index = data->get_index();
	    }
	    else
	    {
		cmd_error("*** command_buffer::init_general_data ***");
		cmd_error("Unable to create new database object");
	    }
	}
	else
	{
		cmd_error("*** command_buffer::init_general_data ***");
		cmd_error("NULL data_list");
	}

	return index;
}

// set data value

int command_buffer::set_general_data(char* name,char* val)
{
	int index = 0;
	nsl_data* data;

	if (data_list != NULL)
	{
	   if ((data = get_data_obj(data_list,name)) != NULL)
	   {
		data->set_value(val);
		index = data->get_index();
	   }
	   else
	   {
		cmd_error("*** command_buffer::set_general_data ***");
		cmd_error("Unknown variable: ",name);
	   }
	}
	else
	{
		cmd_error("*** command_buffer::set_general_data ***");
		cmd_error("NULL data_list");
	}

	return index;
}
int command_buffer::set_general_data(char* name,float val)
{
	int index = 0;
	nsl_data* data;

	if (data_list != NULL)
	{
	    if ((data = get_data_obj(data_list,name)) != NULL)
	    {
		data->set_value(val);
		index = data->get_index();
	    }
	    else
	    {
		cmd_error("*** command_buffer::set_general_data ***");
		cmd_error("Unknown variable: ",name);
	    }
	}
	else
	{
		cmd_error("*** command_buffer::set_general_data ***");
		cmd_error("NULL data_list");
	}

	return index;
}
int command_buffer::set_general_data(char* name,int val)
{
	int index = 0;
	nsl_data* data;

	if (data_list != NULL)
	{
	    if ((data = get_data_obj(data_list,name)) != NULL)
	    {
	 	data->set_value(val);
		index = data->get_index();
	    }
	    else
	    {
		cmd_error("*** command_buffer::set_general_data ***");
		cmd_error("Unknown variable: ",name);
	    }
	}
	else
	{
		cmd_error("*** command_buffer::set_general_data ***");
		cmd_error("NULL data_list");
	}

	return index;
}
int command_buffer::set_on_off_data(char* name,int val)
{
	int index = 0;
	nsl_data* data;

	if (data_list != NULL)
	{
	    if ((data = get_data_obj(data_list,name)) != NULL)
	    {
		if (val == 0)
	 	   data->set_value("OFF");
		else 
	 	   data->set_value("ON");
		index = data->get_index();
	    }
	    else
	    {
		cmd_error("*** command_buffer::set_on_off_data ***");
		cmd_error("Unknown variable: ",name);
	    }
	}
	else
	{
		cmd_error("*** command_buffer::set_on_off_data ***");
		cmd_error("NULL data_list");
	}

	return index;
}

// get general data

int command_buffer::get_general_data(char* name,int* val)
{
	int index = 0;
	nsl_data* data;

	if (data_list != NULL)
	{
	    if ((data = get_data_obj(data_list,name)) != NULL)
	    {
		*val = data->get_int_value();
		index = data->get_index();
	    }
	    else
	    {
		cmd_error("*** command_buffer::get_general_data ***");
		cmd_error("Unknown variable: ",name);
	    }
	}
	else
	{
		cmd_error("*** command_buffer::get_general_data ***");
		cmd_error("NULL data_list");
	}

	return index;
}
int command_buffer::get_general_data(char* name,float* val)
{
	int index = 0;
	nsl_data* data;

	if (data_list != NULL)
	{
	    if ((data = get_data_obj(data_list,name)) != NULL)
	    {
		*val = data->get_float_value();
		index = data->get_index();
	    }
	    else
	    {
		cmd_error("*** command_buffer::get_general_data ***");
		cmd_error("Unknown variable: ",name);
	    }
	}
	else
	{
		cmd_error("*** command_buffer::get_general_data ***");
		cmd_error("NULL data_list");
	}

	return index;
}
int command_buffer::get_general_data(char* name,char* val)
{
	int index = 0;
	nsl_data* data;

	if (data_list != NULL)
	{
	    if ((data = get_data_obj(data_list,name)) != NULL)
	    {
		strcpy(val,data->get_str_value());
		index = data->get_index();
	    }
	    else
	    {
		cmd_error("*** command_buffer::get_general_data ***");
		cmd_error("Unknown variable: ",name);
    	    }
	}
	else
	{
		cmd_error("*** command_buffer::get_general_data ***");
		cmd_error("NULL data_list");
	}

	return index;
}
int command_buffer::get_on_off_data(char* name)
{
	nsl_data* data;
	char* str;

	if (data_list != NULL)
	{
	    if ((data = get_data_obj(data_list,name)) != NULL)
	    {
		return nsl_get_on_off(data);
	    }
	    else
	    {
		cmd_error("*** command_buffer::get_on_off_data ***");
		cmd_error("Unknown variable: ",name);
	    }
	}
	else
	{
		cmd_error("*** command_buffer::get_on_off_data ***");
		cmd_error("NULL data_list");
	}

	return 0;
}
int command_buffer::get_general_data_type(char* name)
{
	int type = 0;
	nsl_data* data;

	if (data_list != NULL)
	{
	    if ((data = get_data_obj(data_list,name)) != NULL)
		type = data->get_data_type();
	    else
	    {
		cmd_error("*** command_buffer::get_general_data_type ***");
		cmd_error("Unknown variable: ",name);
	    }
	}
	else
	{
		cmd_error("command_buffer::get_general_data_type ***");
		cmd_error("NULL data_list");
	}

	return type;
}
nsl_data* command_buffer::get_data(char* name)
{
	int type = 0;
	nsl_data* data;

	if (data_list != NULL)
	{
    	    if ((data = get_data_obj(data_list,name)) == NULL)
	    {
		cmd_error("*** command_buffer::get_data ***");
		cmd_error("Unknown variable: ",name);
	    }
	}
	else
	{
		cmd_error("*** command_buffer::get_data ***");
		cmd_error("NULL data_list");
	}

	return data;
}
void command_buffer::print(ostream& out)
{
	if (data_list != NULL)
	{
	     data_type_iterator next(*data_list);
	     nsl_data* data;

	     while (data = next())
		out << "set " << data->get_name() << " " << *data;

	     out.flush();
	}
	else
	{
		cmd_error("*** command_buffer::print ***");
		cmd_error("NULL data_list");
	}
}

