/**************************************************************/
/*                                                            */
/*              	 nsl_command_enable.c                 */
/*                                                            */
/* by Alfredo Weitzenfeld 				      */
/*                                                            */
/**************************************************************/

# include "nsl_command.h"

extern void show_frame(Frame);
extern void unshow_frame(Frame);

void nsl_help_enable()
{
	cout << "\nenable 'type' ['option']\n";
	cout << "\n";
	cout << "enable processing for object type\n";
	cout << "\n";
	cout << "    type    \t  option  \t    description\n";
	cout << "------------\t----------\t-------------------\n";
	cout << "\n";
	cout << "network\t\t\t\tenable current model\n"; // model
	cout << "\t\t'name'\t\tenable a particular model by name\n";
	cout << "\t\t'index'\t\tenable a particular model by index\n";
	cout << "module\t\t\t\tenable current module\n";
	cout << "\t\t'name'\t\tenable a particular module by name\n";
	cout << "\t\t'index'\t\tenable a particular module by index\n";
	cout << "layer\t\t\t\tenable current layer\n";
	cout << "\t\t'name'\t\tenable a particular layer by name\n";
	cout << "\t\t'index'\t\tenable a particular layer by index\n";
	cout << "input_layer\t\t\tenable current input layer\n";
	cout << "\t\t'name'\t\tenable an input layer by name\n";
	cout << "\t\t'index'\t\tenable an input layer by index\n";
	cout << "input_stim\t\t\tenable current input stim\n";
	cout << "\t\t'name'\t\tenable an input stim by name\n";
	cout << "\t\t'index'\t\tenable an input stim by index\n";
	cout << "display_panel\t\t\tenable control panel\n";
	cout << "\t\t'name'\t\tenable control panel by name\n";
	cout << "\t\t'index'\t\tenable control panel by index\n";
	cout << "canvas_item\t\t\tenable current canvas item\n";
	cout << "\t\t'index'\t\tenable canvas item by index\n";
	cout << "panel_item\t\t\tenable current panel item\n";
	cout << "\t\t'index'\t\tenable panel item by index\n";
	cout << "\n";
	cout << "- 'name' is either a string name or 'ALL'\n";
	cout << "- 'index' is an integer\n";
}
void nsl_help_disable()
{
	cout << "\ndisable 'type' ['option']\n";
	cout << "\n";
	cout << "disable processing for object type\n";
	cout << "\n";
	cout << "    type    \t  option  \t    description\n";
	cout << "------------\t----------\t-------------------\n";
	cout << "\n";
	cout << "network\t\t\t\tdisable current model\n"; // model
	cout << "\t\t'name'\t\tdisable a particular model by name\n";
	cout << "\t\t'index'\t\tdisable a particular model by index\n";
	cout << "module\t\t\t\tdisable current module\n";
	cout << "\t\t'name'\t\tdisable a particular module by name\n";
	cout << "\t\t'index'\t\tdisable a particular module by index\n";
	cout << "layer\t\t\t\tdisable current layer\n";
	cout << "\t\t'name'\t\tdisable a particular layer by name\n";
	cout << "\t\t'index'\t\tdisable a particular layer by index\n";
	cout << "input_layer\t\t\tdisable current input layer\n";
	cout << "\t\t'name'\t\tdisable an input layer by name\n";
	cout << "\t\t'index'\t\tdisable an input layer by index\n";
	cout << "input_stim\t\t\tdisable current input stim\n";
	cout << "\t\t'name'\t\tdisable an input stim by name\n";
	cout << "\t\t'index'\t\tdisable an input stim by index\n";
	cout << "display_panel\t\t\tdisable control panel\n";
	cout << "\t\t'name'\t\tdisable control panel by name\n";
	cout << "\t\t'index'\t\tdisable control panel by index\n";
	cout << "canvas_item\t\t\tdisable current canvas item\n";
	cout << "\t\t'index'\t\tdisable canvas item by index\n";
	cout << "panel_item\t\t\tdisable current panel item\n";
	cout << "\t\t'index'\t\tdisable panel item by index\n";
	cout << "\n";
	cout << "- 'name' is either a string name or 'ALL'\n";
	cout << "- 'index' is an integer\n";
}
int nsl_enable_command(istream& in_file)
{
	int status = 1;
	string str1,str2;
	int index;

	if (nsl_get_str(in_file,str1) == 0)
	{
		nsl_help_enable();
		return 1;
	}

	set_global_pointers();

	if (strcmp("network",str1) == 0 || strcmp("model",str1) == 0)
	{
	        if (nsl_get_str(in_file,str2) == 0)
		    NSL_MODEL->enable(); // set network
		else if ((index = atoi(str2)) != 0)
		    NSL_SYSTEM->enable_model(index);
		else if (strcmp("ALL",str2) == 0)
		    NSL_SYSTEM->enable_all_model();
		else
		    NSL_SYSTEM->enable_model(str2);
	}
	else if (strcmp("module",str1) == 0)
	{
	        if (nsl_get_str(in_file,str2) == 0)
		    NSL_MODULE->enable();
		else if ((index = atoi(str2)) != 0)
		    NSL_MODEL->enable_module(index);
		else if (strcmp("ALL",str2) == 0)
		    NSL_MODEL->enable_all_module();
		else
		    NSL_MODEL->enable_module(str2);
	    }
	else if (strcmp("layer",str1) == 0)
	{
	        if (nsl_get_str(in_file,str2) == 0)
		    NSL_LAYER->enable();
		else if ((index = atoi(str2)) != 0)
		    NSL_MODEL->enable_layer(index);
		else if (strcmp("ALL",str2) == 0)
		    NSL_MODEL->enable_all_layer();
		else
		    NSL_MODEL->enable_layer(str2);
	}
	else if (strcmp("input_layer",str1) == 0)
	{
	        if (nsl_get_str(in_file,str2) == 0)
		    NSL_INPUT_LAYER->enable();
		else if ((index = atoi(str2)) != 0)
		    NSL_MODEL->enable_layer(index); // input
		else if (strcmp("ALL",str2) == 0)
		    NSL_MODEL->enable_all_layer(); // input
		else
		    NSL_MODEL->enable_layer(str2); // input
	}
	else if (strcmp("input_stim",str1) == 0)
	{
	        if (nsl_get_str(in_file,str2) == 0)
		    NSL_INPUT_STIM->enable();
/*		else if ((index = atoi(str2)) != 0)
		    NSL_INPUT_LAYER->enable_input_stim(index); // input
		else if (strcmp("ALL",str2) == 0)
		    NSL_INPUT_LAYER->enable_all_input_stim(); // input
*/		else
//		    NSL_INPUT_LAYER->enable_input_stim(str2); // input
			cmd_out("Currently unavailable option: ",str2);
	}
	else if (strcmp("window_interface",str1) == 0)
	{
	}
	else if (strcmp("display_frame",str1) == 0)
	{
	}
	else if (strcmp("display_window",str1) == 0)
	{
	}
	else if (strcmp("display_canvas",str1) == 0)
	{
	}
	else if (strcmp("display_panel",str1) == 0)
	{
	        if (nsl_get_str(in_file,str2) == 0)
		    show_frame(NSL_PANEL->get_frame());
/*		else if ((index = atoi(str2)) != 0)
		    NSL_FRAME->enable_display_panel(index);
		else if (strcmp("ALL",str2) == 0)
		    NSL_FRAME->enable_all_display_panel();
		else
		    NSL_FRAME->enable_display_panel(str2);
*/	}
	else if (strcmp("canvas_item",str1) == 0)
	{
	    if (nsl_get_str(in_file,str2) == 0)
	    {
		if (NSL_CANVAS_ITEM != NULL)
		    NSL_CANVAS_ITEM->enable();
		else
		    cmd_error("NULL NSL_CANVAS_ITEM");
	    }
	    else if ((index = atoi(str2)) != 0)
		NSL_CANVAS->enable_item(index);
	    else if (strcmp("ALL",str2) == 0)
		NSL_CANVAS->enable_all_item();
	    else
		NSL_CANVAS->enable_item(str2);
	}
	else if (strcmp("panel_item",str1) == 0)
	{
	    if (nsl_get_str(in_file,str2) == 0)
		NSL_PANEL_ITEM->enable();
	}
	else
	{
		cmd_error("*** enable command ***");
		cmd_error("unknown modifier: ",str1);
//		status = 1;
	}

	return status;
}
int nsl_disable_command(istream& in_file)
{
	int status = 1;
	string str1,str2;
	int index;

	if (nsl_get_str(in_file,str1) == 0)
	{
		nsl_help_disable();
		return 1;
	}

	set_global_pointers();

	if (strcmp("network",str1) == 0 || strcmp("model",str1) == 0)
	{
	        if (nsl_get_str(in_file,str2) == 0)
		    NSL_MODEL->disable();
		else if ((index = atoi(str2)) != 0)
		    NSL_SYSTEM->disable_model(index);
		else if (strcmp("ALL",str2) == 0)
		    NSL_SYSTEM->disable_all_model();
		else
		    NSL_SYSTEM->disable_model(str2);
	}
	else if (strcmp("module",str1) == 0)
	{
	        if (nsl_get_str(in_file,str2) == 0)
		    NSL_MODULE->disable();
		else if ((index = atoi(str2)) != 0)
		    NSL_MODEL->disable_module(index);
		else if (strcmp("ALL",str2) == 0)
		    NSL_MODEL->disable_all_module();
		else
		    NSL_MODEL->disable_module(str2);
	}
	else if (strcmp("layer",str1) == 0)
	{
	        if (nsl_get_str(in_file,str2) == 0)
		    NSL_LAYER->disable();
		else if ((index = atoi(str2)) != 0)
		    NSL_MODEL->disable_layer(index);
		else if (strcmp("ALL",str2) == 0)
		    NSL_MODEL->disable_all_layer();
		else
		    NSL_MODEL->disable_layer(str2);
	}
	else if (strcmp("input_layer",str1) == 0)
	{
	        if (nsl_get_str(in_file,str2) == 0)
		    NSL_INPUT_LAYER->disable();
		else if ((index = atoi(str2)) != 0)
		    NSL_MODEL->disable_layer(index); // input
		else if (strcmp("ALL",str2) == 0)
		    NSL_MODEL->disable_all_layer(); // input
		else
		    NSL_MODEL->disable_layer(str2); // input
	}
	else if (strcmp("input_stim",str1) == 0)
	{
	        if (nsl_get_str(in_file,str2) == 0)
		    NSL_INPUT_STIM->disable();
/*		else if ((index = atoi(str2)) != 0)
		    NSL_INPUT_LAYER->disable_input_stim(index); // input
		else if (strcmp("ALL",str2) == 0)
		    NSL_INPUT_LAYER->disable_all_input_stim(); // input
*/		else
//		    NSL_INPUT_LAYER->disable_input_stim(str2); // input
			cmd_out("Currently unavailable option: ",str2);
	}
	else if (strcmp("window_interface",str1) == 0)
	{
	}
	else if (strcmp("display_frame",str1) == 0)
	{
	}
	else if (strcmp("display_window",str1) == 0)
	{
	}
	else if (strcmp("display_canvas",str1) == 0)
	{
	}
	else if (strcmp("display_panel",str1) == 0)
	{
	        if (nsl_get_str(in_file,str2) == 0)
		    unshow_frame(NSL_PANEL->get_frame());
/*		else if ((index = atoi(str2)) != 0)
		    NSL_FRAME->disable_display_panel(index);
		else if (strcmp("ALL",str2) == 0)
		    NSL_FRAME->disable_all_display_panel();
		else
		    NSL_FRAME->disable_display_panel(str2);
*/	}
	else if (strcmp("canvas_item",str1) == 0)
	{
	    if (nsl_get_str(in_file,str2) == 0)
	    {
		if (NSL_CANVAS_ITEM != NULL)
		    NSL_CANVAS_ITEM->disable();
		else
		    cmd_error("NULL NSL_CANVAS_ITEM");
	    }
	    else if ((index = atoi(str2)) != 0)
		NSL_CANVAS->disable_item(index);
	    else if (strcmp("ALL",str2) == 0)
		NSL_CANVAS->disable_all_item();
	    else
		NSL_CANVAS->disable_item(str2);
	}
	else if (strcmp("panel_item",str1) == 0)
	{
	    if (nsl_get_str(in_file,str2) == 0)
		NSL_PANEL_ITEM->disable();
	}
	else
	{
		cmd_error("*** enable command ***");
		cmd_error("unknown modifier: ",str1);
//		status = 1;
	}

	return status;
}
