/**************************************************************/
/*                                                            */
/*                   nsl_command_set.c                        */
/*                                                            */
/* by Alfredo Weitzenfeld 				      */
/*                                                            */
/**************************************************************/

# include "nsl_command.h"

extern nsl_buffer* NSL_BUFFER;
extern NSL_DIFF DIFF;

void set_input_dx(float val)
{
	int type = NSL_INPUT_LAYER->get_layer_type();
	switch (type)
	{
	   case 1:
		break;
	   case 2:
		((nsl_input_vector*) NSL_INPUT_LAYER)->set_dx(val);
		break;
	   case 3:
		((nsl_input_matrix*) NSL_INPUT_LAYER)->set_dx(val);
		break;
	   default:
		cmd_out("**** set input dx ****");
		cmd_out("Unknown type",type);
	}
}
void set_input_dy(float val)
{
	int type = NSL_INPUT_LAYER->get_layer_type();
	switch (type)
	{
	   case 1:
		break;
	   case 2:
		break;
	   case 3:
		((nsl_input_matrix*) NSL_INPUT_LAYER)->set_dy(val);
		break;
	   default:
		cmd_out("**** set input dy ****");
		cmd_out("Unknown type",type);
	}
}
void set_input_xz(int val)
{
	int type = NSL_INPUT_LAYER->get_layer_type();
	switch (type)
	{
	   case 1:
		break;
	   case 2:
		((nsl_input_vector*) NSL_INPUT_LAYER)->set_xz(val);
		break;
	   case 3:
		((nsl_input_matrix*) NSL_INPUT_LAYER)->set_xz(val);
		break;
	   default:
		cmd_out("**** set input xz ****");
		cmd_out("Unknown type",type);
	}
}
void set_input_yz(int val)
{
	int type = NSL_INPUT_LAYER->get_layer_type();
	switch (type)
	{
	   case 1:
		break;
	   case 2:
		break;
	   case 3:
		((nsl_input_matrix*) NSL_INPUT_LAYER)->set_yz(val);
		break;
	   default:
		cmd_out("**** set input yz ****");
		cmd_out("Unknown type",type);
	}
}
void nsl_help_set_network() // model
{
	cout << "\nset 'type' ['option']\n";
	cout << "\n";
	cout << "set buffer for object type\n";
	cout << "\n";
	cout << "    type    \t  option  \t    description\n";
	cout << "------------\t----------\t-------------------\n";
	cout << "\n";
/*	cout << "VERBOSE_FG\t\t\tverbose processing (intepreter only)\n";
	cout << "DEBUG_FG\t\t\tVERY verbose processing (interpreter only)\n";
	cout << "SIM_FG\t\t\t\tverbose processing\n";
	cout << "SIM_TIME_FG\t\t\tverbose time processing\n";
*/	cout << "network\t\t\t\tnetwork (model) name\n";
	cout << "\t\t'name'\t\tnetwork by name\n";
	cout << "\t\t'index'\t\tnetwork by index\n";
	cout << "end_time\t\t\tsimulation end time\n";
	cout << "\t\t'value'\t\tfloat value\n";
	cout << "current_time\t\t\tsimulation current time\n";
	cout << "\t\t'value'\t\tfloat value\n";
	cout << "delta\t\t\t\tsimulation time step\n";
	cout << "\t\t'value'\t\tfloat value\n";
	cout << "integration\t\t\tintegration method\n";
	cout << "\t\t'method'\t\tEULER numerical method\n";
}
void nsl_help_set_module()
{
	cout << "\nset 'type' ['option']\n";
	cout << "\n";
	cout << "set buffer for object type\n";
	cout << "\n";
	cout << "    type    \t  option  \t    description\n";
	cout << "------------\t----------\t-------------------\n";
	cout << "\n";
	cout << "module\t\t\t\tmodule name\n";
	cout << "\t\t'name'\t\tmodule by name\n";
	cout << "\t\t'index'\t\tmodule by index\n";
/*	cout << "module_able_fg\t\t\tverbose processing\n";
	cout << "\t\t'value'\t\tON/OFF\n";
	cout << "module_verbose_fg\t\tverbose processing\n";
	cout << "\t\t'value'\t\tON/OFF\n";
*/
}
void nsl_help_set_layer()
{
	cout << "\nset 'type' ['option']\n";
	cout << "\n";
	cout << "set buffer for object type\n";
	cout << "\n";
	cout << "    type    \t  option  \t    description\n";
	cout << "------------\t----------\t-------------------\n";
	cout << "\n";
	cout << "layer\t\t\t\tlayer name\n";
	cout << "\t\t'name'\t\tlayer by name\n";
	cout << "\t\t'index'\t\tlayer by index\n";
	cout << "data_value\t\t'name'\tlayer name followed by all values\n";
	cout << "all_data_value";
	cout << "\t\t'name'\tlayer name followed by single data value\n";
	cout << "\t\t\t\tfor whole layer\n";
	cout << "elem_i\t\t'index'\t\tlayer row element index\n";
	cout << "elem_j\t\t'index'\t\tlayer col element index\n";
	cout << "elem_data_value";
	cout << "\t\t'name'\tlayer name followed by elem data value\n";
	cout << "\t\t\t\tused in conjunction with elem_i and elem_j\n";
}
void nsl_help_set_input()
{
	cout << "\nset 'type' ['option']\n";
	cout << "\n";
	cout << "set buffer for object type\n";
	cout << "\n";
	cout << "    type    \t  option  \t    description\n";
	cout << "------------\t----------\t-------------------\n";
	cout << "\n";
	cout << "input_layer\t\t\tinput layer name\n";
	cout << "\t\t'name'\t\tinput layer by name\n";
	cout << "\t\t'index'\t\tinput layer by index\n";
	cout << "input_xz\t\t\tzero coordinate x layer element,\n";
	cout << "\t\t\t\tdefault is 0\n";
	cout << "input_yz\t\t\tzero coordinate y layer element,\n";
	cout << "\t\t\t\tdefault is 0\n";
	cout << "input_dx\t\t\tdistance between adjacent x layer elements,\n";
	cout << "\t\t\t\tdefault is 1.0\n";
	cout << "input_dy\t\t\tdistance between adjacent y layer elements,\n";
	cout << "\t\t\t\tdefault is 1.0\n";
}
void nsl_help_set_stim()
{
	cout << "\nset 'type' ['option']\n";
	cout << "\n";
	cout << "set buffer for object type\n";
	cout << "\n";
	cout << "    type    \t  option  \t    description\n";
	cout << "------------\t----------\t-------------------\n";
	cout << "\n";
	cout << "stim\t\t\t\tcurrent stim\n";
	cout << "\t\t'index'\t\tstim by index index\n";
	cout << "stim_index\t\t'index'\t\tstim by index index\n";
	cout << "stim_dx\t\t'value'\t\tstim x size (width), default is 1.0\n";
	cout << "stim_dy\t\t'value'\t\tstim y size (height), default is 1.0\n";
	cout << "stim_xc\t\t'value'\t\tstim x center initial location,\n";
	cout << "\t\t\t\tdefault is 0.0\n";
	cout << "stim_yc\t\t'value'\t\tstim y center initial location\n";
	cout << "\t\t\t\tdefault is 0.0\n";
	cout << "stim_x0\t\t'value'\t\tstim x left initial location,\n";
	cout << "\t\t\t\tdefault is 0.0\n";
	cout << "stim_y0\t\t'value'\t\tstim y top initial location\n";
	cout << "\t\t\t\tdefault is 0.0\n";
	cout << "stim_spec\t\t'type'\tCENTER or BLOCK\n";
	cout << "stim_type\t\t'type'\tBLOCK or ICON\n";
	cout << "stim_val\t\t'value'\tstim value (BLOCK), default is 1.0\n";
	cout << "stim_mat\t\t'matrix'\tstim icon matrix (ICON)\n";
	cout << "stim_vx\t\t'value'\t\tstim x velocity, positive going right\n";
	cout << "stim_vy\t\t'value'\t\tstim y velocity, positive going up\n";
	cout << "stim_t0\t\t'value'\t\tbeginning of stim time interval\n";
	cout << "stim_t1\t\t'value'\t\tend of stim time interval\n";
}

void nsl_help_set_frame()
{
	cout << "\nset 'type' ['option']\n";
	cout << "\n";
	cout << "set buffer for display frame\n";
	cout << "\n";
	cout << "    type    \t  option  \t    description\n";
	cout << "------------\t----------\t-------------------\n";
	cout << "\n";
	cout << "display_frame\t\t\tframe name\n";
	cout << "\t\t'name'\t\tdisplay frame by name\n";
	cout << "\t\t'index'\t\tdisplay frame by index\n";
	cout << "frame_name\t'name'\t\tframe name\n";
	cout << "frame_X0\t\t\tframe left corner location in pixels\n";
	cout << "frame_Y0\t\t\tframe top corner location in pixels\n";
	cout << "frame_width\t\t\tframe width\n";
	cout << "frame_height\t\t\tframe height\n";
	cout << "frame_auto_resize\t\t'ON' - auto resize, 'OFF' otherwise\n";
	cout << "frame_cols\t\t\tnumber of window columns when resizing\n";
	cout << "frame_rows\t\t\tnumber of window rows when resizing\n";
}
void nsl_help_set_window()
{
	cout << "\nset 'type' ['option']\n";
	cout << "\n";
	cout << "set buffer for object type\n";
	cout << "\n";
	cout << "    type    \t\t  option  \t    description\n";
	cout << "------------\t\t----------\t-------------------\n";
	cout << "\n";
	cout << "display_window\t\t\t\twindow name\n";
	cout << "\t\t\t'name'\t\tdisplay window by name\n";
	cout << "\t\t\t'index'\t\tdisplay window by index\n";
	cout << "window_name\t\t'name'\t\twindow name\n";
	cout << "window_X0\t\t'n'\twindow relative left corner location\n";
	cout << "window_Y0\t\t'n'\twindow relative top corner location\n";
	cout << "window_width\t\t'n'\twindow width\n";
	cout << "window_height\t\t'n'\twindow height\n";
	cout << "window_step\t\t'n'\twindow update step (default 1)\n";
	cout << "window_overlay\t\t'n'\twindow graph overlay (default OFF)\n";
	cout << "window_layer\t\t'name'\tlayer to be displayed\n";
	cout << "window_layer_i0\t\t'n'\tlayer i0 display element\n";
	cout << "window_layer_i1\t\t'n'\tlayer i1 display element\n";
	cout << "window_layer_j0\t\t'n'\tlayer j0 display element\n";
	cout << "window_layer_j1\t\t'n'\tlayer j1 display element\n";
	cout << "window_graph\t\t'type'\tgraph type to be displayed\n";
	cout << "\t\t\t\t\tarea_level_graph\n";
	cout << "\t\t\t\t\tspatial_graph_2\n";
	cout << "\t\t\t\t\tspatial_graph_3\n";
	cout << "\t\t\t\t\ttemporal_graph\n";
	cout << "\t\t\t\t\tuser_graph\n";
	cout << "window_wymin\t\t'value'\tgraph min y value\n";
	cout << "window_wymax\t\t'value'\tgraph max y value\n";
	cout << "window_t0\t\t'value'\ttemporal graph starting time\n";
	cout << "window_t1\t\t'value'\ttemporal graph ending time\n";
	cout << "window_wymin\t\t'value'\tinteractive mouse input value\n";
	cout << "window_pulse\t\t'type'\tdraw using 'pulse' type";
	cout << "lines (ON/OFF)\n";
	cout << "window_hidden_line\t\t'type'\tdraw 3D with hidden";
	cout << "line removal (ON/OFF)\n";
	cout << "window_vec_type\t\t\t'horizontal' or 'vertical' drawing\n";
	cout << "window_ax\t\t'value'\t3D graph x reference point\n";
	cout << "window_by\t\t'value'\t3D graph y reference point\n";
	cout << "window_cz\t\t'value'\t3D graph z reference point\n";
	cout << "window_sx\t\t'value'\t3D graph x scale\n";
	cout << "window_sy\t\t'value'\t3D graph y scale\n";
	cout << "window_sz\t\t'value'\t3D graph z scale\n";
}
void nsl_help_set_dump() // rasterfile
{
	cout << "\nset 'type' ['option']\n";
	cout << "\n";
	cout << "set buffer for object type\n";
	cout << "\n";
	cout << "    type    \t  option  \t    description\n";
	cout << "------------\t----------\t-------------------\n";
	cout << "\n";
	cout << "dump_file\t\t\tname of dump file\n";
	cout << "dump_type\t\t\t'ras', 'ps', or 'printer'\n";
	cout << "dump_ras_index\t\t\tindex for 'ras' file name\n";
	cout << "dump_ps_index\t\t\tindex for 'ps' file name\n";
	cout << "dump_auto_name_fg\t\t'ON' - generate different file name\n";
	cout << "\t\t\t\t\tfor every win_dump\n";
	cout << "\t\t\t\t'OFF' - otherwise\n";
}
void nsl_help_set_file() // datafile ??
{
	cout << "\nset 'type' ['option']\n";
	cout << "\n";
	cout << "set buffer for object type\n";
	cout << "\n";
	cout << "    type    \t  option  \t    description\n";
	cout << "------------\t----------\t-------------------\n";
	cout << "\n";
	cout << "file_index\t\t\tindex for data file name\n";
	cout << "file_auto_name\t\t\t'ON' - generate NSL file name\n";
	cout << "\t\t\t\t\tfor every data file\n";
	cout << "\t\t\t\t'OFF' - user specified file name\n";
	cout << "file_type\t\t\t'input' or 'output'\n";
//	cout << "file_name\t\t\tdata file name\n";
}
void nsl_help_set()
{
	cout << "\nset 'type' ['option']\n";
	cout << "\n";
	cout << "set buffer for objects in the next categories\n";
	cout << "(also available 'help set ['type']')\n";
	cout << "\n";
	cout << "    type    \t    description\n";
	cout << "------------\t-------------------\n";
	cout << "\n";
	cout << "network\t\tnetwork information\n";
	cout << "module\t\tmodule information\n";
	cout << "layer\t\tlayer information\n";
	cout << "input_layer\tinput layer information\n";
	cout << "input_stim\tstimulus information\n";
	cout << "\n";
	cout << "display_frame\tdisplay frame information\n";
	cout << "display_window\tdisplay window information\n";
//	cout << "display_canvas\t\tdisplay canvas information\n";
//	cout << "display_panel\t\tdisplay panel information\n";
	cout << "window_item\twindow item information\n";
	cout << "canvas_item\tcanvas item information\n";
	cout << "panel_item\tpanel item information\n";
//	cout << "rasterfile\traster file information\n";
//	cout << "datafile\tdata file information\n";
}
void set_command_data(nsl_data* data,char* str2)
{
	string str;
	int ival;
	float fval;

	int type = data->get_data_type();
	if (strcmp(str2,"") != 0)
	{
//		in >> *data; // error checking needed
		switch (type)
		{
		case 1:
			ival = atoi(str2);
			data->set_value(ival);
			break;
		case 2:
			fval = atof(str2);
			data->set_value(fval);
			break;
		case 3:
			data->set_value(str2);
			break;
		default:
			cmd_error("Bad data type: ",type);
		}
//		data->set_able_fg(1);
	}
}
char* get_command_data(nsl_data* data)
{
	string str;
	char* sval;
	int ival;
	float fval;

	int type = data->get_data_type();
	
	switch (type)
	{
	case 1:
		ival = data->get_int_value();
		sprintf(str,"%d",ival);
		break;
	case 2:
		fval = data->get_float_value();
		sprintf(str,"%f",fval);
		break;
	case 3:
		sval = data->get_str_value();
		sprintf(str,"%s",sval);
		break;
	default:
		cmd_error("Bad data type: ",type);
	}

	return str;
}
void set_global_pointers()
{
	if (NSL_SYSTEM == NULL)
	{
		cmd_error("NULL NSL_SYS");
		exit(-1);
	}
	if (NSL_MODEL == NULL)
	{
//		cmd_error("NULL NSL_MODEL");
		if ((NSL_MODEL = NSL_SYSTEM->get_model(1)) == NULL)
		{
			cmd_error("NO MODELS in the system, exiting....");
			exit(-1);
		}
//		else
//			cmd_error("setting NSL_MODEL with index 1");
	}
}
void set_global_model_pointers()
{
	if ((NSL_MODULE = NSL_MODEL->get_module(1)) == NULL)
	{
		cmd_error("NO MODULES in the network, exiting....");
		exit(-1);
	}
//	else
//		cmd_error("setting NSL_MODULE with index 1");
	if ((NSL_LAYER = NSL_MODEL->get_layer()) == NULL)
	{
		cmd_error("NO LAYERS in the network, exiting....");
		exit(-1);
	}
//	else
//		cmd_error("setting NSL_LAYER with index 1");
	if ((NSL_INPUT_LAYER = NSL_MODEL->get_input_layer()) == NULL)
	{
//		cmd_error("NO INPUT LAYERS in the network....");
//		exit(-1);
	}
//	else
//		cmd_error("setting NSL_LAYER with index 1");
}
void set_global_display_pointers()
{
	if (NSL_INTERFACE != NULL)
	{
	    if (NSL_FRAME == NULL)
	    	;
	    if (NSL_WINDOW == NULL)
		;
	    if (NSL_CANVAS == NULL)
		;
	    if (NSL_PANEL == NULL)
		;
	    if (NSL_WINDOW_ITEM == NULL)
		;
	    if (NSL_CANVAS_ITEM == NULL)
		;
	    if (NSL_PANEL_ITEM == NULL)
		;
	}
}
void set_window_defaults(display_window* win)
{
	display_canvas* canvas;
	display_panel* panel;

	if (win != NULL)
	{
	    if ((canvas = win->get_window_canvas()) != NULL)
	    {
		NSL_CANVAS = canvas;
		CMD_BUF->set_general_data("canvas_index",win->get_index());
		CMD_BUF->set_general_data("canvas_name",win->get_name());
	    }
	    else
	    	cmd_error("Cannot open display canvas");

	    if ((panel = win->get_control_panel()) != NULL)
	    {
		NSL_PANEL = panel;
		CMD_BUF->set_general_data("panel_index",win->get_index());
		CMD_BUF->set_general_data("panel_name",win->get_name());
	    }
	    else
	    	cmd_error("Cannot open display canvas");
	}
	else
	    cmd_error("NULL display window");
}
void set_sim_defaults()
{
	string str;
	nsl_data* data;
	char* file_name;

	int index = 0;

	if ((data = CMD_BUF->get_data("network_name")) != 0)
		file_name = data->get_str_value();

	sprintf(str,"%s.%s.%d",file_name,"ras",index);
	CMD_BUF->set_general_data("dump_ras_file",str);
	sprintf(str,"%s.%s.%d",file_name,"ps",index);
	CMD_BUF->set_general_data("dump_ps_file",str);
}
int nsl_set_command(istream& in_file)
{
	string str1,str2,str3;
	int status = 1,i = 0,j = 0,ival=0;
	float fval=0;
	nsl_data* data;
	nsl_model* model;
	nsl_module* module;
	nsl_layer* layer;
	nsl_stim_obj* stim;
	display_frame* frame;
	display_window* window;
	display_canvas* canvas;
	display_panel* panel;
	nsl_data* window_item;
	canvas_object* canvas_item;
	panel_object* panel_item;

	if (nsl_get_str(in_file,str1) == 0)
	{
		nsl_help_set();
		return 1;
	}

// old compatibility
	if (strcmp(str1,"window_layer_x0") == 0)
		strcpy(str1,"window_layer_i0");
	else if (strcmp(str1,"window_layer_x1") == 0)
		strcpy(str1,"window_layer_i1");
	else if (strcmp(str1,"window_layer_y0") == 0)
		strcpy(str1,"window_layer_j0");
	else if (strcmp(str1,"window_layer_y1") == 0)
		strcpy(str1,"window_layer_j1");

	int attr = 1;
	int flag = 0;
	int index = 0;
	if ((attr = nsl_get_str(in_file,str2)) == 0)
	{
	    if ((data = CMD_BUF->get_data(str1)) != NULL)
	    {
		strcpy(str2,get_command_data(data));
		(*NSL_BUFFER) << str2;
	    }
	    else
		cmd_error("Bad attribute");
	    return 1;
	}
	else
	{
	   index = atoi(str2); // set index value just in case
	   ival = index;
	   fval = atof(str2);
	}

// check for these and also update buffer
	if (strcmp(str1,"VERBOSE_FG") == 0)
	{
	    if (strcmp(str2,"ON") == 0)
		VERBOSE_FG = ON;
	    else if (strcmp(str2,"OFF") == 0)
		VERBOSE_FG = OFF;
	    else
		cmd_out("// Unknown VERBOSE_FG modifier: ",str2);
	}
	else if (strcmp(str1,"DEBUG_FG") == 0)
	{
	    if (strcmp(str2,"ON") == 0)
		DEBUG_FG = ON;
	    else if (strcmp(str2,"OFF") == 0)
	      DEBUG_FG =OFF; 
	    else
		cmd_out("// Unknown DEBUG_FG modifier: ",str2);
	}
	else if (strcmp(str1,"SIM_FG") == 0)
	{
	  if (NSL_MODEL != NULL)
	  {
	    if (strcmp(str2,"ON") == 0)
		NSL_MODEL->set_all_verbose_fg(ON);
	    else if (strcmp(str2,"OFF") == 0)
		NSL_MODEL->set_all_verbose_fg(OFF);
	    else
		cmd_out("// Unknown SIM_FG modifier: ",str2);
	  }
	  else
	   	cmd_error("NULL NSL_MODEL");
	}
	else if (strcmp(str1,"SIM_TIME_FG") == 0)
	{
	  if (NSL_MODEL != NULL)
	  {
		if (strcmp(str2,"ON") == 0)
		    NSL_MODEL->set_print_time_fg(ON);
		else if (strcmp(str2,"OFF") == 0)
		    NSL_MODEL->set_print_time_fg(OFF);
		else
		    cmd_out("// Unknown SIM_TIME_FG modifier: ",str2);
	  }
	  else
	   	cmd_error("NULL NSL_MODEL");
	}
	else if (strcmp(str1,"integration") == 0)
	{
	        if (strcmp(str2,"INTERPOLATION") == 0)
		{
	    	    CMD_BUF->set_general_data("integration",str2);
		    DIFF.set_type(2);
		}
	        else if (strcmp(str2,"EULER") == 0)
		{
	    	    CMD_BUF->set_general_data("integration",str2);
		    DIFF.set_type(1);
		}
	        else 
	    	    cmd_error("Unknown integration: ",str2);
	}
	else if (strcmp(str1,"end_time") == 0)
	{
	    if (NSL_MODEL != NULL)
		NSL_MODEL->set_end_time(fval);
	    else
		cmd_error("NULL NSL_MODEL");
	}
	else if (strcmp(str1,"current_time") == 0)
	{
	    if (NSL_MODEL != NULL)
		NSL_MODEL->set_time(fval);
	    else
		cmd_error("NULL NSL_MODEL");
	}
	else if (strcmp(str1,"delta") == 0)
	{
	    if (NSL_MODEL != NULL)
		NSL_MODEL->set_delta(fval);
	    else
		cmd_error("NULL NSL_MODEL");
	}

// check for these or update buffer
	if (strcmp(str1,"elem_data_value") == 0) // elem_i,elem_j
	{
	    if ((data = CMD_BUF->get_data("elem_i")) != NULL)
		i = data->get_int_value();
	    else 
		cmd_out("NULL 'elem_i' buffer data");
	    if ((data = CMD_BUF->get_data("elem_j")) != NULL)
		j = data->get_int_value();
	    else 
		cmd_out("NULL 'elem_j' buffer data");

	    if ((layer = NSL_MODEL->get_layer(str2)) != NULL)
	     	CMD_BUF->set_layer_elem_data_value(in_file,layer,i,j);
	    else
		cmd_error("Unknown layer: ",str2);
	}
	else if (strcmp(str1,"all_data_value") == 0)
	{
	    if ((layer = NSL_MODEL->get_layer(str2)) != NULL)
	      	CMD_BUF->set_layer_all_data_value(in_file,layer);
	    else
		cmd_error("Unknown layer: ",str2);
	}
	else if (strcmp(str1,"data_value") == 0)
	{
	    if ((layer = NSL_MODEL->get_layer(str2)) != NULL)
	      	CMD_BUF->set_layer_data_value(in_file,layer);
	    else
		cmd_error("Unknown layer: ",str2);
	}
	else if (strcmp(str1,"network") == 0 || strcmp(str1,"model") == 0)
	{
	  if (NSL_SYSTEM != NULL)
	  {
	    if (index > 0 && 
		(model = NSL_SYSTEM->get_model(index)) != NULL)
	    {
		NSL_MODEL = model;
		set_global_model_pointers(); // set default model pointers
		NSL_SYSTEM->disable_all_model();
		NSL_MODEL->enable();
		CMD_BUF->set_general_data("network_index",model->get_index());
		CMD_BUF->set_general_data("network_name",model->get_name());
		CMD_BUF->set_general_data("network",model->get_name());
	    }
	    else if ((model = NSL_SYSTEM->get_model(str2)) != NULL)
	    {
		NSL_MODEL = model;
		set_global_model_pointers(); // set default model pointers
		NSL_SYSTEM->disable_all_model();
		NSL_MODEL->enable();
		CMD_BUF->set_general_data("network_index",model->get_index());
		CMD_BUF->set_general_data("network_name",model->get_name());
		CMD_BUF->set_general_data("network",model->get_name());
	    }
	    else
	    {
		status = -1;
		cmd_error("// Unknown network: ",str2);
	    }
	  }
	  else
	   	cmd_error("NULL NSL_SYSTEM");
	}
	else if (strcmp(str1,"network_name") == 0 
		|| strcmp(str1,"model_name") == 0)
	{
	  if (NSL_SYSTEM != NULL)
	  {
	    if ((model = NSL_SYSTEM->get_model(str2)) != NULL)
	    {
		NSL_MODEL = model;
		set_global_model_pointers(); // set default model pointers
		NSL_SYSTEM->disable_all_model();
		NSL_MODEL->enable();
		CMD_BUF->set_general_data("network_index",model->get_index());
		CMD_BUF->set_general_data("network_name",model->get_name());
		CMD_BUF->set_general_data("network",model->get_name());
		set_sim_defaults();
	    }
	    else
	    {
		status = -1;
		cmd_error("// Unknown network: ",str2);
	    }
	  }
	  else
	   	cmd_error("NULL NSL_SYSTEM");
	}
	else if (strcmp(str1,"network_index") == 0 
		|| strcmp(str1,"model_index") == 0)
	{
	  if (NSL_SYSTEM != NULL)
	  {
	    if (index > 0 && 
		(model = NSL_SYSTEM->get_model(index)) != NULL)
	    {
		NSL_MODEL = model;
		set_global_model_pointers(); // set default model pointers
		NSL_SYSTEM->disable_all_model();
		NSL_MODEL->enable();
		CMD_BUF->set_general_data("network_index",model->get_index());
		CMD_BUF->set_general_data("network_name",model->get_name());
		CMD_BUF->set_general_data("network",model->get_name());
	    }
	    else
	    {
		status = -1;
		cmd_error("// Unknown network: ",index);
	    }
	  }
	  else
	   	cmd_error("NULL NSL_SYSTEM");
	}
	else if (strcmp(str1,"module") == 0)
	{
	  if (NSL_MODEL != NULL)
	  {
	    if (index > 0 && 
		(module = NSL_MODEL->get_module(index)) != NULL)
	    {
		NSL_MODULE = module;
		CMD_BUF->set_general_data("module_index",module->get_index());
		CMD_BUF->set_general_data("module_name",module->get_name());
		CMD_BUF->set_general_data("module",module->get_name());
	    }
	    else if ((module = NSL_MODEL->get_module(str2)) != NULL)
	    {
		NSL_MODULE = module;
		CMD_BUF->set_general_data("module_index",module->get_index());
		CMD_BUF->set_general_data("module_name",module->get_name());
		CMD_BUF->set_general_data("module",module->get_name());
	    }
	    else
		cmd_error("// Unknown module: ",str2);
	  }
	  else
	   	cmd_error("NULL NSL_MODEL");
	}
	else if (strcmp(str1,"module_name") == 0)
	{
	  if (NSL_MODEL != NULL)
	  {
	    if ((module = NSL_MODEL->get_module(str2)) != NULL)
	    {
		NSL_MODULE = module;
		CMD_BUF->set_general_data("module_index",module->get_index());
		CMD_BUF->set_general_data("module_name",module->get_name());
		CMD_BUF->set_general_data("module",module->get_name());
	    }
	    else
		cmd_error("// Unknown module: ",str2);
	  }
	  else
	   	cmd_error("NULL NSL_MODEL");
	}
	else if (strcmp(str1,"module_index") == 0)
	{
	  if (NSL_MODEL != NULL)
	  {
	    if (index > 0 && 
		(module = NSL_MODEL->get_module(index)) != NULL)
	    {
		NSL_MODULE = module;
		CMD_BUF->set_general_data("module_index",module->get_index());
		CMD_BUF->set_general_data("module_name",module->get_name());
		CMD_BUF->set_general_data("module",module->get_name());
	    }
	    else
		cmd_error("// Unknown module: ",index);
	  }
	  else
	   	cmd_error("NULL NSL_MODEL");
	}
	else if (strcmp(str1,"layer") == 0)
	{
	  if (NSL_MODEL != NULL)
	  {
	    if ((layer = NSL_MODEL->get_layer(str2)) != NULL)
	    {
		NSL_LAYER = layer;
		CMD_BUF->set_general_data("layer_index",layer->get_index());
		CMD_BUF->set_general_data("layer_name",layer->get_name());
		CMD_BUF->set_general_data("layer",layer->get_name());
	    }
	    else if (index > 0 && 
		(layer = NSL_MODEL->get_layer(index)) != NULL)
	    {
		NSL_LAYER = layer;
		CMD_BUF->set_general_data("layer_index",layer->get_index());
		CMD_BUF->set_general_data("layer_name",layer->get_name());
		CMD_BUF->set_general_data("layer",layer->get_name());
	    }
	    else
		cmd_error("// Unknown layer: ",str2);
	  }
	  else
	   	cmd_error("NULL NSL_MODEL");
	}
	else if (strcmp(str1,"layer_name") == 0)
	{
	  if (NSL_MODEL != NULL)
	  {
	    if ((layer = NSL_MODEL->get_layer(str2)) != NULL)
	    {
		NSL_LAYER = layer;
		CMD_BUF->set_general_data("layer_index",layer->get_index());
		CMD_BUF->set_general_data("layer_name",layer->get_name());
		CMD_BUF->set_general_data("layer",layer->get_name());
	    }
	    else
		cmd_error("// Unknown layer: ",str2);
	  }
	  else
	   	cmd_error("NULL NSL_MODEL");
	}
	else if (strcmp(str1,"layer_index") == 0)
	{
	  if (NSL_MODEL != NULL)
	  {
	    if (index > 0 && 
		(layer = NSL_MODEL->get_layer(index)) != NULL)
	    {
		NSL_LAYER = layer;
		CMD_BUF->set_general_data("layer_index",layer->get_index());
		CMD_BUF->set_general_data("layer_name",layer->get_name());
		CMD_BUF->set_general_data("layer",layer->get_name());
	    }
	    else
		cmd_error("// Unknown layer: ",index);
	  }
	  else
	   	cmd_error("NULL NSL_MODEL");
	}
	else if (strcmp(str1,"input_layer") == 0)
	{
	  if (NSL_MODEL != NULL)
	  {
	    if ((layer = NSL_MODEL->get_input_layer(str2)) != NULL)
	    {
	      NSL_INPUT_LAYER = layer;
	      CMD_BUF->set_general_data("input_layer_index",layer->get_index());
	      CMD_BUF->set_general_data("input_layer_name",layer->get_name());
	      CMD_BUF->set_general_data("input_layer",layer->get_name());
	    }
	    else if (index > 0 && 
		(layer = NSL_MODEL->get_input_layer(index)) != NULL)
	    {
	      NSL_INPUT_LAYER = layer;
	      CMD_BUF->set_general_data("input_layer_index",layer->get_index());
	      CMD_BUF->set_general_data("input_layer_name",layer->get_name());
	      CMD_BUF->set_general_data("input_layer",layer->get_name());
	    }
	    else
		cmd_error("// Unknown input_layer: ",str2);
	  }
	  else
	   	cmd_error("NULL NSL_MODEL");
	}
	else if (strcmp(str1,"input_layer_name") == 0)
	{
	  if (NSL_MODEL != NULL)
	  {
	    if ((layer = NSL_MODEL->get_input_layer(str2)) != NULL)
	    {
	      NSL_INPUT_LAYER = layer;
	      CMD_BUF->set_general_data("input_layer_index",layer->get_index());
	      CMD_BUF->set_general_data("input_layer_name",layer->get_name());
	      CMD_BUF->set_general_data("input_layer",layer->get_name());
	    }
	    else
		cmd_error("// Unknown input_layer: ",str2);
	  }
	  else
	   	cmd_error("NULL NSL_MODEL");
	}
	else if (strcmp(str1,"input_layer_index") == 0)
	{
	  if (NSL_MODEL != NULL)
	  {
	    if (index > 0 && 
		(layer = NSL_MODEL->get_input_layer(index)) != NULL)
	    {
	      NSL_INPUT_LAYER = layer;
	      CMD_BUF->set_general_data("input_layer_index",layer->get_index());
	      CMD_BUF->set_general_data("input_layer_name",layer->get_name());
	      CMD_BUF->set_general_data("input_layer",layer->get_name());
	    }
	    else
		cmd_error("// Unknown input_layer: ",index);
	  }
	  else
	   	cmd_error("NULL NSL_MODEL");
	}
	else if (strcmp(str1,"input_dx") == 0)
	{
	  if (NSL_MODEL != NULL)
	  {
	    if (NSL_INPUT_LAYER != NULL)
	    {
	        CMD_BUF->set_general_data("input_dx",fval);
		set_input_dx(fval);
	    }
	    else
		cmd_error("// NULL NSL_INPUT_LAYER");
	  }
	  else
	   	cmd_error("NULL NSL_MODEL");
	}
	else if (strcmp(str1,"input_dy") == 0)
	{
	  if (NSL_MODEL != NULL)
	  {
	    if (NSL_INPUT_LAYER != NULL)
	    {
	        CMD_BUF->set_general_data("input_dy",fval);
		set_input_dy(fval);
	    }
	    else
		cmd_error("// NULL NSL_INPUT_LAYER");
	  }
	  else
	   	cmd_error("NULL NSL_MODEL");
	}
	else if (strcmp(str1,"input_xz") == 0)
	{
	  if (NSL_MODEL != NULL)
	  {
	    if (NSL_INPUT_LAYER != NULL)
	    {
	        CMD_BUF->set_general_data("input_xz",ival);
		set_input_xz(ival);
	    }
	    else
		cmd_error("// NULL NSL_INPUT_LAYER");
	  }
	  else
	   	cmd_error("NULL NSL_MODEL");
	}
	else if (strcmp(str1,"input_yz") == 0)
	{
	  if (NSL_MODEL != NULL)
	  {
	    if (NSL_INPUT_LAYER != NULL)
	    {
	        CMD_BUF->set_general_data("input_yz",ival);
		set_input_yz(ival);
	    }
	    else
		cmd_error("// NULL NSL_INPUT_LAYER");
	  }
	  else
	   	cmd_error("NULL NSL_MODEL");
	}
	else if (strcmp(str1,"stim") == 0)
	{
	  if (NSL_INPUT_LAYER != NULL)
	  {
	    if (index > 0 && 
		(stim = NSL_INPUT_LAYER->get_stim(index)) != NULL)
	    {
		NSL_INPUT_STIM = stim;
		CMD_BUF->set_general_data("stim",stim->get_index());
		CMD_BUF->set_general_data("stim_index",stim->get_index());
//		CMD_BUF->set_general_data("stim_name",stim->get_name());
	    }
/*	    else if ((stim = NSL_INPUT_LAYER->get_stim(str2)) != NULL)
	    {
		NSL_INPUT_STIM = stim;
		CMD_BUF->set_general_data("stim",stim->get_index());
		CMD_BUF->set_general_data("stim_index",stim->get_index());
//		CMD_BUF->set_general_data("stim_name",stim->get_name());
	    }
*/	    else
	    {
		CMD_BUF->set_general_data("stim",0);
		CMD_BUF->set_general_data("stim_index",0);
//		CMD_BUF->set_general_data("stim_name",str2);
		cmd_error("Unknown stim index: ",index);
	    }
	  }
	  else
	   	cmd_error("NULL NSL_INPUT_LAYER");
	}
/*	else if (strcmp(str1,"stim_name") == 0)
	{
	  if (NSL_INPUT_LAYER != NULL)
	  {
	    if ((stim = NSL_INPUT_LAYER->get_stim(str2)) != NULL)
	    {
		NSL_INPUT_STIM = stim;
		CMD_BUF->set_general_data("stim",stim->get_index());
		CMD_BUF->set_general_data("stim_index",stim->get_index());
//		CMD_BUF->set_general_data("stim_name",stim->get_name());
	    }
	    else
	    {
		CMD_BUF->set_general_data("stim",0);
		CMD_BUF->set_general_data("stim_index",0);
//		CMD_BUF->set_general_data("stim_name",str2);
	    }
	  }
	  else
	   	cmd_error("NULL NSL_INPUT_LAYER");
	}
*/	else if (strcmp(str1,"stim_index") == 0)
	{
	  if (NSL_INPUT_LAYER != NULL)
	  {
	    if (index > 0 && 
		(stim = NSL_INPUT_LAYER->get_stim(index)) != NULL)
	    {
		NSL_INPUT_STIM = stim;
		CMD_BUF->set_general_data("stim",stim->get_index());
		CMD_BUF->set_general_data("stim_index",stim->get_index());
//		CMD_BUF->set_general_data("stim_name",stim->get_name());
	    }
	    else
		cmd_error("// Unknown stim: ",str2);
	  }
	  else
	   	cmd_error("NULL NSL_INPUT_LAYER");
	}
	else if (strcmp(str1,"stim_x0") == 0)
	{
	  if (NSL_INPUT_LAYER != NULL)
	  {
		NSL_INPUT_STIM = stim;
		CMD_BUF->set_general_data("stim_x0",fval);
		CMD_BUF->set_general_data("stim_spec","CORNER");
	  }
	  else
	   	cmd_error("NULL NSL_INPUT_LAYER");
	}
	else if (strcmp(str1,"stim_y0") == 0)
	{
	  if (NSL_INPUT_LAYER != NULL)
	  {
		NSL_INPUT_STIM = stim;
		CMD_BUF->set_general_data("stim_y0",fval);
		CMD_BUF->set_general_data("stim_spec","CORNER");
	  }
	  else
	   	cmd_error("NULL NSL_INPUT_LAYER");
	}
	else if (strcmp(str1,"stim_xc") == 0)
	{
	  if (NSL_INPUT_LAYER != NULL)
	  {
		NSL_INPUT_STIM = stim;
		CMD_BUF->set_general_data("stim_xc",fval);
		CMD_BUF->set_general_data("stim_spec","CENTER");
	  }
	  else
	   	cmd_error("NULL NSL_INPUT_LAYER");
	}
	else if (strcmp(str1,"stim_yc") == 0)
	{
	  if (NSL_INPUT_LAYER != NULL)
	  {
		NSL_INPUT_STIM = stim;
		CMD_BUF->set_general_data("stim_yc",fval);
		CMD_BUF->set_general_data("stim_spec","CENTER");
	  }
	  else
	   	cmd_error("NULL NSL_INPUT_LAYER");
	}
// window interface
	else if (strcmp(str1,"frame") == 0 || strcmp(str1,"display_frame") == 0)
	{
	  if (NSL_INTERFACE != NULL)
	  {
	    if (index > 0 && 
		(frame = NSL_INTERFACE->get_display_frame(index)) != NULL)
	    {
		NSL_FRAME = frame;
		CMD_BUF->set_general_data("frame_index",index);
		CMD_BUF->set_general_data("frame_name",frame->get_name());
		CMD_BUF->set_general_data("display_frame",frame->get_name());
	    }
	    else if ((frame = NSL_INTERFACE->get_display_frame(str2)) != NULL)
	    {
		NSL_FRAME = frame;
		CMD_BUF->set_general_data("frame_index",frame->get_index());
		CMD_BUF->set_general_data("frame_name",str2);
		CMD_BUF->set_general_data("display_frame",frame->get_name());
	    }
	    else
	    {
		CMD_BUF->set_general_data("display_frame",str2);
		CMD_BUF->set_general_data("frame_name",str2);
		CMD_BUF->set_general_data("frame_index",0);
	    }
	  }
	  else
	   	cmd_error("NULL NSL_INTERFACE");
	}
	else if (strcmp(str1,"frame_name") == 0)
	{
	  if (NSL_INTERFACE != NULL)
	  {
	    if ((frame = NSL_INTERFACE->get_display_frame(str2)) != NULL)
	    {
		NSL_FRAME = frame;
		CMD_BUF->set_general_data("frame_index",frame->get_index());
		CMD_BUF->set_general_data("frame_name",str2);
		CMD_BUF->set_general_data("display_frame",str2);
	    }
	    else
	    {
		CMD_BUF->set_general_data("display_frame",str2);
		CMD_BUF->set_general_data("frame_name",str2);
		CMD_BUF->set_general_data("frame_index",0);
	    }
	  }
	  else
	   	cmd_error("NULL NSL_INTERFACE");
	}
	else if (strcmp(str1,"frame_index") == 0)
	{
	  if (NSL_INTERFACE != NULL)
	  {
	    if (index > 0 && 
		(frame = NSL_INTERFACE->get_display_frame(index)) != NULL)
	    {
		NSL_FRAME = frame;
		CMD_BUF->set_general_data("frame_index",index);
		CMD_BUF->set_general_data("frame_name",frame->get_name());
		CMD_BUF->set_general_data("display_frame",frame->get_name());
	    }
	    else
		cmd_error("// Cannot open display frame: ",index);
	  }
	  else
	   	cmd_error("NULL NSL_INTERFACE");
	}
	else if (strcmp(str1,"window") == 0 || strcmp(str1,"display_window") == 0)
	{
	  if (NSL_FRAME != NULL)
	  {
	    if (index > 0 && 
		(window = NSL_FRAME->get_display_window(index)) != NULL)
	    {
		NSL_WINDOW = window;
		NSL_CANVAS = window->get_window_canvas();
		NSL_PANEL = window->get_control_panel();
		CMD_BUF->set_general_data("window_index",index);
		CMD_BUF->set_general_data("window_name",window->get_name());
		CMD_BUF->set_general_data("display_window",window->get_name());
		set_window_defaults(NSL_WINDOW);
	    }
	    else if ((window = NSL_FRAME->get_display_window(str2)) != NULL)
	    {
		NSL_WINDOW = window;
		NSL_CANVAS = window->get_window_canvas();
		NSL_PANEL = window->get_control_panel();
		CMD_BUF->set_general_data("window_index",window->get_index());
		CMD_BUF->set_general_data("window_name",str2);
		CMD_BUF->set_general_data("display_window",str2);
		set_window_defaults(NSL_WINDOW);
	    }
	    else
	    {
		CMD_BUF->set_general_data("display_window",str2);
		CMD_BUF->set_general_data("window_name",str2);
		CMD_BUF->set_general_data("window_index",0);
	    }
	  }
	  else
	   	cmd_error("NULL NSL_FRAME");
	}
	else if (strcmp(str1,"window_name") == 0)
	{
	  if (NSL_FRAME != NULL)
	  {
	    if ((window = NSL_FRAME->get_display_window(str2)) != NULL)
	    {
		NSL_WINDOW = window;
		CMD_BUF->set_general_data("window_index",window->get_index());
		CMD_BUF->set_general_data("window_name",str2);
		CMD_BUF->set_general_data("display_window",str2);
		set_window_defaults(NSL_WINDOW);
	    }
	    else
	    {
		CMD_BUF->set_general_data("window_name",str2);
		CMD_BUF->set_general_data("display_window",str2);
		CMD_BUF->set_general_data("window_index",0);
	    }
	  }
	  else
	   	cmd_error("NULL NSL_FRAME");
	}
	else if (strcmp(str1,"window_index") == 0)
	{
	  if (NSL_FRAME != NULL)
	  {
	    if (index > 0 && 
		(window = NSL_FRAME->get_display_window(index)) != NULL)
	    {
		NSL_WINDOW = window;
		CMD_BUF->set_general_data("window_index",index);
		CMD_BUF->set_general_data("window_name",window->get_name());
		CMD_BUF->set_general_data("display_window",window->get_name());
		set_window_defaults(NSL_WINDOW);
	    }
	    else
		cmd_error("// Cannot open display window: ",index);
	  }
	  else
	   	cmd_error("NULL NSL_FRAME");
	}
	else if (strcmp(str1,"window_layer") == 0) // reset layer sizes
	{
	        CMD_BUF->set_general_data("window_layer",str2);
	        CMD_BUF->update_layer_buffer(str2);
	}
	else if (strcmp(str1,"canvas") == 0 || strcmp(str1,"display_canvas") == 0)
	{
		cmd_error("Use 'set window'");
	}
	else if (strcmp(str1,"panel") == 0 || strcmp(str1,"display_panel") == 0)
	{
		cmd_error("Use 'set window'");
	}
	else if (strcmp(str1,"window_item") == 0) 
	{
	  if (NSL_WINDOW != NULL)
	  {
	    if (index > 0 &&
		(window_item = NSL_WINDOW->get_window_data(index)) != NULL)
	    {
		NSL_WINDOW_ITEM = window_item;
		CMD_BUF->set_general_data("window_item_index",index);
		CMD_BUF->set_general_data("window_item_name",
			window_item->get_name());
	    }
	    else if ((window_item = NSL_WINDOW->get_window_data(str2)) != NULL)
	    {
		NSL_WINDOW_ITEM = window_item;
		CMD_BUF->set_general_data("window_item_name",str2);
		CMD_BUF->set_general_data("window_item_index",
			window_item->get_index());
	    }
	    else
	    {
		CMD_BUF->set_general_data("window_name",str2);
		CMD_BUF->set_general_data("window_index",0);
	    }
	  }
	  else
	   	cmd_error("NULL NSL_WINDOW");
	}
	else if (strcmp(str1,"window_item_name") == 0)
	{
	  if (NSL_WINDOW != NULL)
	  {
	    if ((window_item = NSL_WINDOW->get_window_data(str2)) != NULL)
	    {
		NSL_WINDOW_ITEM = window_item;
		CMD_BUF->set_general_data("window_item_name",str2);
		CMD_BUF->set_general_data("window_item_index",
			window_item->get_index());
	    }
	    else
	    {
		CMD_BUF->set_general_data("window_name",str2);
		CMD_BUF->set_general_data("window_index",0);
	    }
	  }
	  else
	   	cmd_error("NULL NSL_WINDOW");
	}
	else if (strcmp(str1,"window_item_index") == 0)
	{
	  if (NSL_WINDOW != NULL)
	  {
	    if (index > 0 &&
		(window_item = NSL_WINDOW->get_window_data(index)) != NULL)
	    {
		NSL_WINDOW_ITEM = window_item;
		CMD_BUF->set_general_data("window_item_index",index);
		CMD_BUF->set_general_data("window_item_name",
			window_item->get_name());
	    }
	    else
		cmd_error("// Cannot open window item: ",index);
	  }
	  else
	   	cmd_error("NULL NSL_WINDOW");
	}	
	else if (strcmp(str1,"canvas_item") == 0) 
	{
	  if (NSL_CANVAS != NULL)
	  {
	    if (index > 0 &&
		(canvas_item = NSL_CANVAS->get_canvas_item(index)) != NULL)
	    {
		NSL_CANVAS_ITEM = canvas_item;
		CMD_BUF->set_general_data("canvas_item_index",index);
		CMD_BUF->set_on_off_data("canvas_item_enable",
			canvas_item->get_able_fg());
		CMD_BUF->set_general_data("canvas_item_wx",canvas_item->get_wx());
		CMD_BUF->set_general_data("canvas_item_wy",canvas_item->get_wy());
	    }
	    else
		cmd_error("// Cannot set canvas item: ",index);
	  }
	  else
	   	cmd_error("NULL NSL_CANVAS");
	}
	else if (strcmp(str1,"canvas_item_index") == 0)
	{
	  if (NSL_CANVAS != NULL)
	  {
	    if (index > 0 &&
		(canvas_item = NSL_CANVAS->get_canvas_item(index)) != NULL)
	    {
		NSL_CANVAS_ITEM = canvas_item;
		CMD_BUF->set_general_data("canvas_item_index",index);
		CMD_BUF->set_on_off_data("canvas_item_enable",
			canvas_item->get_able_fg());
		CMD_BUF->set_general_data("canvas_item_wx",canvas_item->get_wx());
		CMD_BUF->set_general_data("canvas_item_wy",canvas_item->get_wy());
	    }
	    else
		cmd_error("// Cannot set canvas item: ",index);
	  }
	  else
	   	cmd_error("NULL NSL_CANVAS");
	}
	else if (strcmp(str1,"panel_item") == 0) 
	{
	  if (NSL_PANEL != NULL)
	  {
	    if (index > 0 && 
		(panel_item = NSL_PANEL->get_panel_item(index)) != NULL)
	    {
		NSL_PANEL_ITEM = panel_item;
		CMD_BUF->set_general_data("panel_item_index",index);
		CMD_BUF->set_on_off_data("panel_item_enable",
			panel_item->get_able_fg());
		CMD_BUF->set_general_data("panel_item_wx",panel_item->get_wx());
		CMD_BUF->set_general_data("panel_item_wy",panel_item->get_wy());
	    }
	    else
		cmd_error("// Cannot set panel item: ",index);
	  }
	  else
	   	cmd_error("NULL NSL_PANEL");
	}
	else if (strcmp(str1,"panel_item_index") == 0)
	{
	  if (NSL_PANEL != NULL)
	  {
	    if (index > 0 && 
		(panel_item = NSL_PANEL->get_panel_item(index)) != NULL)
	    {
		NSL_PANEL_ITEM = panel_item;
		CMD_BUF->set_general_data("panel_item_index",index);
		CMD_BUF->set_on_off_data("panel_item_enable",
			panel_item->get_able_fg());
		CMD_BUF->set_general_data("panel_item_wx",panel_item->get_wx());
		CMD_BUF->set_general_data("panel_item_wy",panel_item->get_wy());
	    }
	    else
		cmd_error("// Cannot set panel item: ",index);
	  }
	  else
	   	cmd_error("NULL NSL_PANEL");
	}
	else if ((data = CMD_BUF->get_data(str1)) != NULL)
		set_command_data(data,str2);
	else
		cmd_error("// Unknown 'set' modifier: ",str1);

	return status;
}
