//
//	nsl_data_library.c
//

# include "nsl.h"
# include "nsl_headers.h"

extern nsl_model* NSL_MODEL;

// data threshold functions

// NSLstep

// if a > 0 then 1 else 0

nsl_data NSLstep(nsl_data& a)
{
	nsl_data c;
	if (a.v > 0)
		c.v = 1.0;
	else
		c.v = 0;
	return c;
}

// if a > b then 1 else 0

nsl_data NSLstep(nsl_data& a,num_type b)
{
	nsl_data c;
	if (a.v > b)
		c.v = 1.0;
	else
		c.v = 0;
	return c;
}
nsl_data NSLstep(nsl_data& a,nsl_data& b)
{
	nsl_data c;
	if (a.v > b.v)
		c.v = 1.0;
	else
		c.v = 0;
	return c;
}
nsl_data NSLstep(nsl_data& a,num_type b,num_type y1,num_type y2)
{
	nsl_data c;
	if (a.v > b)
		c.v = y2;
	else
		c.v = y1;
	return c;
}
nsl_data NSLstep(nsl_data& a,nsl_data& b,nsl_data& y1,nsl_data& y2)
{
	nsl_data c;
	if (a.v > b.v)
		c.v = y2.v;
	else
		c.v = y1.v;
	return c;
}

// NSLramp

// if a > 0 then a else 0

nsl_data NSLramp(nsl_data& a) 
{
	nsl_data c;
	if (a.v > (num_type) 0)
		c.v = a.v;
	else
		c.v = 0;
	return c;
}

// if a > b then a - b else 0

nsl_data NSLramp(nsl_data& a,num_type b)
{
	nsl_data c;
	if (a.v > b)
		c.v = a.v - b;
	else
		c.v = 0;
	return c;
}
nsl_data NSLramp(nsl_data& a,nsl_data& b)
{
	nsl_data c;
	if (a.v > b.v)
		c.v = a.v - b.v;
	else
		c.v = 0;
	return c;
}
nsl_data NSLramp(nsl_data& a,num_type b,num_type y1,num_type y2)
{
	nsl_data c;
	if (a.v > b)
		c.v = a.v - b + y2;
	else
		c.v = y1;
	return c;
}
nsl_data NSLramp(nsl_data& a,nsl_data& b,nsl_data& y1,nsl_data& y2)
{
	nsl_data c;
	if (a.v > b.v)
		c.v = a.v - b.v + y2.v;
	else
		c.v = y1.v;
	return c;
}

// NSLsat

nsl_data NSLsat(nsl_data& a)
{
	nsl_data c;
	if (a.v > 1.0)
		c.v = 1.0;
	else if (a.v > 0)
		c.v = a.v;
	else
		c.v = 0;
	return c;
}
nsl_data NSLsat(nsl_data& a,num_type x1,num_type x2)
{
	nsl_data c;
	if (a.v > x2)
		c.v = 1.0;
	else if (a.v > x1)
		c.v = (a.v-x1)/(x2-x1);
	else
		c.v = 0;
	return c;
}
nsl_data NSLsat(nsl_data& a,nsl_data& x1,nsl_data& x2)
{
	nsl_data c;
	if (a.v > x2.v)
		c.v = 1.0;
	else if (a.v > x1.v)
		c.v = (a.v-x1.v)/(x2.v-x1.v);
	else
		c.v = 0;
	return c;
}
nsl_data NSLsat(nsl_data& a,num_type x1,num_type x2,num_type y1,num_type y2)
{
	nsl_data c;
	if (a.v > x2)
	    c.v = y2;
	else if (a.v > x1)
	    c.v = (y2-y1)*(a.v-x1)/(x2-x1) + y1;
	else
	    c.v = y1;
	return c;
}
nsl_data NSLsat(nsl_data& a,nsl_data& x1,nsl_data& x2,nsl_data& y1,nsl_data& y2)
{
	nsl_data c;
	if (a.v > x2.v)
	    c.v = y2.v;
	else if (a.v > x1.v)
	    c.v = (y2.v-y1.v)*(a.v-x1.v)/(x2.v-x1.v) + y1.v;
	else
	    c.v = y1.v;
	return c;
}

// NSLsigma

nsl_data NSLsigma(nsl_data& a)
{
	nsl_data c;
	if (a.v > 1.0)
		c.v = 1.0;
	else if (a.v > 0)
		c.v = pow(a.v,2.0)*(3.0-2.0*a.v);
	else
		c.v = 0;
	return c;
}
nsl_data NSLsigma(nsl_data& a,num_type x1,num_type x2)
{
	nsl_data c;
        num_type sigma = (a.v-x1)/(x2-x1);
	if (a.v > x2)
	    c.v = 1.0;
	else if (a.v > x1)
	    c.v = pow(sigma,2.0)*(3.0-2.0*sigma);
	else
	    c.v = 0;
	return c;
}
nsl_data NSLsigma(nsl_data& a,nsl_data& x1,nsl_data& x2)
{
	nsl_data c;
        num_type sigma = (a.v-x1.v)/(x2.v-x1.v);
	if (a.v > x2.v)
	    c.v = 1.0;
	else if (a.v > x1.v)
	    c.v = pow(sigma,2.0)*(3.0-2.0*sigma);
	else
	    c.v = 0;
	return c;
}
nsl_data NSLsigma(nsl_data& a,num_type x1,num_type x2,
	num_type y1,num_type y2)
{
	nsl_data c;
        num_type sigma = (a.v-x1)/(x2-x1);
	if (a.v > x2)
	    c.v = y2;
	else if (a.v > x1)
	    c.v = (y2-y1)*pow(sigma,2.0)*(3.0-2.0*sigma) + y1;
	else
	    c.v = y1;
	return c;
}
nsl_data NSLsigma(nsl_data& a,nsl_data& x1,nsl_data& x2,
	nsl_data& y1,nsl_data& y2)
{
	nsl_data c;
        num_type sigma = (a.v-x1.v)/(x2.v-x1.v);
	if (a.v > x2.v)
	    c.v = y2.v;
	else if (a.v > x1.v)
	    c.v = (y2.v-y1.v)*pow(sigma,2.0)*(3.0-2.0*sigma) + y1.v;
	else
	    c.v = y1.v;
	return c;
}

// NSLmax

// if a > b then a else 0

nsl_data NSLmax(nsl_data& a,nsl_data& b)
{
	nsl_data c;
	if (a.v > b.v)
	    c.v = a.v;
	else
	    c.v = b.v;
	return c;
}
nsl_data NSLmax(nsl_data& a,num_type b)
{
	nsl_data c;
	if (a.v > b)
	    c.v = a.v;
	else
	    c.v = b;
	return c;
}

// NSLmin

// if a < b then a else 0

nsl_data NSLmin(nsl_data& a,nsl_data& b)
{
	nsl_data c;
	if (a.v < b.v)
	    c.v = a.v;
	else
	    c.v = b.v;
	return c;
}
nsl_data NSLmin(nsl_data& a,num_type b)
{
	nsl_data c;
	if (a.v < b)
	    c.v = a.v;
	else
	    c.v = b;
	return c;
}

// NSLeuler method

nsl_data NSLeuler(nsl_data& V,nsl_data& S,nsl_data& tm)
{
	float dt;

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm.get_data() == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm.get_data());
		cmd_error_exit("Bad Integration Parameters");
	}

	return (nsl_data) (V + dt/tm*S);
}
nsl_data NSLeuler(nsl_data& V,nsl_data& S,num_type tm)
{
	float dt;

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm);
		cmd_error_exit("Bad Integration Parameters");
	}

	return (nsl_data) (V + dt/tm*S);
}
nsl_data NSLeuler(nsl_data& V,num_type S,nsl_data& tm)
{
	float dt;

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm.get_data() == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm.get_data());
		cmd_error_exit("Bad Integration Parameters");
	}

	return (nsl_data) (V + dt/tm*S);
}
nsl_data NSLeuler(nsl_data& V,num_type S,num_type tm)
{
	float dt;

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm);
		cmd_error_exit("Bad Integration Parameters");
	}

	return (nsl_data) (V + dt/tm*S);
}
