//
//	nsl_input_matrix.c
//

# include "nsl.h"
# include "nsl_headers.h"

extern nsl_model* NSL_MODEL;

// nsl_input_matrix

nsl_input_matrix::nsl_input_matrix(char* str,int imax,int jmax) : (str,imax,jmax)
{
	h_type = 1; // input layer

    	xz = 0;
	yz = 0;
	dx = 1.0;
	dy = 1.0;

	overlap_fg = 0;

	stim_total = 0;
	stim_list = new nsl_stim_list;
}	
nsl_input_matrix::~nsl_input_matrix()
{
}

// input matrix assignment (input_matrix = matrix is NOT derived !!)

void nsl_input_matrix::operator=(nsl_input_matrix& a)
{
	if (a.imax != imax || a.jmax != jmax)
	{
		cmd_error("*** nsl_input_matrix::operator= ***");
		cmd_error("Exit with the following status");
		a.print_status(cerr);
		this->print_status(cerr);
		cmd_error_exit("Inconsistent matrix sizes");
	}

	for (int i = 0; i < imax; i++)
	   for (int j = 0; j < jmax; j++)
		v[i][j] = a.v[i][j];
}
void nsl_input_matrix::operator=(nsl_matrix& a)
{
	if (a.imax != imax || a.jmax != jmax)
	{
		cmd_error("*** nsl_input_matrix::operator= ***");
		cmd_error("Exit with the following status");
		a.print_status(cerr);
		this->print_status(cerr);
		cmd_error_exit("Inconsistent matrix sizes");
	}

	for (int i = 0; i < imax; i++)
	   for (int j = 0; j < jmax; j++)
		v[i][j] = a.v[i][j];
}
void nsl_input_matrix::operator=(nsl_data& a)
{
	int imax = get_imax();  
	int jmax = get_jmax();

	num_type val = a.get_value();

	for (int i = 0; i < imax; i++)
	   for (int j = 0; j < jmax; j++)
		v[i][j] = val;
}
void nsl_input_matrix::operator=(num_type a)
{
	int imax = get_imax();
	int jmax = get_jmax();

	for (int i = 0; i < imax; i++)
	   for (int j = 0; j < jmax; j++)
		v[i][j] = a;
}

void nsl_input_matrix::clear_stim()
{
	stim_total = 0;
	stim_list = new nsl_stim_list;
}	
void nsl_input_matrix::copy_stim(nsl_input_matrix& a)
{
	nsl_stim_list* l1;
	nsl_stim_list* l2;
	float val;

	if ((l1 = a.get_stim_list()) == NULL || 
		(l2 = this->get_stim_list()) == NULL)
	{
		cmd_error("*** nsl_input_matrix::copy_stim ***");
		cmd_error("Exit with the following status");
		cmd_error("NULL stimulus list");
	}
	else
	{
	   nsl_stim_iterator next1(*l1);
	   nsl_stim_obj* obj1;
	   nsl_stim_obj* obj2;
	
	   this->clear_stim();

	   while (obj1 = next1())
	   {
		obj2 = this->init_block_stim();

		val = ((nsl_block_matrix*) obj1)->get_dx();
		((nsl_block_matrix*) obj2)->set_dx(val);
		val = ((nsl_block_matrix*) obj1)->get_dy();
		((nsl_block_matrix*) obj2)->set_dy(val);

//		obj2 = this->init_icon_stim();

		val = ((nsl_stim_matrix*) obj1)->get_vx();
		((nsl_stim_matrix*) obj2)->set_vx(val);
		val = ((nsl_stim_matrix*) obj1)->get_vy();
		((nsl_stim_matrix*) obj2)->set_vy(val);
		val = ((nsl_stim_matrix*) obj1)->get_xc();
		((nsl_stim_matrix*) obj2)->set_xc(val);
		val = ((nsl_stim_matrix*) obj1)->get_yc();
		((nsl_stim_matrix*) obj2)->set_yc(val);
	   }
	}
}

nsl_block_matrix* nsl_input_matrix::init_block_stim()
{
	nsl_block_matrix* obj;
	if ((obj = new nsl_block_matrix(this)) != NULL)
	{
		obj->set_index(++stim_total);
		stim_list->append(obj);
	}
	else
	{
		cmd_error("*** nsl_input_matrix::init_block_stim ***");
		cmd_error("Unable to create block stim object");
	}

	return obj;
}
nsl_icon_matrix* nsl_input_matrix::init_icon_stim()
{
	nsl_icon_matrix* obj;
	if ((obj = new nsl_icon_matrix(this)) != NULL)
	{
		obj->set_index(++stim_total);
		stim_list->append(obj);
	}
	else
	{
		cmd_error("*** nsl_input_matrix::init_icon_stim ***");
		cmd_error("Unable to create icon stim object");
	}

	return obj;
}

void nsl_input_matrix::run()
{
	*this = 0;

	if (stim_list != NULL)
	{
	   nsl_stim_iterator next(*stim_list);
	   nsl_stim_obj* stim_obj;
	   while (stim_obj = next())
	   {
	    	if (stim_obj->get_able_fg() == 0)
			continue;

		stim_obj->run();
	   }
	}
	else
	   cmd_out("NULL 'stim_list'");
}

// print status

void nsl_input_matrix::print_layer_status(ostream& out)
{
	nsl_matrix::print_layer_status(out);

	out << "// stim_total: " << stim_total << "\n";
	out << "set input_layer " << name << "\n";
	out << "set input_xz " << xz << "\n";
	out << "set input_yz " << yz << "\n";
	out << "set input_dx " << dx << "\n";
	out << "set input_dy " << dy << "\n";
	if (overlap_fg == 1)
		out << "set overlap_fg ON\n";
	else
		out << "set overlap_fg OFF\n";

	out.flush();
}
void nsl_input_matrix::print_status(ostream& out)
{
	print_layer_status(out);
}
void nsl_input_matrix::print_stim(ostream& out)
{
	*this = 0;

	out << "reset stim_list\n";
	out.flush();

	if (stim_list != NULL)
	{
	   nsl_stim_iterator next(*stim_list);
	   nsl_stim_obj* stim_obj;
	   while (stim_obj = next())
	   {
		stim_obj->print_stim(out);
	   }
	}
	else
	   cmd_out("NULL 'stim_list'");
}
void nsl_input_matrix::print_stim(ostream& out,int i)
{
	*this = 0;

	out << "reset stim_list\n";
	out.flush();

	if (stim_list != NULL)
	{
	   nsl_stim_iterator next(*stim_list);
	   nsl_stim_obj* stim_obj;
	   while (stim_obj = next())
	   {
		if (stim_obj->get_index() == i)
			stim_obj->print_stim(out);
	   }
	}
	else
	   cmd_out("NULL 'stim_list'");
}
nsl_stim_obj* nsl_input_matrix::get_stim(int index)
{
	if (stim_list != NULL)
	{
	   nsl_stim_iterator next(*stim_list);
	   nsl_stim_obj* stim_obj;
	   while (stim_obj = next())
	   {
		if (stim_obj->get_index() == index)
			return stim_obj;
	   }
	   cmd_out("NULL stim_obj: ",index);
	}
	else
	   cmd_out("NULL 'stim_list'");

	return (nsl_stim_obj*) 0;
}
nsl_stim_obj* nsl_input_matrix::get_stim(char* str)
{
	if (stim_list != NULL)
	{
	   nsl_stim_iterator next(*stim_list);
	   nsl_stim_obj* stim_obj;
	   while (stim_obj = next())
	   {
		if (strcmp(stim_obj->get_name(),str) == 0)
			return stim_obj;
	   }
	   cmd_out("NULL stim_obj: ",str);
	}
	else
	   cmd_out("NULL 'stim_list'");

	return (nsl_stim_obj*) 0;
}

