//
//	nsl_layer.c
//

# include "nsl.h"
# include "nsl_headers.h"

nsl_layer* NSL_LAYER;
nsl_layer* NSL_INPUT_LAYER;
nsl_stim_obj* NSL_INPUT_STIM;

nsl_layer::nsl_layer(int i)
{
	strcpy(name,"");
	index = 0;
	layer_type = i;
	able_fg = 1;
}
nsl_layer::nsl_layer(char* str,int i)
{
	strcpy(name,str);
	index = 0;
	layer_type = i;
	able_fg = 1;
}
void nsl_layer::operator=(nsl_layer& a)
{
	if (DEBUG_FG == 1)
	{
		cout << "// nsl_layer::operator=\n";
		cout << "// type: " << this->get_layer_type() << ";\n";
		cout << "// " << this->get_name() << " = ";
		cout << a.get_name() << ";\n";
		cout.flush();
	}

	int type = this->get_layer_type();
	int atype = a.get_layer_type();

	switch (type)
	{
	case 1:
		if (atype == 1)
			(nsl_data&) (*this) = (nsl_data&) a;
		else
		{
			cmd_error("Bad equality types");
			this->print_status(cerr);
			a.print_status(cerr);
		}
		break;
	case 2:
		if (atype == 1)
			(nsl_vector&) (*this) = (nsl_data&) a;
		else if (atype == 2)
			(nsl_vector&) (*this) = (nsl_vector&) a;
		else
		{
			cmd_error("Bad equality types");
			this->print_status(cerr);
			a.print_status(cerr);
		}
		break;
	case 3:
		if (atype == 1)
			(nsl_matrix&) (*this) = (nsl_data&) a;
		else if (atype == 3)
			(nsl_matrix&) (*this) = (nsl_matrix&) a;
		else
		{
			cmd_error("Bad equality types");
			this->print_status(cerr);
			a.print_status(cerr);
		}
		break;
	default:
		cmd_error("*** nsl_layer=operator= ***");
		cmd_error("Unknown layer type: ",type);
	}
}
void nsl_layer::operator=(num_type a)
{
	int type = this->get_layer_type();

	switch (type)
	{
	case 1:
		(nsl_data&) (*this) = a;
		break;
	case 2:
		(nsl_vector&) (*this) = a;
		break;
	case 3:
		(nsl_matrix&) (*this) = a;
		break;
	default:
		cmd_error("*** nsl_layer=operator= ***");
		cmd_error("Unknown layer type: ",type);
	}
}
void nsl_layer::operator=(int a)
{
	int type = this->get_layer_type();

	switch (type)
	{
	case 1:
		(nsl_data&) (*this) = a;
		break;
	case 2:
		(nsl_vector&) (*this) = a;
		break;
	case 3:
		(nsl_matrix&) (*this) = a;
		break;
	default:
		cmd_error("*** nsl_layer=operator= ***");
		cmd_error("Unknown layer type: ",type);
	}
}
ostream& operator<<(ostream& out,nsl_layer& a)
{
	int type = a.get_layer_type();

	switch (type)
	{
	case 1:
		out << (nsl_data&) a;
		break;
	case 2:
		out << (nsl_vector&) a;
		break;
	case 3:
		out << (nsl_matrix&) a;
		break;
	default:
		cmd_error("*** operator<< ***");
		cmd_error("Unknown layer type: ",type);
	}

	return out;
}
istream& operator>>(istream& in,nsl_layer& a)
{
	if (!in)
		cmd_error("Read Error");

	int type = a.get_layer_type();

	switch (type)
	{
	case 1:
		in >> (nsl_data&) a;
		break;
	case 2:
		in >> (nsl_vector&) a;
		break;
	case 3:
		in >> (nsl_matrix&) a;
		break;
	default:
		cmd_error("*** operator>> ***");
		cmd_error("Unknown layer type: ",type);
	}

	return in;
}
void nsl_layer::print_layer_status(ostream& out)
{
	out << "\n";
	switch (layer_type)
	{
	case 1: // nsl_data
	    if (h_type == 1) // input_layer
		out << "// INPUT_DAT\n";
	    else // regular layer
		out << "// DATA\n";
	    break;
	case 2: // nsl_vector
	    if (h_type == 1) // input_layer
		out << "// INPUT_VEC\n";
	    else // regular layer
		out << "// VECTOR\n";
	    break;
	case 3: // nsl_matrix
	    if (h_type == 1) // input_layer
		out << "// INPUT_MAT\n";
	    else // regular layer
		out << "// MATRIX\n";
	    break;
	default:
		cmd_error("*** nsl_layer::print_status ***");
		cmd_error("Undefined layer type:",layer_type);
	}
//	out << "set layer_index " << index << "\n";
	out << "set layer_name " << name << "\n";
//	out << "set layer_type ";
//	out << "set layer_able "\n";
/*	if (able_fg == 1)
	   out << "ON\n";
	else // if (able_fg == 0)
	   out << "OFF\n";
*/	if (able_fg == 1)
	   out << "enable layer\n";
	else // if (able_fg == 0)
	   out << "disable layer\n";
	out.flush();
}
