//
//	nsl_matrix_conv_library.c
//

# include "nsl.h"
# include "nsl_headers.h"

// zero edge matrix convolution 

nsl_matrix NSLconv_zero(nsl_matrix& a,nsl_matrix& b)
{
	int saimax = a.get_imax();
	int sajmax = a.get_jmax();
	int sbimax = b.get_imax();
	int sbjmax = b.get_jmax();

	int simax = saimax + sbimax - 1;
	int sjmax = sajmax + sbjmax - 1;

	nsl_matrix c(sbimax,sbjmax);
	nsl_matrix d(simax,sjmax);

	d = d.put_sector(b,saimax/2,sajmax/2); 

	num_type** av = a.get_matrix();
	num_type** cv = c.get_matrix();
	num_type** dv = d.get_matrix();

	for (int i = 0; i < sbimax; i++)
	   for (int j = 0; j < sbjmax; j++)
	   {	
		num_type val = (num_type) 0;
		for (int m = 0; m < saimax; m++)
		   for (int n = 0; n < sajmax; n++)
			val = val + av[m][n] * dv[i+m][j+n];
		cv[i][j] = val;
	   }

	return c;
}
nsl_matrix NSLconv_zero(nsl_vector& a,nsl_matrix& b) // row,col vector
{
	int saimax = a.get_imax();
	int sbimax = b.get_imax();
	int sbjmax = b.get_jmax();

	int vec_type;
	int simax;
	int m;
	num_type val = 0;

	nsl_matrix c(sbimax,sbjmax);

	num_type* av = a.get_vector();
	num_type** cv = c.get_matrix();
	num_type** dv;

	if ((vec_type = a.get_vec_type()) == 0)
	{
		simax = saimax + sbjmax - 1;
		nsl_matrix d(sbimax,simax);
		d = d.put_sector(b,0,saimax/2); 
		dv = d.get_matrix();

		for (int i = 0; i < sbimax; i++)
		   for (int j = 0; j < sbjmax; j++)
		   {	
			val = 0;	
			for (m = 0; m < saimax; m++)
			    val = val + av[m] * dv[i][j+m];
			cv[i][j] = val;
		   }
	}
	else
	{
		simax = saimax + sbimax - 1;
		nsl_matrix d(simax,sbjmax);
		d = d.put_sector(b,saimax/2,0); 
		dv = d.get_matrix();

		for (int i = 0; i < sbimax; i++)
		   for (int j = 0; j < sbjmax; j++)
		   {	
			val = 0;
			for (m = 0; m < saimax; m++)
			    val = val + av[m] * dv[i+m][j];
			cv[i][j] = val;
		   }
	}

	return c;
}
nsl_matrix NSLconv_zero(nsl_matrix& b,nsl_vector& a) // same as previous
{
	int saimax = a.get_imax();
	int sbimax = b.get_imax();
	int sbjmax = b.get_jmax();

	int vec_type;
	int simax;
	int m;
	num_type val = 0;

	nsl_matrix c(sbimax,sbjmax);

	num_type* av = a.get_vector();
	num_type** cv = c.get_matrix();
	num_type** dv;

	if ((vec_type = a.get_vec_type()) == 0)
	{
		simax = saimax + sbjmax - 1;
		nsl_matrix d(sbimax,simax);
		d = d.put_sector(b,0,saimax/2); 
		dv = d.get_matrix();

		for (int i = 0; i < sbimax; i++)
		   for (int j = 0; j < sbjmax; j++)
		   {	
			val = 0;	
			for (m = 0; m < saimax; m++)
			    val = val + av[m] * dv[i][j+m];
			cv[i][j] = val;
		   }
	}
	else
	{
		simax = saimax + sbimax - 1;
		nsl_matrix d(simax,sbjmax);
		d = d.put_sector(b,saimax/2,0); 
		dv = d.get_matrix();

		for (int i = 0; i < sbimax; i++)
		   for (int j = 0; j < sbjmax; j++)
		   {	
			val = 0;
			for (m = 0; m < saimax; m++)
			    val = val + av[m] * dv[i+m][j];
			cv[i][j] = val;
		   }
	}

	return c;
}
nsl_matrix NSLconv_zero(nsl_matrix& a,nsl_data& b)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	nsl_matrix c(aimax,ajmax);

	num_type** av = a.get_matrix();
	num_type bv = b.get_data();
	num_type** cv = c.get_matrix();

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		cv[i][j] = av[i][j] * bv;

	return c;
}
nsl_matrix NSLconv_zero(nsl_data& a, nsl_matrix& b)
{
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	nsl_matrix c(bimax,bjmax);

	num_type av = a.get_data();
	num_type** bv = b.get_matrix();
	num_type** cv = c.get_matrix();

	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
		cv[i][j] = av * bv[i][j];

	return c;
}
nsl_matrix NSLconv_zero(nsl_matrix& a,num_type b)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	nsl_matrix c(aimax,ajmax);

	num_type** av = a.get_matrix();
	num_type** cv = c.get_matrix();

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		cv[i][j] = av[i][j] * b;

	return c;
}
nsl_matrix NSLconv_zero(num_type a, nsl_matrix& b)
{
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	nsl_matrix c(bimax,bjmax);

	num_type** bv = b.get_matrix();
	num_type** cv = c.get_matrix();

	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
		cv[i][j] = a * bv[i][j];

	return c;
}

// wrap around convolution

nsl_matrix NSLconv_wrap(nsl_matrix& a,nsl_matrix& b)
{
      	int saimax = a.get_imax();
	int sajmax = a.get_jmax();
      	int sbimax = b.get_imax();
	int sbjmax = b.get_jmax();

	int smi = saimax/2;
	int smj = sajmax/2;

 	int simax = saimax + sbimax - 1;
	int sjmax = sajmax + sbjmax - 1;

      	nsl_matrix c(sbimax,sbjmax);
      	nsl_matrix d(simax,sjmax);

      	d = d.put_sector(b,saimax/2,sajmax/2); 

	int i,j;

	num_type** av = a.get_matrix();
	num_type** bv = b.get_matrix();
	num_type** cv = c.get_matrix();
	num_type** dv = d.get_matrix();

      	for (i = smi-1; i >= 0; i--)
    	   for (j = smj; j < sbjmax+smj; j++)
 	   	dv[i][j] = bv[(i-smi+sbimax)%sbimax][j-smj];
      	for (i = smi+sbimax; i < simax; i++)
    	   for (j = smj; j < sbjmax+smj; j++)
	   	dv[i][j] = bv[(i-smi)%sbimax][j-smj];

    	for (j = smj-1; j >= 0; j--)
    	   for (i = smi; i < sbimax+smi; i++)
	   	dv[i][j] = bv[i-smi][(j-smj+sbjmax)%sbjmax];
     	for (j = smj+sbjmax; j < sjmax; j++)
    	   for (i = smi; i < sbimax+smi; i++)
	   	dv[i][j] = bv[i-smi][(j-smj)%sbjmax];

      	for (i = 0; i < sbimax; i++)
      	    for (j = 0; j < sbjmax; j++)
            {    
              	num_type val = (num_type) 0;
              	for (int m = 0; m < saimax; m++)
		   for (int n = 0; n < sajmax; n++)
			val = val + av[m][n] * dv[i+m][j+n];
		cv[i][j] = val;
            }
  
        return c;
}
nsl_matrix NSLconv_copy(nsl_matrix& a,nsl_matrix& b)
{
      	int saimax = a.get_imax();
	int sajmax = a.get_jmax();
      	int sbimax = b.get_imax();
	int sbjmax = b.get_jmax();

	int smi = saimax/2;
	int smj = sajmax/2;

 	int simax = saimax + sbimax - 1;
	int sjmax = sajmax + sbjmax - 1;

      	nsl_matrix c(sbimax,sbjmax);
      	nsl_matrix d(simax,sjmax);

      	d = d.put_sector(b,saimax/2,sajmax/2); 

	int i,j;

	num_type** av = a.get_matrix();
	num_type** bv = b.get_matrix();
	num_type** cv = c.get_matrix();
	num_type** dv = d.get_matrix();

      	for (i = smi-1; i >= 0; i--)
    	   for (j = smj; j < sbjmax+smj; j++)
 	   	dv[i][j] = bv[0][j-smj];
      	for (i = smi+sbimax; i < simax; i++)
    	   for (j = smj; j < sbjmax+smj; j++)
	   	dv[i][j] = bv[sbimax-1][j-smj];

    	for (j = smj-1; j >= 0; j--)
    	   for (i = smi; i < sbimax+smi; i++)
	   	dv[i][j] = bv[i-smi][0];
     	for (j = smj+sbjmax; j < sjmax; j++)
    	   for (i = smi; i < sbimax+smi; i++)
	   	dv[i][j] = bv[i-smi][sbjmax-1];

      	for (i = 0; i < sbimax; i++)
      	    for (j = 0; j < sbjmax; j++)
            {    
              	num_type val = (num_type) 0;
              	for (int m = 0; m < saimax; m++)
		   for (int n = 0; n < sajmax; n++)
			val = val + av[m][n] * dv[i+m][j+n];
		cv[i][j] = val;
            }
  
        return c;
}
