//
//	nsl_matrix_library.c
//

# include "nsl.h"
# include "nsl_headers.h"

extern nsl_model* NSL_MODEL;

// matrix threshold functions

// NSLstep

// if a > 0 then 1 else 0

nsl_matrix NSLstep(nsl_matrix& a)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > (num_type) 0)
			c.v[i][j] = (num_type) 1;
		else
			c.v[i][j] = 0;
	return c;
}

// if a > b then 1 else 0

nsl_matrix NSLstep(nsl_matrix& a,nsl_matrix& b)
{
	if (a.get_imax() != b.get_imax() || a.get_jmax() != b.get_jmax())
	{
		cmd_error("*** NSLstep ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("jmax1: ",a.get_jmax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error("jmax2: ",b.get_jmax());
		cmd_error_exit("bad matrix sector");
	}
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > b.v[i][j])
			c.v[i][j] = (num_type) 1;
		else
			c.v[i][j] = 0;
	return c;
}
nsl_matrix NSLstep(nsl_matrix& a,num_type b)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > b)
			c.v[i][j] = (num_type) 1;
		else
			c.v[i][j] = 0;
	return c;
}
nsl_matrix NSLstep(nsl_matrix& a,nsl_data& b)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > b.v)
			c.v[i][j] = (num_type) 1;
		else
			c.v[i][j] = 0;
	return c;
}

// if a > b then y2 else y1

nsl_matrix NSLstep(nsl_matrix& a,nsl_matrix& b,nsl_matrix& y1,nsl_matrix& y2)
{
	if (a.get_imax() != b.get_imax() || a.get_jmax() != b.get_jmax() ||
	    a.get_imax() != y1.get_imax() || a.get_jmax() != y1.get_jmax() ||
	    a.get_imax() != y2.get_imax() || a.get_jmax() != y2.get_jmax())
	{
		cmd_error("*** NSLstep ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("jmax1: ",a.get_jmax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error("jmax2: ",b.get_jmax());
		cmd_error("imax3: ",y1.get_imax());
		cmd_error("jmax3: ",y1.get_jmax());
		cmd_error("imax4: ",y2.get_imax());
		cmd_error("jmax4: ",y2.get_jmax());
		cmd_error_exit("bad matrix sector");
	}
	int s1 = b.get_imax();
	int s2 = b.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > b.v[i][j])
			c.v[i][j] = y2.v[i][j];
		else
			c.v[i][j] = y1.v[i][j];
	return c;
}
nsl_matrix NSLstep(nsl_matrix& a,nsl_matrix& b,nsl_data& y1,nsl_data& y2)
{
	if (a.get_imax() != b.get_imax() || a.get_jmax() != b.get_jmax())
	{
		cmd_error("*** NSLstep ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("jmax1: ",a.get_jmax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error("jmax2: ",b.get_jmax());
		cmd_error_exit("bad matrix sector");
	}
	int s1 = b.get_imax();
	int s2 = b.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > b.v[i][j])
			c.v[i][j] = y2.v;
		else
			c.v[i][j] = y1.v;
	return c;
}
nsl_matrix NSLstep(nsl_matrix& a,num_type b,num_type y1,num_type y2)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > b)
			c.v[i][j] = y2;
		else
			c.v[i][j] = y1;
	return c;
}
nsl_matrix NSLstep(nsl_matrix& a,nsl_data& b,nsl_data& y1,nsl_data& y2)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > b.v)
			c.v[i][j] = y2.v;
		else
			c.v[i][j] = y1.v;
	return c;
}

// NSLramp

// if a > 0 then a else 0

nsl_matrix NSLramp(nsl_matrix& a) 
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > (num_type) 0)
			c.v[i][j] = a.v[i][j];
		else
			c.v[i][j] = 0;
	return c;
}

// if a > b then a - b else 0

nsl_matrix NSLramp(nsl_matrix& a,nsl_matrix& b)
{
	if (a.get_imax() != b.get_imax() || a.get_jmax() != b.get_jmax())
	{
		cmd_error("*** NSLramp ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("jmax1: ",a.get_jmax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error("jmax2: ",b.get_jmax());
		cmd_error_exit("bad matrix sector");
	}
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > b.v[i][j])
			c.v[i][j] = a.v[i][j] - b.v[i][j];
		else
			c.v[i][j] = 0;
	return c;
}
nsl_matrix NSLramp(nsl_matrix& a,nsl_data& b)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > b.v)
			c.v[i][j] = a.v[i][j] - b.v;
		else
			c.v[i][j] = 0;
	return c;
}
nsl_matrix NSLramp(nsl_matrix& a,num_type b)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > b)
			c.v[i][j] = a.v[i][j] - b;
		else
			c.v[i][j] = 0;
	return c;
}

// if a > b then a - b + y2 else y1

nsl_matrix NSLramp(nsl_matrix& a,nsl_matrix& b,nsl_matrix& y1,nsl_matrix& y2)
{
	if (a.get_imax() != b.get_imax() || a.get_jmax() != b.get_jmax() ||
	    a.get_imax() != y1.get_imax() || a.get_jmax() != y1.get_jmax() ||
	    a.get_imax() != y2.get_imax() || a.get_jmax() != y2.get_jmax())
	{
		cmd_error("*** NSLramp ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("jmax1: ",a.get_jmax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error("jmax2: ",b.get_jmax());
		cmd_error("imax3: ",y1.get_imax());
		cmd_error("jmax3: ",y1.get_jmax());
		cmd_error("imax4: ",y2.get_imax());
		cmd_error("jmax4: ",y2.get_jmax());
		cmd_error_exit("bad matrix sector");
	}
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > b.v[i][j])
			c.v[i][j] = a.v[i][j] - b.v[i][j] + y2.v[i][j];
		else
			c.v[i][j] = y1.v[i][j];
	return c;
}
nsl_matrix NSLramp(nsl_matrix& a,nsl_matrix& b,num_type y1,num_type y2)
{
	if (a.get_imax() != b.get_imax() || a.get_jmax() != b.get_jmax())
	{
		cmd_error("*** NSLramp ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("jmax1: ",a.get_jmax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error("jmax2: ",b.get_jmax());
		cmd_error_exit("bad matrix sector");
	}
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > b.v[i][j])
			c.v[i][j] = a.v[i][j] - b.v[i][j] + y2;
		else
			c.v[i][j] = y1;
	return c;
}
nsl_matrix NSLramp(nsl_matrix& a,nsl_matrix& b,nsl_data& y1,nsl_data& y2)
{
	if (a.get_imax() != b.get_imax() || a.get_jmax() != b.get_jmax())
	{
		cmd_error("*** NSLramp ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("jmax1: ",a.get_jmax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error("jmax2: ",b.get_jmax());
		cmd_error_exit("bad matrix sector");
	}
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > b.v[i][j])
			c.v[i][j] = a.v[i][j] - b.v[i][j] + y2.v;
		else
			c.v[i][j] = y1.v;
	return c;
}
nsl_matrix NSLramp(nsl_matrix& a,num_type b,num_type y1,num_type y2)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > b)
			c.v[i][j] = a.v[i][j] - b + y2;
		else
			c.v[i][j] = y1;
	return c;
}
nsl_matrix NSLramp(nsl_matrix& a,nsl_data& b,nsl_data& y1,nsl_data& y2)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > b.v)
			c.v[i][j] = a.v[i][j] - b.v + y2.v;
		else
			c.v[i][j] = y1.v;
	return c;
}

// NSLsat

nsl_matrix NSLsat(nsl_matrix& a)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > (num_type) 1)
			c.v[i][j] = (num_type) 1;
		else if (a.v[i][j] > 0)
			c.v[i][j] = a.v[i][j];
		else
			c.v[i][j] = 0;
	return c;
}
nsl_matrix NSLsat(nsl_matrix& a,nsl_matrix& x1,nsl_matrix& x2)
{
	if (a.get_imax() != x1.get_imax() || a.get_jmax() != x1.get_jmax()||
	    a.get_imax() != x2.get_imax() || a.get_jmax() != x2.get_jmax())
	{
		cmd_error("*** NSLsat ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("jmax1: ",a.get_jmax());
		cmd_error("imax2: ",x1.get_imax());
		cmd_error("jmax2: ",x1.get_jmax());
		cmd_error("imax3: ",x2.get_imax());
		cmd_error("jmax3: ",x2.get_jmax());
		cmd_error_exit("bad matrix sector");
	}
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > x2.v[i][j])
		    c.v[i][j] = 1.0;
		else if (a.v[i][j] > x1.v[i][j])
		    c.v[i][j] = (a.v[i][j]-x1.v[i][j])/(x2.v[i][j]-x1.v[i][j]);
		else
		    c.v[i][j] = 0;
	return c;
}
nsl_matrix NSLsat(nsl_matrix& a,num_type x1,num_type x2)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > x2)
			c.v[i][j] = 1.0;
		else if (a.v[i][j] > x1)
			c.v[i][j] = (a.v[i][j]-x1)/(x2-x1);
		else
			c.v[i][j] = 0;
	return c;
}
nsl_matrix NSLsat(nsl_matrix& a,nsl_data& x1,nsl_data& x2)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > x2.v)
			c.v[i][j] = 1.0;
		else if (a.v[i][j] > x1.v)
			c.v[i][j] = (a.v[i][j]-x1.v)/(x2.v-x1.v);
		else
			c.v[i][j] = 0;
	return c;
}
nsl_matrix NSLsat(nsl_matrix& a,nsl_matrix& x1,nsl_matrix& x2,
	nsl_matrix& y1,nsl_matrix& y2)
{
	if (a.get_imax() != x1.get_imax() || a.get_jmax() != x1.get_jmax() ||
	    a.get_imax() != x2.get_imax() || a.get_jmax() != x2.get_jmax() ||
	    a.get_imax() != y1.get_imax() || a.get_jmax() != y1.get_jmax() ||
	    a.get_imax() != y2.get_imax() || a.get_jmax() != y2.get_jmax())
	{
		cmd_error("*** NSLsat ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("jmax1: ",a.get_jmax());
		cmd_error("imax2: ",x1.get_imax());
		cmd_error("jmax2: ",x1.get_jmax());
		cmd_error("imax3: ",x2.get_imax());
		cmd_error("jmax3: ",x2.get_jmax());
		cmd_error("imax4: ",y1.get_imax());
		cmd_error("jmax4: ",y1.get_jmax());
		cmd_error("imax5: ",y2.get_imax());
		cmd_error("jmax5: ",y2.get_jmax());
		cmd_error_exit("bad matrix sector");
	}
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > x2.v[i][j])
		    c.v[i][j] = y2.v[i][j];
		else if (a.v[i][j] > x1.v[i][j])
		    c.v[i][j] = (y2.v[i][j]-y1.v[i][j])*(a.v[i][j]-x1.v[i][j])/
				(x2.v[i][j]-x1.v[i][j]) + y1.v[i][j];
		else
		    c.v[i][j] = y1.v[i][j];
	return c;
}
nsl_matrix NSLsat(nsl_matrix& a,nsl_matrix& x1,nsl_matrix& x2,
	num_type y1,num_type y2)
{
	if (a.get_imax() != x1.get_imax() || a.get_jmax() != x1.get_jmax() ||
	    a.get_imax() != x2.get_imax() || a.get_jmax() != x2.get_jmax())
	{
		cmd_error("*** NSLsat ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("jmax1: ",a.get_jmax());
		cmd_error("imax2: ",x1.get_imax());
		cmd_error("jmax2: ",x1.get_jmax());
		cmd_error("imax3: ",x2.get_imax());
		cmd_error("jmax3: ",x2.get_jmax());
		cmd_error_exit("bad matrix sector");
	}
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > x2.v[i][j])
		    c.v[i][j] = y2;
		else if (a.v[i][j] > x1.v[i][j])
		    c.v[i][j] = (y2-y1)*(a.v[i][j]-x1.v[i][j])/
				(x2.v[i][j]-x1.v[i][j]) + y1;
		else
		    c.v[i][j] = y1;
	return c;
}
nsl_matrix NSLsat(nsl_matrix& a,nsl_matrix& x1,nsl_matrix& x2,
	nsl_data& y1,nsl_data& y2)
{
	if (a.get_imax() != x1.get_imax() || a.get_jmax() != x1.get_jmax() ||
	    a.get_imax() != x2.get_imax() || a.get_jmax() != x2.get_jmax())
	{
		cmd_error("*** NSLsat ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("jmax1: ",a.get_jmax());
		cmd_error("imax2: ",x1.get_imax());
		cmd_error("jmax2: ",x1.get_jmax());
		cmd_error("imax3: ",x2.get_imax());
		cmd_error("jmax3: ",x2.get_jmax());
		cmd_error_exit("bad matrix sector");
	}
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > x2.v[i][j])
		    c.v[i][j] = y2.v;
		else if (a.v[i][j] > x1.v[i][j])
		    c.v[i][j] = (y2.v-y1.v)*(a.v[i][j]-x1.v[i][j])/
				(x2.v[i][j]-x1.v[i][j]) + y1.v;
		else
		    c.v[i][j] = y1.v;
	return c;
}
nsl_matrix NSLsat(nsl_matrix& a,num_type x1,num_type x2,
	num_type y1,num_type y2)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > x2)
		    c.v[i][j] = y2;
		else if (a.v[i][j] > x1)
		    c.v[i][j] = (y2-y1)*(a.v[i][j]-x1)/(x2-x1) + y1;
		else
		    c.v[i][j] = y1;
	return c;
}
nsl_matrix NSLsat(nsl_matrix& a,nsl_data& x1,nsl_data& x2,
	nsl_data& y1,nsl_data& y2)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > x2.v)
		    c.v[i][j] = y2.v;
		else if (a.v[i][j] > x1.v)
		    c.v[i][j] = (y2.v-y1.v)*(a.v[i][j]-x1.v)/(x2.v-x1.v) + y1.v;
		else
		    c.v[i][j] = y1.v;
	return c;
}

// NSLsigma

nsl_matrix NSLsigma(nsl_matrix& a)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	    for (int j = 0; j < s2; j++)
	   	if (a.v[i][j] > 1.0)
		   c.v[i][j] = 1.0;
	   	else if (a.v[i][j] > 0)
		   c.v[i][j] = pow(a.v[i][j],2.0)*(3.0-2.0*a.v[i][j]);
	  	else
		   c.v[i][j] = 0;
	return c;
}
nsl_matrix NSLsigma(nsl_matrix& a,nsl_matrix& x1,nsl_matrix& x2)
{
 	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	num_type sigma;
	for (int i = 0; i < s1; i++)
	    for (int j = 0; j < s2; j++)
	    {
            	sigma = (a.v[i][j]-x1.v[i][j])/(x2.v[i][j]-x1.v[i][j]);
	    	if (a.v[i][j] > x2.v[i][j])
		    c.v[i][j] = 1.0;
	    	else if (a.v[i][j] > x1.v[i][j])
		    c.v[i][j] = pow(sigma,2.0)*(3.0-2.0*sigma);
	    	else
		    c.v[i][j] = 0;
	    }
	return c;
}
nsl_matrix NSLsigma(nsl_matrix& a,num_type x1,num_type x2)
{
 	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	num_type sigma;
	for (int i = 0; i < s1; i++)
	    for (int j = 0; j < s2; j++)
	    {
            	sigma = (a.v[i][j]-x1)/(x2-x1);
	    	if (a.v[i][j] > x2)
		    c.v[i][j] = 1.0;
	    	else if (a.v[i][j] > x1)
		    c.v[i][j] = pow(sigma,2.0)*(3.0-2.0*sigma);
	    	else
		    c.v[i][j] = 0;
	    }
	return c;
}
nsl_matrix NSLsigma(nsl_matrix& a,nsl_data& x1,nsl_data& x2)
{
 	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	num_type sigma;
	for (int i = 0; i < s1; i++)
	    for (int j = 0; j < s2; j++)
	    {
            	sigma = (a.v[i][j]-x1.v)/(x2.v-x1.v);
	    	if (a.v[i][j] > x2.v)
		    c.v[i][j] = 1.0;
	    	else if (a.v[i][j] > x1.v)
		    c.v[i][j] = pow(sigma,2.0)*(3.0-2.0*sigma);
	    	else
		    c.v[i][j] = 0;
	    }
	return c;
}
nsl_matrix NSLsigma(nsl_matrix& a,nsl_matrix& x1,nsl_matrix& x2,
	nsl_matrix& y1,nsl_matrix& y2)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	    for (int j = 0; j < s2; j++)
	    {
            	num_type sigma = (a.v[i][j]-x1.v[i][j])/(x2.v[i][j]-x1.v[i][j]);
	    	if (a.v[i][j] > x2.v[i][j])
		    c.v[i][j] = y2.v[i][j];
	    	else if (a.v[i][j] > x1.v[i][j])
		    c.v[i][j] = (y2.v[i][j]-y1.v[i][j])*pow(sigma,2.0)*
			(3.0-2.0*sigma) + y1.v[i][j];
	    	else
		    c.v[i][j] = y1.v[i][j];
	    }
	return c;
}
nsl_matrix NSLsigma(nsl_matrix& a,nsl_matrix& x1,nsl_matrix& x2,
	num_type y1,num_type y2)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	    for (int j = 0; j < s2; j++)
	    {
            	num_type sigma = (a.v[i][j]-x1.v[i][j])/(x2.v[i][j]-x1.v[i][j]);
	    	if (a.v[i][j] > x2.v[i][j])
		    c.v[i][j] = y2;
	    	else if (a.v[i][j] > x1.v[i][j])
		    c.v[i][j] = (y2-y1)*pow(sigma,2.0)*(3.0-2.0*sigma) + y1;
	    	else
		    c.v[i][j] = y1;
	    }
	return c;
}
nsl_matrix NSLsigma(nsl_matrix& a,nsl_matrix& x1,nsl_matrix& x2,
	nsl_data& y1,nsl_data& y2)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	    for (int j = 0; j < s2; j++)
	    {
            	num_type sigma = (a.v[i][j]-x1.v[i][j])/(x2.v[i][j]-x1.v[i][j]);
	    	if (a.v[i][j] > x2.v[i][j])
		    c.v[i][j] = y2.v;
	    	else if (a.v[i][j] > x1.v[i][j])
		    c.v[i][j] = (y2.v-y1.v)*pow(sigma,2.0)*(3.0-2.0*sigma) + y1.v;
	    	else
		    c.v[i][j] = y1.v;
	    }
	return c;
}
nsl_matrix NSLsigma(nsl_matrix& a,num_type x1,num_type x2,
	num_type y1,num_type y2)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	    for (int j = 0; j < s2; j++)
	    {
                num_type sigma = (a.v[i][j]-x1)/(x2-x1);
	    	if (a.v[i][j] > x2)
		    c.v[i][j] = y2;
	    	else if (a.v[i][j] > x1)
		    c.v[i][j] = (y2-y1)*pow(sigma,2.0)*(3.0-2.0*sigma) + y1;
	    	else
		    c.v[i][j] = y1;
	    }
	return c;
}
nsl_matrix NSLsigma(nsl_matrix& a,nsl_data& x1,nsl_data& x2,
	nsl_data& y1,nsl_data& y2)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	    for (int j = 0; j < s2; j++)
	    {
                num_type sigma = (a.v[i][j]-x1.v)/(x2.v-x1.v);
	    	if (a.v[i][j] > x2.v)
		    c.v[i][j] = y2.v;
	    	else if (a.v[i][j] > x1.v)
		    c.v[i][j] = (y2.v-y1.v)*pow(sigma,2.0)*(3.0-2.0*sigma) + y1.v;
	    	else
		    c.v[i][j] = y1.v;
	    }
	return c;
}

// NSLmax

num_type NSLmax(nsl_matrix& a) 
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();

	num_type c = 0;

	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > c)
			c = a.v[i][j];
	return c;
}

// if a > b then a else b

nsl_matrix NSLmax(nsl_matrix& a,nsl_matrix& b)
{
	if (a.get_imax() != b.get_imax() || a.get_jmax() != b.get_jmax())
		cmd_error_exit("bad matrix sector");
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > b.v[i][j])
			c.v[i][j] = a.v[i][j];
		else
			c.v[i][j] = b.v[i][j];
	return c;
}
nsl_matrix NSLmax(nsl_matrix& a,nsl_data& b)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > b.v)
			c.v[i][j] = a.v[i][j];
		else
			c.v[i][j] = b.v;
	return c;
}
nsl_matrix NSLmax(nsl_matrix& a,num_type b)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] > b)
			c.v[i][j] = a.v[i][j];
		else
			c.v[i][j] = b;
	return c;
}

// NSLmin

num_type NSLmin(nsl_matrix& a) 
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();

	num_type c = 0;

	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] < c)
			c = a.v[i][j];
	return c;
}

// if a < b then a else b

nsl_matrix NSLmin(nsl_matrix& a,nsl_matrix& b)
{
	if (a.get_imax() != b.get_imax() || a.get_jmax() != b.get_jmax())
		cmd_error_exit("bad matrix sector");
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] < b.v[i][j])
			c.v[i][j] = a.v[i][j];
		else
			c.v[i][j] = b.v[i][j];
	return c;
}
nsl_matrix NSLmin(nsl_matrix& a,nsl_data& b)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] < b.v)
			c.v[i][j] = a.v[i][j];
		else
			c.v[i][j] = b.v;
	return c;
}
nsl_matrix NSLmin(nsl_matrix& a,num_type b)
{
	int s1 = a.get_imax();
	int s2 = a.get_jmax();
	nsl_matrix c(s1,s2);
	for (int i = 0; i < s1; i++)
	   for (int j = 0; j < s2; j++)
		if (a.v[i][j] < b)
			c.v[i][j] = a.v[i][j];
		else
			c.v[i][j] = b;
	return c;
}

// exponential decay

nsl_matrix NSLexpdecay(nsl_matrix& T,nsl_data& kx1)
{
        int imax = T.get_imax();
        int jmax = T.get_jmax();

	nsl_matrix Q(imax,jmax);

        int i0 = T.get_i0();
        int i1 = T.get_i1();
        int j0 = T.get_j0();
        int j1 = T.get_j1();

        num_type dt = NSL_MODEL->get_delta();

        int i,j;
        num_type val;
        
        for (i=i0; i<=i1; i++)
            for (j=j0; j<=j1; j++)
            {
                val = exp(-dt/kx1.elem());
                Q.elem(i,j) = val*T.elem(i,j);
            }

	return Q;
}

// build a kernel

nsl_matrix KERNEL(int imax,int jmax,nsl_data& dia,nsl_data& sig,nsl_data& wgt)
{
        int i,j;
	num_type r,rad;

	nsl_matrix Q(imax,jmax);

//	int imax = Q.get_imax();
//	int jmax = Q.get_jmax();
	int imax2 = imax/2;
	int jmax2 = jmax/2;
        
    	num_type rad0 = dia.elem()/2.0;
	num_type sum = 0.0;
  	if (dia.elem() == 0.0)
		return Q;

        for (i=0; i<imax; i++)
            for (j=0; j<jmax; j++)
	    {
		/* r is in units of 1/2 degrees, same as kernel array */
		r = (num_type) sqrt((num_type) pow((num_type)(i-imax2),2.0) + 
			(num_type) pow((num_type)(j-jmax2),2.0));
		/* rad is in units of degrees */
		rad = r/2.0;
		if (rad < rad0)
	  	     Q.elem(i,j) = (num_type) 
			exp(-(rad*rad)/(2.0*sig.elem()*sig.elem()));
		sum = sum + Q.elem(i,j);
      	    }

	if (sum == 0.0)
		Q = 0;
	else
		Q = Q/sum*wgt;

	return Q;
}

// NSLeuler method

nsl_matrix NSLeuler(nsl_matrix& V,nsl_matrix& S,nsl_data& tm)
{
	float dt;

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm.get_data() == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm.get_data());
		cmd_error_exit("Bad Integration Parameters");
	}

	return (nsl_matrix) (V + dt/tm*S);
}
nsl_matrix NSLeuler(nsl_matrix& V,nsl_matrix& S,num_type tm)
{
	float dt;

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm);
		cmd_error_exit("Bad Integration Parameters");
	}

	return (nsl_matrix) (V + dt/tm*S);
}
nsl_matrix NSLeuler(nsl_matrix& V,nsl_data& S,nsl_data& tm)
{
	float dt;

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm.get_data() == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm.get_data());
		cmd_error_exit("Bad Integration Parameters");
	}

	return (nsl_matrix) (V + dt/tm*S);
}
nsl_matrix NSLeuler(nsl_matrix& V,nsl_data& S,num_type tm)
{
	float dt;

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm);
		cmd_error_exit("Bad Integration Parameters");
	}

	return (nsl_matrix) (V + dt/tm*S);
}
nsl_matrix NSLeuler(nsl_matrix& V,num_type S,nsl_data& tm)
{
	float dt;

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm.get_data() == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm.get_data());
		cmd_error_exit("Bad Integration Parameters");
	}

	return (nsl_matrix) (V + dt/tm*S);
}
nsl_matrix NSLeuler(nsl_matrix& V,num_type S,num_type tm)
{
	float dt;

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm);
		cmd_error_exit("Bad Integration Parameters");
	}

	return (nsl_matrix) (V + dt/tm*S);
}
