//
//	nsl_model.c
//

# include "nsl.h"
# include "nsl_headers.h"

# include "nsl_winds.h"
# include "nsl_system.h"

nsl_model* NSL_MODEL;
extern nsl_system* NSL_SYSTEM_OBJ;

nsl_model::nsl_model(char* str)
{
	strcpy(name,str);

	NSL_MODEL = this;
	nsl_layer_list = new layer_obj_list; 
	nsl_module_list = new module_type_list; 

	layer_total = 0;
	module_total = 0;

	if (NSL_SYSTEM_OBJ == NULL)
	{
		NSL_SYSTEM_OBJ = new nsl_system;
	}

	if (NSL_SYSTEM_OBJ != NULL)
	{
		NSL_SYSTEM_OBJ->add_model(this);
	}
	else
	{
		cmd_error("CANNOT initialize NSL");
		exit(0);
	}
		
}
nsl_model::nsl_model()
{
	NSL_MODEL = this;
	nsl_layer_list = new layer_obj_list; 
	nsl_module_list = new module_type_list; 

	layer_total = 0;
	module_total = 0;

	if (NSL_SYSTEM_OBJ == NULL)
	{
		NSL_SYSTEM_OBJ = new nsl_system;
	}

	if (NSL_SYSTEM_OBJ != NULL)
	{
		NSL_SYSTEM_OBJ->add_model(this);
	}
	else
	{
		cmd_error("CANNOT initialize NSL");
		exit(0);
	}
}

// reset
void nsl_model::reset_time()
{
	CURRENT_TIME_STEP = 0;
	CURRENT_TIME = 0.0;
}
// delta
void nsl_model::set_delta(float i)
{
	DELTA = i;
}
// set
void nsl_model::set_time(int i)
{
	CURRENT_TIME_STEP = i;
	CURRENT_TIME = CURRENT_TIME_STEP*DELTA;
}
void nsl_model::set_time(float i)
{
	CURRENT_TIME = i;
	CURRENT_TIME_STEP = rint(CURRENT_TIME/DELTA);
}
// end
void nsl_model::set_end_time(int i)
{
	END_TIME_STEP = i;
	END_TIME = END_TIME_STEP*DELTA;
}
void nsl_model::set_end_time(float i)
{
	END_TIME = i;
	END_TIME_STEP = rint(END_TIME/DELTA);
}
// increment
void nsl_model::incr_time()
{
	++CURRENT_TIME_STEP;
	CURRENT_TIME = CURRENT_TIME_STEP*DELTA;
}

// add module
 
int nsl_model::add_module(nsl_module* module)
{
	module->set_index(++module_total);
	nsl_module_list->append(module);

	return module_total;
}

// get module

nsl_module* nsl_model::get_module(char* name)
{
	if (nsl_module_list != NULL)
	{
	   char* str;
	   module_type_iterator next(*nsl_module_list);
	   nsl_module* module;
	   while (module = next())
	   {
		str = module->get_name();
		if (strcmp(name,str) == 0)
			return (nsl_module*) module;
	   }
	}
	else
	   cmd_out("NULL 'nsl_module_list'");

	return (nsl_module*) 0;
}
nsl_module* nsl_model::get_module(int index)
{
	if (nsl_module_list != NULL)
	{
	   module_type_iterator next(*nsl_module_list);
	   nsl_module* module;
	   while (module = next())
	   {
		if (index == module->get_index())
			return (nsl_module*) module;
	   }
	}
	else
	   cmd_out("NULL 'nsl_module_list'");

	return (nsl_module*) 0;
}

// run module

void nsl_model::run()
{
	if (print_time_fg == 1 || verbose_fg == 1)
		cmd_out("// simulation time: ",get_time());

	if (nsl_module_list != NULL)
	{
	    module_type_iterator next(*nsl_module_list);
	    nsl_module* m;
	    while (m = next()) 
		((nsl_module*) m)->proc(CURRENT_TIME); // global
	}
	else
	   cmd_out("NULL 'nsl_module_list'");
}

// set all verbose: model,module

void nsl_model::set_all_verbose_fg(int fg)
{
	set_verbose_fg(fg);
	if (nsl_module_list != NULL)
	{
	    module_type_iterator next(*nsl_module_list);
	    nsl_module* m;
	    while (m = next()) 
		((nsl_module*) m)->set_verbose_fg(fg);
	}
	else
	   cmd_out("NULL 'nsl_module_list'");
}

// add layers

int nsl_model::add_layer(nsl_data* layer)
{
	layer->set_index(++layer_total);
	nsl_layer_list->append(layer);

	return layer_total;
}
int nsl_model::add_layer(nsl_vector* layer)
{
	layer->set_index(++layer_total);
	nsl_layer_list->append(layer);

	return layer_total;
}
int nsl_model::add_layer(nsl_matrix* layer)
{
	layer->set_index(++layer_total);
	nsl_layer_list->append(layer);

	return layer_total;
}

// get layer

nsl_layer* nsl_model::get_layer()
{
	if (nsl_layer_list != NULL)
	{
	   char* str;
	   layer_obj_iterator next(*nsl_layer_list);
	   nsl_layer* layer;
	   
		layer = next(); // return first layer
		return layer;
	}
	else
	   cmd_out("NULL 'nsl_layer_list'");

	return (nsl_layer*) 0;
}
nsl_layer* nsl_model::get_layer(char* name)
{
	if (nsl_layer_list != NULL)
	{
	   char* str;
	   layer_obj_iterator next(*nsl_layer_list);
	   nsl_layer* layer;
	   while (layer = next())
	   {
		str = layer->get_name();
		if (strcmp(name,str) == 0)
			return layer;
	   }
	}
	else
	   cmd_out("NULL 'nsl_layer_list'");

	return (nsl_layer*) 0;
}
nsl_layer* nsl_model::get_layer(int index)
{
	if (nsl_layer_list != NULL)
	{
	   layer_obj_iterator next(*nsl_layer_list);
	   nsl_layer* layer;
	   while (layer = next())
	   {
		if (index == layer->get_index())
			return layer;
	   }
	}
	else
	   cmd_out("NULL 'nsl_layer_list'");

	return (nsl_layer*) 0;
}

// get input layer

nsl_layer* nsl_model::get_input_layer()
{
	if (nsl_layer_list != NULL)
	{
	   char* str;
	   layer_obj_iterator next(*nsl_layer_list);
	   nsl_layer* layer;
	   while (layer = next())
	   {
		if (layer->get_h_type() == 1)
			return layer;
	   }
	}
	else
	   cmd_out("NULL 'nsl_layer_list'");

	return (nsl_layer*) 0;
}
nsl_layer* nsl_model::get_input_layer(char* name)
{
	if (nsl_layer_list != NULL)
	{
	   char* str;
	   layer_obj_iterator next(*nsl_layer_list);
	   nsl_layer* layer;
	   while (layer = next())
	   {
		str = layer->get_name();
		if (strcmp(name,str) == 0)
		{
		    if (layer->get_h_type() == 1)
			return layer;
		    else
		    {
			cmd_error("Not an input layer");
			return (nsl_layer*) 0;
		    }
		}
	   }
	}
	else
	   cmd_out("NULL 'nsl_layer_list'");

	return (nsl_layer*) 0;
}
nsl_layer* nsl_model::get_input_layer(int index)
{
	if (nsl_layer_list != NULL)
	{
	   layer_obj_iterator next(*nsl_layer_list);
	   nsl_layer* layer;
	   while (layer = next())
	   {
		if (index == layer->get_index())
		{
		    if (layer->get_h_type() == 1)
			return layer;
		    else
		    {
			cmd_error("Not an input layer");
			return (nsl_layer*) 0;
		    }
		}
	   }
	}
	else
	   cmd_out("NULL 'nsl_layer_list'");

	return (nsl_layer*) 0;
}

void nsl_model::file_header(ostream& out)
{
	if (nsl_layer_list != NULL)
	{
	   out << "set delta " << DELTA << "\n";
	   out << "set end_time " << END_TIME << "\n";

	   out << "disable module ALL\n";
	   out << "disable layer ALL\n";

	   layer_obj_iterator next(*nsl_layer_list);
	   nsl_layer* layer;
	   while (layer = next())
		if (layer->get_able_fg() == 1)
		    out << "enable layer " << layer->get_name() << "\n";

//	   out << "set data_value ALL_TIME\n";
	}
	else
	   cmd_out("NULL 'nsl_layer_list'");
}


// stream

extern int nsl_skip_comment(istream&);

ostream& operator<<(ostream& out,nsl_model& model)
{
	layer_obj_list* list;

	if ((list = model.get_layer_list()) != NULL)
	{
	   layer_obj_iterator next(*list);
	   nsl_layer* layer;
	   while (layer = next())
	   { 
		if (layer->get_able_fg() == 0)
			continue;

                out << "// " << layer->get_name();
                out << ", t=" << model.get_time();
                out << "\n";

	    	switch (layer->get_layer_type())
	    	{
		    case 1:
		 	out << *((nsl_data*) layer);
			break;
		    case 2:
		 	out << *((nsl_vector*) layer);
			break;
		    case 3:
		 	out << *((nsl_matrix*) layer);
			break;
		}
	    }
	}
	else
	{
		cmd_out("Null model layer list");
	}

	return out;
}
istream& operator>>(istream& in,nsl_model& model)
{
	layer_obj_list* list;

	if ((list = model.get_layer_list()) != NULL)
	{
	   layer_obj_iterator next(*list);
	   nsl_layer* layer;
	   while (layer = next())
	   { 
		if (layer->get_able_fg() == 0)
			continue;

		nsl_skip_comment(in); // single line comment

	    	switch (layer->get_layer_type())
	    	{
		    case 1:
		 	in >> *((nsl_data*) layer);
			break;
		    case 2:
		 	in >> *((nsl_vector*) layer);
			break;
		    case 3:
		 	in >> *((nsl_matrix*) layer);
			break;
		}
	    }
	}
	else
	{
		cmd_out("Null model layer list");
	}

	return in;
}
ostream& operator<<(ostream& out,nsl_model* model)
{
	layer_obj_list* list;

	if ((list = model->get_layer_list()) != NULL)
	{
	   layer_obj_iterator next(*list);
	   nsl_layer* layer;
	   while (layer = next())
	   { 
		if (layer->get_able_fg() == 0)
			continue;

                out << "// " << layer->get_name();
                out << ", t=" << model->get_time();
                out << "\n";

	    	switch (layer->get_layer_type())
	    	{
		    case 1:
		 	out << *((nsl_data*) layer);
			break;
		    case 2:
		 	out << *((nsl_vector*) layer);
			break;
		    case 3:
		 	out << *((nsl_matrix*) layer);
			break;
		}
	    }
	}
	else
	{
		cmd_out("Null model layer list");
	}

	return out;
}
istream& operator>>(istream& in,nsl_model* model)
{
	layer_obj_list* list;

	if ((list = model->get_layer_list()) != NULL)
	{
	   layer_obj_iterator next(*list);
	   nsl_layer* layer;
	   while (layer = next())
	   { 
		if (layer->get_able_fg() == 0)
			continue;

		nsl_skip_comment(in); // single line comment

	    	switch (layer->get_layer_type())
	    	{
		    case 1:
		 	in >> *((nsl_data*) layer);
			break;
		    case 2:
		 	in >> *((nsl_vector*) layer);
			break;
		    case 3:
		 	in >> *((nsl_matrix*) layer);
			break;
		}
	    }
	}
	else
	{
		cmd_out("Null model layer list");
	}

	return in;
}

// print model status info to 'out' stream

void nsl_model::print_status(ostream& out)
{
	out << "\n";
	out << "// " << index << ": " << name << "\n";
	out << "// total_layers: " << layer_total << "\n";
	out << "// module_total: " << module_total << "\n";
	out << "set network " << name << "\n";
	if (able_fg == 1)
	   out << "enable network\n";
	else // if (able_fg == 0)
	   out << "disable network\n";
/*	if (able_fg == 1)
	   out << "set able_fg ON\n";
	else // if (able_fg == 0)
	   out << "set able_fg OFF\n";
	if (verbose_fg == 1)
	   out << "set verbose_fg ON\n";
	else // if (verbose_fg == 0)
	   out << "set verbose_fg OFF\n";
*/	if (print_time_fg == 1)
	   out << "set print_time_fg ON\n";
	else // if (print_time_fg == 0)
	   out << "set print_time_fg OFF\n";
	out << "set delta " << DELTA << "\n";
	out << "set current_time " << CURRENT_TIME << "\n";
	out << "set end_time " << END_TIME << "\n";
	out.flush();
}

// print model status info on all modules

void nsl_model::print_all_module_status(ostream& out)
{
	if (nsl_module_list != NULL)
	{
	   module_type_iterator next(*nsl_module_list);
	   nsl_module* module;
	   while (module = next()) 
		module->print_status(out);
	}
}
void nsl_model::print_module_status(ostream& out,char* str)
{
	nsl_module* module;
	if ((module = get_module(str)) != NULL)
	    module->print_status(out);
	else
	    cmd_error("Unknown Module: ",str);
}
void nsl_model::print_module_status(ostream& out,int i)
{
	nsl_module* module;
	if ((module = get_module(i)) != NULL)
	    module->print_status(out);
	else
	    cmd_error("Unknown Module Index: ",i);
}

// enable module

void nsl_model::enable_all_module()
{
	if (nsl_module_list != NULL)
	{
	   module_type_iterator next(*nsl_module_list);
	   nsl_module* module;
	   while (module = next()) 
		module->enable();
	}
	else
	   cmd_out("NULL 'nsl_init_module_list'");
}
void nsl_model::enable_module(char* name)
{
	nsl_module* module;
	if ((module = get_module(name)) != NULL)
		module->enable();
	else
		cmd_error("Unknown Module: ",name);
}
void nsl_model::enable_module(int index)
{
	nsl_module* module;
	if ((module = get_module(index)) != NULL)
		module->enable();
	else
		cmd_error("Unknown Module: ",index);
}

// disable module

void nsl_model::disable_all_module()
{
	if (nsl_module_list != NULL)
	{
	   module_type_iterator next(*nsl_module_list);
	   nsl_module* module;
	   while (module = next()) 
		module->disable();
	}
	else
	   cmd_out("NULL 'nsl_init_module_list'");
}
void nsl_model::disable_module(char* name)
{
	nsl_module* module;
	if ((module = get_module(name)) != NULL)
		module->disable();
	else
		cmd_error("Unknown Module: ",name);
}
void nsl_model::disable_module(int index)
{
	nsl_module* module;
	if ((module = get_module(index)) != NULL)
		module->disable();
	else
		cmd_error("Unknown Module: ",index);
}

// enable layer

void nsl_model::enable_all_layer()
{
	if (nsl_layer_list != NULL)
	{
	   layer_obj_iterator next(*nsl_layer_list);
	   nsl_layer* layer;
	   while (layer = next()) 
		layer->enable();
	}
	else
	   cmd_out("NULL 'nsl_layer_list'");
}
void nsl_model::enable_layer(char* name)
{
	nsl_layer* layer;
	if ((layer = get_layer(name)) != NULL)
		layer->enable();
	else
		cmd_error("Unknown Layer: ",name);
}
void nsl_model::enable_layer(int index)
{
	nsl_layer* layer;
	if ((layer = get_layer(index)) != NULL)
		layer->enable();
	else
		cmd_error("Unknown Layer Index: ",index);
}

// disable layer

void nsl_model::disable_all_layer()
{
	if (nsl_layer_list != NULL)
	{
	   layer_obj_iterator next(*nsl_layer_list);
	   nsl_layer* layer;
	   while (layer = next()) 
		layer->disable();
	}
	else
	   cmd_out("NULL 'nsl_layer_list'");
}
void nsl_model::disable_layer(char* name)
{
	nsl_layer* layer;
	if ((layer = get_layer(name)) != NULL)
		layer->disable();
	else
		cmd_error("Unknown Layer: ",name);
}
void nsl_model::disable_layer(int index)
{
	nsl_layer* layer;
	if ((layer = get_layer(index)) != NULL)
		layer->disable();
	else
		cmd_error("Unknown Layer Index: ",index);
}

// print layer data

void nsl_model::print_all_layer_data(ostream& out)
{
	if (nsl_layer_list != NULL)
	{
	   layer_obj_iterator next(*nsl_layer_list);
	   nsl_layer* layer;
	   while (layer = next()) 
		layer->print(out);
	}
	else
	   cmd_out("NULL 'nsl_layer_list'");
}
void nsl_model::print_layer_data(ostream& out,char* name)
{
	nsl_layer* layer;
	if ((layer = get_layer(name)) != NULL)
		layer->print(out);
	else
		cmd_error("Unknown Layer: ",name);
}
void nsl_model::print_layer_data(ostream& out,int index)
{
	nsl_layer* layer;
	if ((layer = get_layer(index)) != NULL)
		layer->print(out);
	else
		cmd_error("Unknown Layer Index: ",index);
}

// print layer status

void nsl_model::print_all_layer_status(ostream& out)
{
	if (nsl_layer_list != NULL)
	{
	   layer_obj_iterator next(*nsl_layer_list);
	   nsl_layer* layer;
	   while (layer = next()) 
		layer->print_status(out);
	}
	else
	   cmd_out("NULL 'nsl_layer_list'");
}
void nsl_model::print_layer_status(ostream& out,char* name)
{
	nsl_layer* layer;
	if ((layer = get_layer(name)) != NULL)
		layer->print_status(out);
	else
		cmd_error("Unknown Layer: ",name);
}
void nsl_model::print_layer_status(ostream& out,int index)
{
	nsl_layer* layer;
	if ((layer = get_layer(index)) != NULL)
		layer->print_status(out);
	else
		cmd_error("Unknown Layer Index: ",index);
}

// input layer

void nsl_model::print_all_input_layer_status(ostream& out)
{
	if (nsl_layer_list != NULL)
	{
	   layer_obj_iterator next(*nsl_layer_list);
	   nsl_layer* layer;
	   while (layer = next())
	   {
	      if (layer->get_h_type() == 1)
		layer->print_status(out);
	   }
	}
	else
	   cmd_out("NULL 'nsl_layer_list'");
}
void nsl_model::print_input_layer_status(ostream& out,char* name)
{
	nsl_layer* layer;
	if ((layer = get_layer(name)) != NULL)
	{
	    if (layer->get_h_type() == 1)
		layer->print_status(out);
	    else
		cmd_error("Not an input layer");
	}
	else
		cmd_error("Unknown Layer: ",name);
}
void nsl_model::print_input_layer_status(ostream& out,int index)
{
	nsl_layer* layer;
	if ((layer = get_layer(index)) != NULL)
	{
	    if (layer->get_h_type() == 1)
		layer->print_status(out);
	    else
		cmd_error("Not an input layer");
	}
	else
		cmd_error("Unknown Layer Index: ",index);
}

// input stim

void nsl_model::print_all_input_layer_stim(ostream& out)
{
	if (nsl_layer_list != NULL)
	{
	   layer_obj_iterator next(*nsl_layer_list);
	   nsl_layer* layer;
	   while (layer = next())
	   {
	      if (layer->get_h_type() == 1)
		layer->print_stim(out);
	   }
	}
	else
	   cmd_out("NULL 'nsl_layer_list'");
}
void nsl_model::print_input_layer_stim(ostream& out,char* name)
{
	nsl_layer* layer;
	if ((layer = get_layer(name)) != NULL)
	{
	      if (layer->get_h_type() == 1)
		layer->print_stim(out);
	}
	else
		cmd_error("Unknown Layer: ",name);
}
void nsl_model::print_input_layer_stim(ostream& out,int index)
{
	nsl_layer* layer;
	if ((layer = get_layer(index)) != NULL)
	{
	      if (layer->get_h_type() == 1)
		layer->print_stim(out);
	}
	else
		cmd_error("Unknown Layer Index: ",index);
}

