//
//	nsl_number_library.c
//

# include "nsl.h"
# include "nsl_headers.h"

extern nsl_model* NSL_MODEL;

// numerical threshold functions

// NSLstep

// if a > 0 then 1 else 0

num_type NSLstep(num_type a)
{
	num_type c;
	if (a > (num_type) 0)
		c = (num_type) 1;
	else
		c = 0;
	return c;
}

// if a > b then 1 else 0

num_type NSLstep(num_type a,num_type b)
{
	num_type c;
	if (a > b)
		c = (num_type) 1;
	else
		c = 0;
	return c;
}

// if a > b then y2 else y1

num_type NSLstep(num_type a,num_type b,num_type y1,num_type y2)
{
	num_type c;
	if (a > b)
		c = y2;
	else
		c = y1;
	return c;
}

// NSLramp

// if a > 0 then a else 0

num_type NSLramp(num_type a) 
{
	num_type c;
	if (a > (num_type) 0)
		c = a;
	else
		c = 0;
	return c;
}

// if a > b then a - b else 0

num_type NSLramp(num_type a,num_type b)
{
	num_type c;
	if (a > b)
		c = a - b;
	else
		c = 0;
	return c;
}

// if a > b then a - b + y2 else y1

num_type NSLramp(num_type a,num_type b,num_type y1,num_type y2)
{
	num_type c;
	if (a > b)
		c = a - b + y2;
	else
		c = y1;
	return c;
}

// NSLmax

num_type NSLmax(num_type a,num_type b) 
{
	num_type c = (a < b) ? a : b;
	return c;
}
num_type val_NSLmin(num_type a,num_type b)
{
	num_type c = (a < b) ? a : b;
	return c;
}
int NSLmax(int a,int b)
{
	int c = (a > b) ? a : b;
	return c;
}
int NSLmin(int a,int b)
{
	int c = (a < b) ? a : b;
	return c;
}

// NSLsat

num_type NSLsat(num_type a)
{
	num_type c;
	if (a > (num_type) 0)
		c = (num_type) 1;
	else if (a > 0)
		c = a;
	else
		c = 0;
	return c;
}
num_type NSLsat(num_type a,num_type x1,num_type x2)
{
	num_type c;
	if (a > x2)
		c = (num_type) 1;
	else if (a > x1)
		c = (a-x1)/(x2-x1);
	else
		c = 0;
	return c;
}
num_type NSLsat(num_type a,num_type x1,num_type x2,num_type y1,num_type y2)
{
	num_type c;
	if (a > x2)
		c = y2;
	else if (a > x1)
		c = (y2-y1)*(a-x1)/(x2-x1) + y1;
	else
		c = y1;
	return c;
}

// NSLsigma

num_type NSLsigma(num_type a)
{
	num_type c;
	if (a > 1.0)
		c = 1.0;
	else if (a > 0)
		c = pow(a,2.0)*(3.0-2.0*a);
	else
		c = 0;
	return c;
}
num_type NSLsigma(num_type a,num_type x1,num_type x2)
{
	num_type c;
        num_type sigma = (a-x1)/(x2-x1);
	if (a > x2)
		c = 1.0;
	else if (a > x1)
		c = pow(sigma,2.0)*(3.0-2.0*sigma);
	else
		c = 0.0;
	return c;
}
num_type NSLsigma(num_type a,num_type x1,num_type x2,num_type y1,num_type y2)
{
	num_type c;
        num_type sigma = (a-x1)/(x2-x1);
	if (a > x2)
		c = y2;
	else if (a > x1)
		c = (y2-y1)*pow(sigma,2.0)*(3.0-2.0*sigma) + y1;
	else
		c = y1;
	return c;
}

// NSLeuler
num_type NSLeuler(num_type V,num_type S,num_type tm)
{
	float dt;

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm);
		cmd_error_exit("Bad Integration Parameters");
	}

	return (num_type) (V + dt/tm*S);
}
