//
//	nsl_vector.c
//

# include "nsl.h"
# include "nsl_headers.h"

extern nsl_model* NSL_MODEL;

nsl_vector::nsl_vector() : (2)
{
	i0 = 0;
	i1 = 0;
	imax = 0;
	v = (num_type*) 0;
}
int nsl_vector::alloc_mem(int s)
{
	if (s <= 0)
	{
		cmd_error("*** nsl_vector::nsl_vector ***");
		cmd_error("Exit with status:");
		cmd_error("name: ",name);
		cmd_error("s: ",s);
		cmd_error_exit("Cannot create vector");
	}
	i0 = 0; i1 = s-1;
	imax = s;
	v = new num_type[s];

	for (int i = 0; i < s; i++)
		v[i] = 0;

	return 1;
}

// nsl_vector constructors

nsl_vector::nsl_vector(int s) : (2)
{
	if (s <= 0)
	{
		cmd_error("*** nsl_vector::nsl_vector ***");
		cmd_error("Exit with status:");
		cmd_error("name: ",name);
		cmd_error("s: ",s);
		cmd_error_exit("Cannot create vector");
	}
	i0 = 0; i1 = s-1;
	imax = s;
	v = new num_type[s];

	for (int i = 0; i < s; i++)
		v[i] = 0;
}
nsl_vector::nsl_vector(nsl_vector& a) : (2)
{
        int s = a.get_imax();
        v = new num_type[s];
        i0 = 0; i1 = s-1;
        imax = s;
        for (int i = 0; i < s; i++)
                v[i] = a.v[i];
}
nsl_vector::nsl_vector(nsl_model* model,char* str,int s) : (str,2)
{
	if (s <= 0)
	{
		cmd_error("*** nsl_vector::nsl_vector ***");
		cmd_error("Exit with status:");
		cmd_error("name: ",name);
		cmd_error("s: ",s);
		cmd_error_exit("Cannot create vector");
	}
	i0 = 0; i1 = s-1;
	imax = s;
	v = new num_type[s];

	for (int i = 0; i < s; i++)
		v[i] = 0;

	model->add_layer(this);
}
nsl_vector::nsl_vector(nsl_model& model,char* str,int s) : (str,2)
{
	if (s <= 0)
	{
		cmd_error("*** nsl_vector::nsl_vector ***");
		cmd_error("Exit with status:");
		cmd_error("name: ",name);
		cmd_error("s: ",s);
		cmd_error_exit("Cannot create vector");
	}
	i0 = 0; i1 = s-1;
	imax = s;
	v = new num_type[s];

	for (int i = 0; i < s; i++)
		v[i] = 0;

	model.add_layer(this);
}
nsl_vector::nsl_vector(char* str,int s) : (str,2)
{
	if (s <= 0)
	{
		cmd_error("*** nsl_vector::nsl_vector ***");
		cmd_error("Exit with status:");
		cmd_error("name: ",name);
		cmd_error("s: ",s);
		cmd_error_exit("Cannot create vector");
	}
	i0 = 0; i1 = s-1;
	imax = s;
	v = new num_type[s];

	for (int i = 0; i < s; i++)
		v[i] = 0;

	NSL_MODEL->add_layer(this);
}
nsl_vector::nsl_vector(char* str,int s,int type) : (str,2)
{
	if (s <= 0)
	{
		cmd_error("*** nsl_vector::nsl_vector ***");
		cmd_error("Exit with status:");
		cmd_error("name: ",name);
		cmd_error("s: ",s);
		cmd_error_exit("Cannot create vector");
	}
	i0 = 0; i1 = s-1;
	imax = s;
	v = new num_type[s];

	for (int i = 0; i < s; i++)
		v[i] = 0;

	vec_type = (type == 1) ? 1 : 0;

	NSL_MODEL->add_layer(this);
}

// nsl_vector destructors

nsl_vector::~nsl_vector()
{
	delete v;
}

// vector object element

num_type& nsl_vector::operator[](int i)
{
	if (i < 0 || imax <= i)
	{
		cmd_error("*** nsl_vector::operator[] ***");
		cmd_error("Exit with status:");
		cmd_error("name: ",name);
		cmd_error("i: ",i);
		cmd_error_exit("Vector index out of range");
	}

	return (num_type&) v[i];
}
num_type& nsl_vector::elem(int i)
{
	if (i < 0 || imax <= i)
	{
		cmd_error("*** nsl_vector::elem ***");
		cmd_error("Exit with status:");
		cmd_error("name: ",name);
		cmd_error("i: ",i);
		cmd_error_exit("Vector index out of range");
	}

	return (num_type&) v[i];
}


// vector assignment

void nsl_vector::operator=(nsl_vector& a)
{
	if (DEBUG_FG == 1)
	{
		cout << "// nsl_vector::operator=\n";
		cout << "// (vector) " << this->get_name() << " = (vector) ";
		cout << a.get_name() << ";\n";
		cout.flush();
	}

	if (a.imax != imax)
	{
		cmd_error("*** nsl_vector::operator= ***");
		cmd_error("Exit with status:");
		a.print_status(cerr);
		this->print_status(cerr);
		cmd_error_exit("Inconsistent vector sizes");
	}

	if (able_fg == 0)
		return;

	for (int i = 0; i < imax; i++)
		v[i] = a.v[i];
}
void nsl_vector::operator=(nsl_data& a)
{
	if (DEBUG_FG == 1)
	{
		cout << "// nsl_vector::operator=\n";
		cout << "// (vector) " << this->get_name() << " = (data) ";
		cout << a.get_name() << ";\n";
		cout.flush();
	}

	int imax = get_imax();  

	num_type val = a.get_value();

	if (able_fg == 0)
		return;

	for (int i = 0; i < imax; i++)
		v[i] = val;
}
/*
void nsl_vector::operator=(num_type* a) // no boundary checking
{
	if (able_fg == 0)
		return;

	for (int i = 0; i < imax; i++)
		v[i] = a[i];
}
*/
void nsl_vector::operator=(num_type a)
{
	int imax = get_imax();

	if (able_fg == 0)
		return;

	for (int i = 0; i < imax; i++)
		v[i] = a;
}

// vector addition

nsl_vector operator+(nsl_vector& a)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	for (int i = 0; i < aimax; i++)
		c.v[i] = a.v[i];

	return c;
}
nsl_vector operator+(nsl_vector& a,nsl_vector& b)
{
	int aimax = a.get_imax();
	int bimax = b.get_imax();

	if (aimax != bimax)
	{
		cmd_error("*** nsl_vector::operator+ ***");
		cmd_error("Exit with status:");
		a.print_status(cerr);
		b.print_status(cerr);
		cmd_error_exit("Inconsistent vector sizes");
	}

	nsl_vector c(aimax);

	for (int i = 0; i < aimax; i++)
		c.v[i] = a.v[i] + b.v[i];

	return c;
}
nsl_vector operator+(nsl_vector& a,num_type b)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	for (int i = 0; i < aimax; i++)
		c.v[i] = a.v[i] + b;

	return c;
}
nsl_vector operator+(num_type a, nsl_vector& b)
{
	int bimax = b.get_imax();

	nsl_vector c(bimax);

	for (int i = 0; i < bimax; i++)
		c.v[i] = a + b.v[i];

	return c;
}
nsl_vector operator+(nsl_vector& a,nsl_data& b)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	for (int i = 0; i < aimax; i++)
		c.v[i] = a.v[i] + b.v;
	return c;
}
nsl_vector operator+(nsl_data& a, nsl_vector& b)
{
	int bimax = b.get_imax();

	nsl_vector c(bimax);

	for (int i = 0; i < bimax; i++)
		c.v[i] = a.v + b.v[i];

	return c;
}

// vector subtraction

nsl_vector operator-(nsl_vector& a)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	for (int i = 0; i < aimax; i++)
		c.v[i] = -a.v[i];

	return c;
}
nsl_vector operator-(nsl_vector& a,nsl_vector& b)
{
	int aimax = a.get_imax();
	int bimax = b.get_imax();

	if (aimax != bimax)
	{
		cmd_error("*** nsl_vector::operator- ***");
		cmd_error("Exit with status:");
		a.print_status(cerr);
		b.print_status(cerr);
		cmd_error_exit("Inconsistent vector sizes");
	}

	nsl_vector c(aimax);

	for (int i = 0; i < aimax; i++)
		c.v[i] = a.v[i] - b.v[i];
	return c;
}
nsl_vector operator-(nsl_vector& a,num_type b)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	for (int i = 0; i < aimax; i++)
		c.v[i] = a.v[i] - b;

	return c;
}
nsl_vector operator-(num_type a, nsl_vector& b)
{
	int bimax = b.get_imax();

	nsl_vector c(bimax);

	for (int i = 0; i < bimax; i++)
		c.v[i] = a - b.v[i];

	return c;
}
nsl_vector operator-(nsl_vector& a,nsl_data& b)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	for (int i = 0; i < aimax; i++)
		c.v[i] = a.v[i] - b.v;

	return c;
}
nsl_vector operator-(nsl_data& a, nsl_vector& b)
{
	int bimax = b.get_imax();

	nsl_vector c(bimax);

	for (int i = 0; i < bimax; i++)
		c.v[i] = a.v - b.v[i];

	return c;
}

// vector pointwise multiplication

nsl_vector operator^(nsl_vector& a,nsl_vector& b)
{
	int aimax = a.get_imax();
	int bimax = b.get_imax();

	if (aimax != bimax)
	{
		cmd_error("*** nsl_vector::operator^ ***");
		cmd_error("Exit with status:");
		a.print_status(cerr);
		b.print_status(cerr);
		cmd_error_exit("Inconsistent vector sizes");
	}
	nsl_vector c(aimax);

	for (int i = 0; i < aimax; i++)
		c.v[i] = a.v[i] * b.v[i];

	return c;
}
nsl_vector operator^(nsl_vector& a,num_type b)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	for (int i = 0; i < aimax; i++)
		c.v[i] = a.v[i] * b;

	return c;
}
nsl_vector operator^(num_type a, nsl_vector& b)
{
	int bimax = b.get_imax();

	nsl_vector c(bimax);

	for (int i = 0; i < bimax; i++)
		c.v[i] = a * b.v[i];

	return c;
}
nsl_vector operator^(nsl_vector& a,nsl_data& b)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	for (int i = 0; i < aimax; i++)
		c.v[i] = a.v[i] * b.v;
	return c;
}
nsl_vector operator^(nsl_data& a, nsl_vector& b)
{
	int bimax = b.get_imax();

	nsl_vector c(bimax);

	for (int i = 0; i < bimax; i++)
		c.v[i] = a.v * b.v[i];

	return c;
}

// vector convolution

nsl_vector operator*(nsl_vector& a,nsl_vector& b)
{
	int saimax = a.get_imax();
	int sbimax = b.get_imax();

	int simax = saimax + sbimax - 1;

	nsl_vector c(sbimax);
	nsl_vector d(simax);

	d = d.put_sector(b,saimax/2); 
//	d.print();
	for (int i = 0; i < sbimax; i++)
	   {	
		num_type val = (num_type) 0;
		for (int m = 0; m < saimax; m++)
			val = val + a.v[m] * d.v[i+m];
		c.v[i] = val;
	   }
//	c.print();
	return c;
}
nsl_vector operator*(nsl_vector& a,num_type b)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	for (int i = 0; i < aimax; i++)
		c.v[i] = a.v[i] * b;

	return c;
}
nsl_vector operator*(num_type a, nsl_vector& b)
{
	int bimax = b.get_imax();

	nsl_vector c(bimax);

	for (int i = 0; i < bimax; i++)
		c.v[i] = a * b.v[i];

	return c;
}
nsl_vector operator*(nsl_vector& a,nsl_data& b)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	for (int i = 0; i < aimax; i++)
		c.v[i] = a.v[i] * b.v;

	return c;
}
nsl_vector operator*(nsl_data& a, nsl_vector& b)
{
	int bimax = b.get_imax();

	nsl_vector c(bimax);

	for (int i = 0; i < bimax; i++)
		c.v[i] = a.v * b.v[i];

	return c;
}

// vector pointwise division

nsl_vector operator/(nsl_vector& a,nsl_vector& b)
{
	int aimax = a.get_imax();
	int bimax = b.get_imax();

	if (aimax != bimax)
	{
		cmd_error("*** nsl_vector::operator/ ***");
		cmd_error("Exit with status:");
		a.print_status(cerr);
		b.print_status(cerr);
		cmd_error_exit("Inconsistent vector sizes");
	}
	nsl_vector c(aimax);

	for (int i = 0; i < aimax; i++)
		c.v[i] = a.v[i] / b.v[i];

	return c;
}
nsl_vector operator/(nsl_vector& a,num_type b)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	for (int i = 0; i < aimax; i++)
		c.v[i] = a.v[i] / b;

	return c;
}
nsl_vector operator/(num_type a, nsl_vector& b)
{
	int bimax = b.get_imax();

	nsl_vector c(bimax);

	for (int i = 0; i < bimax; i++)
		c.v[i] = a / b.v[i];

	return c;
}
nsl_vector operator/(nsl_vector& a,nsl_data& b)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	for (int i = 0; i < aimax; i++)
		c.v[i] = a.v[i] / b.v;

	return c;
}
nsl_vector operator/(nsl_data& a, nsl_vector& b)
{
	int bimax = b.get_imax();

	nsl_vector c(bimax);

	for (int i = 0; i < bimax; i++)
		c.v[i] = a.v / b.v[i];

	return c;
}

// max and min

num_type nsl_vector::max()
{
	int simax = get_imax();

	num_type c = v[0];

	for (int i = 0; i < simax; i++)
		if (v[i] > c)
			c = v[i];

	return c;
}
num_type nsl_vector::min()
{
	int simax = get_imax();

	num_type c = v[0];

	for (int i = 0; i < simax; i++)
		if (v[i] < c)
			c = v[i];

	return c;
}

// element sum

num_type nsl_vector::sum()
{
	num_type c = 0;

	for (int i = 0; i < imax; i++)
		c = v[i] + c;
	return c;
}

// expand

nsl_matrix nsl_vector::expand(int n)
{
	if (vec_type == 1)
		return expand_col(n);
	else
		return expand_row(n);
}
nsl_matrix nsl_vector::expand_col(int n)
{
	int simax = get_imax();
	int sjmax = n;

	nsl_matrix c(simax,sjmax);
	
	for (int i = 0; i < simax; i++)
	    for (int j = 0; j < sjmax; j++)
		c.v[i][j] = v[i];

	return c;
}
nsl_matrix nsl_vector::expand_row(int n)
{
	int sjmax = get_imax();
	int simax = n;

	nsl_matrix c(simax,sjmax);
	
	for (int i = 0; i < simax; i++)
	    for (int j = 0; j < sjmax; j++)
		c.v[i][j] = v[j];

	return c;
}

// transpose

nsl_vector nsl_vector::transpose()
{
	vec_type = (vec_type == 1) ? 0 : 1;

	return *this;
}
nsl_vector nsl_vector::transpose_row()
{
	vec_type = 0;  // row 

	return *this;
}
nsl_vector nsl_vector::transpose_col()
{
	vec_type = 1;  // col 

	return *this;
}

// vector sector

nsl_vector nsl_vector::get_sector(int si0, int si1)
{
	if (si0 > si1 || si0 < 0 || si1 > get_imax())
	{
		cmd_error("*** nsl_vector::get_sector ***");
		cmd_error("Exit with status:");
		this->print_status(cerr);
		cmd_error("si0: ",si0);
		cmd_error("si1: ",si1);
		cmd_error_exit("Bad vector sector");
	}

	int simax = si1 - si0 + 1;

	nsl_vector c(simax);

	for (int i = si0; i <= si1; i++)
		c.v[i-si0] = v[i]; 

	return c;
}
nsl_vector nsl_vector::set_sector(nsl_vector& a, int si0)
{
	if (si0 < 0)
	{
		cmd_error("*** nsl_vector::set_sector ***");
		cmd_error("Exit with status:");
		a.print_status(cerr);
		this->print_status(cerr);
		cmd_error("i0: ",si0);
		cmd_error_exit("Bad vector sector");
	}

	int saimax = a.get_imax();
	int simax = get_imax();

	nsl_vector c(simax);

	int smax = (saimax < simax) ? saimax : simax;

	c = (*this);

	for (int i = 0; i < smax; i++)
		c.v[i+si0] = a.v[i];

	return c;
}
nsl_vector nsl_vector::put_sector(nsl_vector& a, int si0)
{
	return set_sector(a,si0);
}

// stream

extern int nsl_get_str(istream&,char*);

ostream& operator<<(ostream& out,nsl_vector& a)
{
	int imax = a.get_imax();

	for (int i = 0; i < imax; i++)
		out << a.v[i] << " ";
	out << "\n";

	return out;
}
istream& operator>>(istream& in,nsl_vector& a)
{
	int imax = a.get_imax();

	float v;
	string str;
	char* aptr[NAME_SIZE];
	char** ptr = aptr;

	for (int i = 0; in && i < imax; i++)
	{
//		in >> a.v[i];

		// loop until some string is encountered on any line
		while (nsl_get_str(in,str) == 0)
		{
		    if (strcmp(str,"EOF") == 0) // check for EOF
		    {
			cmd_error("Not enough vector input values");
			return in;
		    }
		}

		v = strtod(str,ptr); // similar to 'atof(str)' w/ error check

		if (strcmp(str,*ptr) == 0)
		{
		    cmd_error("Bad vector input value: ",str);
		    return in;
		}
		else
		{
			a.v[i] = v;
		}
	}

	return in;
}

// print vector values

void nsl_vector::print(ostream& out)
{
	out << "\n";
	out << "set data_value " << name << "\n";
	out << *this;
	out.flush();
}
void nsl_vector::print(ostream& out,int si0, int si1)
{
	if (si0 > si1 || si0 < 0 || si1 > get_imax())
	{
		cmd_error("*** nsl_matrix::print ***");
		cmd_error("Exit with status:");
		this->print_status(cerr);
		cmd_error("si0: ",si0);
		cmd_error("si1: ",si1);
		cmd_error("i0: ",i0);
		cmd_error("i1: ",i1);
		cmd_error_exit("Bad vector sector");
	}

	out << "\n// " << name << "\n";
	out << "set data_value " << name << "\n";
	for (int i = si0; i <= si1; i++)
		out << v[i] << " "; 
	out << "\n";
	out.flush();
}
void nsl_vector::print_layer_status(ostream& out)
{
	nsl_layer::print_layer_status(out);

	out << "// imax: " << imax << "\n";
	out << "// i0: " << i0 << "\n";
	out << "// i1: " << i1 << "\n";
	out.flush();
}
void nsl_vector::print_status(ostream& out)
{
	print_layer_status(out);
}
	
