//
//	nsl_vector_mult_library.c
//

# include "nsl.h"
# include "nsl_headers.h"

// vector pointwise multiplication

nsl_vector NSLmult(nsl_vector& a,nsl_vector& b)
{
	int aimax = a.get_imax();
	int bimax = b.get_imax();

	if (aimax != bimax)
	{
		cmd_error("*** nsl_vector::NSLmult ***");
		cmd_error("Exit with status:");
		a.print_status(cerr);
		b.print_status(cerr);
		cmd_error_exit("Inconsistent vector sizes");
	}

	nsl_vector c(aimax);

	num_type* cv = c.get_vector();
	num_type* av = a.get_vector();
	num_type* bv = b.get_vector();

	for (int i = 0; i < aimax; i++)
		cv[i] = av[i] * bv[i];

	return c;
}
nsl_vector NSLmult(nsl_vector& a,num_type b)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	num_type* cv = c.get_vector();
	num_type* av = a.get_vector();

	for (int i = 0; i < aimax; i++)
		cv[i] = av[i] * b;

	return c;
}
nsl_vector NSLmult(num_type a, nsl_vector& b)
{
	int bimax = b.get_imax();

	nsl_vector c(bimax);

	num_type* cv = c.get_vector();
	num_type* bv = b.get_vector();

	for (int i = 0; i < bimax; i++)
		cv[i] = a * bv[i];

	return c;
}
nsl_vector NSLmult(nsl_vector& a,nsl_data& b)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	num_type* cv = c.get_vector();
	num_type* av = a.get_vector();
	num_type bv = b.get_data();

	for (int i = 0; i < aimax; i++)
		cv[i] = av[i] * bv;

	return c;
}
nsl_vector NSLmult(nsl_data& a, nsl_vector& b)
{
	int bimax = b.get_imax();

	nsl_vector c(bimax);

	num_type* cv = c.get_vector();
	num_type* bv = b.get_vector();
	num_type av = a.get_data();

	for (int i = 0; i < bimax; i++)
		cv[i] = av * bv[i];

	return c;
}

