//
//	nsl_vector_library.c
//

# include "nsl.h"
# include "nsl_headers.h"

extern nsl_model* NSL_MODEL;

// vector threshold functions

// NSLstep

// if a > 0 then 1 else 0

nsl_vector NSLstep(nsl_vector& a)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > (num_type) 0)
			cv[i] = (num_type) 1;
		else
			cv[i] = 0;
	return c;
}

// if a > b then 1 else 0

nsl_vector NSLstep(nsl_vector& a,nsl_vector& b)
{
	if (a.get_imax() != b.get_imax())
	{
		cmd_error("*** NSLstep ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error_exit("bad vector sector");
	}
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* bv = b.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > bv[i])
			cv[i] = (num_type) 1;
		else
			cv[i] = 0;
	return c;
}
nsl_vector NSLstep(nsl_vector& a,num_type b)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > b)
			cv[i] = (num_type) 1;
		else
			cv[i] = 0;
	return c;
}
nsl_vector NSLstep(nsl_vector& a,nsl_data& b)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type bv = b.get_data();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > bv)
			cv[i] = (num_type) 1;
		else
			cv[i] = 0;
	return c;
}

// if a > b then y2 else y1

nsl_vector NSLstep(nsl_vector& a,nsl_vector& b,nsl_vector& y1,nsl_vector& y2)
{
	if (a.get_imax() != b.get_imax())
	{
		cmd_error("*** NSLstep ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error("imax3: ",y1.get_imax());
		cmd_error("imax4: ",y2.get_imax());
		cmd_error_exit("bad vector sector");
	}
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* bv = b.get_vector();
	num_type* cv = c.get_vector();
	num_type* y1v = y1.get_vector();
	num_type* y2v = y2.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > bv[i])
			cv[i] = y2v[i];
		else
			cv[i] = y1v[i];
	return c;
}
nsl_vector NSLstep(nsl_vector& a,nsl_vector& b,num_type y1,num_type y2)
{
	if (a.get_imax() != b.get_imax())
	{
		cmd_error("*** NSLstep ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error_exit("bad vector sector");
	}
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* bv = b.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > bv[i])
			cv[i] = y2;
		else
			cv[i] = y1;
	return c;
}
nsl_vector NSLstep(nsl_vector& a,nsl_vector& b,nsl_data& y1,nsl_data& y2)
{
	if (a.get_imax() != b.get_imax())
	{
		cmd_error("*** NSLstep ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error_exit("bad vector sector");
	}
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* bv = b.get_vector();
	num_type* cv = c.get_vector();
	num_type y1v = y1.get_data();
	num_type y2v = y2.get_data();

	for (int i = 0; i < s1; i++)
		if (av[i] > bv[i])
			cv[i] = y2v;
		else
			cv[i] = y1v;
	return c;
}
nsl_vector NSLstep(nsl_vector& a,num_type b,num_type y1,num_type y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > b)
			cv[i] = y2;
		else
			cv[i] = y1;
	return c;
}
nsl_vector NSLstep(nsl_vector& a,nsl_data& b,nsl_data& y1,nsl_data& y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type bv = b.get_data();
	num_type* cv = c.get_vector();
	num_type y1v = y1.get_data();
	num_type y2v = y2.get_data();

	for (int i = 0; i < s1; i++)
		if (av[i] > bv)
			cv[i] = y2v;
		else
			cv[i] = y1v;
	return c;
}

// NSLramp

// if a > 0 then a else 0

nsl_vector NSLramp(nsl_vector& a) 
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > (num_type) 0)
			cv[i] = av[i];
		else
			cv[i] = 0;
	return c;
}

// if a > b then a - b else 0

nsl_vector NSLramp(nsl_vector& a,nsl_vector& b)
{
	if (a.get_imax() != b.get_imax())
	{
		cmd_error("*** NSLramp ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error_exit("bad vector sector");
	}
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* bv = b.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > bv[i])
			cv[i] = av[i] - bv[i];
		else
			cv[i] = 0;
	return c;
}
nsl_vector NSLramp(nsl_vector& a,num_type b)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (a.v[i] > b)
			cv[i] = av[i] - b;
		else
			cv[i] = 0;
	return c;
}
nsl_vector NSLramp(nsl_vector& a,nsl_data& b)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type bv = b.get_data();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > bv)
			cv[i] = av[i] - bv;
		else
			cv[i] = 0;
	return c;
}

// if a > b then a - b + y2 else y1

nsl_vector NSLramp(nsl_vector& a,nsl_vector& b,nsl_vector& y1,nsl_vector& y2)
{
	if (a.get_imax() != b.get_imax())
	{
		cmd_error("*** NSLramp ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error("imax3: ",y1.get_imax());
		cmd_error("imax4: ",y2.get_imax());
		cmd_error_exit("bad vector sector");
	}
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* bv = b.get_vector();
	num_type* cv = c.get_vector();
	num_type* y1v = y1.get_vector();
	num_type* y2v = y2.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > bv[i])
			cv[i] = av[i] - bv[i] + y2v[i];
		else
			cv[i] = y1v[i];
	return c;
}
nsl_vector NSLramp(nsl_vector& a,nsl_vector& b,num_type y1,num_type y2)
{
	if (a.get_imax() != b.get_imax())
	{
		cmd_error("*** NSLstep ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error_exit("bad vector sector");
	}
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* bv = b.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > bv[i])
			cv[i] = av[i] - bv[i] + y2;
		else
			cv[i] = y1;
	return c;
}
nsl_vector NSLramp(nsl_vector& a,nsl_vector& b,nsl_data& y1,nsl_data& y2)
{
	if (a.get_imax() != b.get_imax())
	{
		cmd_error("*** NSLstep ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",b.get_imax());
		cmd_error_exit("bad vector sector");
	}
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* bv = b.get_vector();
	num_type* cv = c.get_vector();
	num_type y1v = y1.get_data();
	num_type y2v = y2.get_data();

	for (int i = 0; i < s1; i++)
		if (av[i] > bv[i])
			cv[i] = av[i] - bv[i] + y2v;
		else
			cv[i] = y1v;
	return c;
}
nsl_vector NSLramp(nsl_vector& a,num_type b,num_type y1,num_type y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > b)
			cv[i] = av[i] - b + y2;
		else
			cv[i] = y1;
	return c;
}
nsl_vector NSLramp(nsl_vector& a,nsl_data& b,nsl_data& y1,nsl_data& y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type bv = b.get_data();
	num_type* cv = c.get_vector();
	num_type y1v = y1.get_data();
	num_type y2v = y2.get_data();

	for (int i = 0; i < s1; i++)
		if (av[i] > bv)
			cv[i] = av[i] - bv + y2v;
		else
			cv[i] = y1v;
	return c;
}

// NSLsat

nsl_vector NSLsat(nsl_vector& a)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > (num_type) 1)
			cv[i] = (num_type) 1;
		else if (av[i] > 0)
			cv[i] = a.v[i];
		else
			cv[i] = 0;
	return c;
}
nsl_vector NSLsat(nsl_vector& a,nsl_vector& x1,nsl_vector& x2)
{
	if (a.get_imax() != x1.get_imax() || a.get_imax() != x2.get_imax())
	{
		cmd_error("*** NSLsat ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",x1.get_imax());
		cmd_error("imax3: ",x2.get_imax());
		cmd_error_exit("bad vector sector");
	}
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();
	num_type* x1v = x1.get_vector();
	num_type* x2v = x2.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > x2v[i])
			cv[i] = (num_type) 1;
		else if (av[i] > x1v[i])
			cv[i] = (av[i]-x1v[i])/(x2v[i]-x1v[i]);
		else
			cv[i] = 0;
	return c;
}
nsl_vector NSLsat(nsl_vector& a,num_type x1,num_type x2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > x2)
			cv[i] = (num_type) 1;
		else if (av[i] > x1)
			cv[i] = (av[i] - x1)/(x2 - x1);
		else
			cv[i] = 0;
	return c;
}
nsl_vector NSLsat(nsl_vector& a,nsl_data& x1,nsl_data& x2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();
	num_type x1v = x1.get_data();
	num_type x2v = x2.get_data();

	for (int i = 0; i < s1; i++)
		if (av[i] > x2v)
			cv[i] = (num_type) 1;
		else if (av[i] > x1v)
			cv[i] = (av[i] - x1v)/(x2v - x1v);
		else
			cv[i] = 0;
	return c;
}
nsl_vector NSLsat(nsl_vector& a,nsl_vector& x1,nsl_vector& x2,
	nsl_vector& y1,nsl_vector& y2)
{
	if (a.get_imax() != x1.get_imax() ||  a.get_imax() != x2.get_imax() || 
	    a.get_imax() != y1.get_imax() ||  a.get_imax() != y2.get_imax())
	{
		cmd_error("*** NSLsat ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",x1.get_imax());
		cmd_error("imax3: ",x2.get_imax());
		cmd_error("imax4: ",y1.get_imax());
		cmd_error("imax5: ",y2.get_imax());
		cmd_error_exit("bad vector sector");
	}
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();
	num_type* x1v = x1.get_vector();
	num_type* x2v = x2.get_vector();
	num_type* y1v = y1.get_vector();
	num_type* y2v = y2.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > x2v[i])
		    cv[i] = y2v[i];
		else if (av[i] > x1v[i])
		    cv[i] = (y2v[i]-y1v[i])*(av[i]-x1v[i])/
				(x2v[i]-x1v[i]) + y1v[i];
		else
		    cv[i] = y1v[i];
	return c;
}
nsl_vector NSLsat(nsl_vector& a,nsl_vector& x1,nsl_vector& x2,
	num_type y1,num_type y2)
{
	if (a.get_imax() != x1.get_imax() || a.get_imax() != x2.get_imax())
	{
		cmd_error("*** NSLsat ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",x1.get_imax());
		cmd_error("imax3: ",x2.get_imax());
		cmd_error_exit("bad vector sector");
	}
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();
	num_type* x1v = x1.get_vector();
	num_type* x2v = x2.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > x2v[i])
		    cv[i] = y2;
		else if (av[i] > x1v[i])
		    cv[i] = (y2-y1)*(av[i]-x1v[i])/
				(x2v[i]-x1v[i]) + y1;
		else
		    cv[i] = y1;
	return c;
}
nsl_vector NSLsat(nsl_vector& a,nsl_vector& x1,nsl_vector& x2,
	nsl_data& y1,nsl_data& y2)
{
	if (a.get_imax() != x1.get_imax() || a.get_imax() != x2.get_imax())
	{
		cmd_error("*** NSLsat ***");
		cmd_error("Exit with the following status");
		cmd_error("imax1: ",a.get_imax());
		cmd_error("imax2: ",x1.get_imax());
		cmd_error("imax3: ",x2.get_imax());
		cmd_error_exit("bad vector sector");
	}
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();
	num_type* x1v = x1.get_vector();
	num_type* x2v = x2.get_vector();
	num_type y1v = y1.get_data();
	num_type y2v = y2.get_data();

	for (int i = 0; i < s1; i++)
		if (av[i] > x2v[i])
		    cv[i] = y2v;
		else if (av[i] > x1v[i])
		    cv[i] = (y2v-y1v)*(av[i]-x1v[i])/
				(x2v[i]-x1v[i]) + y1v;
		else
		    cv[i] = y1v;
	return c;
}
nsl_vector NSLsat(nsl_vector& a,num_type x1,num_type x2,
	num_type y1,num_type y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > x2)
		    cv[i] = y2;
		else if (av[i] > x1)
		    cv[i] = (y2-y1)*(av[i]-x1)/(x2-x1) + y1;
		else
		    cv[i] = y1;
	return c;
}
nsl_vector NSLsat(nsl_vector& a,nsl_data& x1,nsl_data& x2,
	nsl_data& y1,nsl_data& y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();
	num_type x1v = x1.get_data();
	num_type x2v = x2.get_data();
	num_type y1v = y1.get_data();
	num_type y2v = y2.get_data();

	for (int i = 0; i < s1; i++)
		if (av[i] > x2v)
		    cv[i] = y2v;
		else if (av[i] > x1v)
		    cv[i] = (y2v-y1v)*(av[i]-x1v)/(x2v-x1v) + y1v;
		else
		    cv[i] = y1v;
	return c;
}

// NSLsigma

nsl_vector NSLsigma(nsl_vector& a)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
	   if (av[i] > 1.0)
		cv[i] = 1.0;
	   else if (av[i] > 0)
		cv[i] = pow(av[i],2.0)*(3.0-2.0*av[i]);
	   else
		cv[i] = 0;
	return c;
}
nsl_vector NSLsigma(nsl_vector& a,nsl_vector& x1,nsl_vector& x2)
{
 	int s1 = a.get_imax();
	nsl_vector c(s1);
	num_type sigma;

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();
	num_type* x1v = x1.get_vector();
	num_type* x2v = x2.get_vector();

	for (int i = 0; i < s1; i++)
	{
            sigma = (av[i]-x1v[i])/(x2v[i]-x1v[i]);
	    if (av[i] > x2v[i])
		    cv[i] = 1.0;
	    else if (av[i] > x1v[i])
		    cv[i] = pow(sigma,2.0)*(3.0-2.0*sigma);
	    else
		    cv[i] = 0;
	}
	return c;
}
nsl_vector NSLsigma(nsl_vector& a,num_type x1,num_type x2)
{
 	int s1 = a.get_imax();
	nsl_vector c(s1);
	num_type sigma;

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
	{
            sigma = (av[i]-x1)/(x2-x1);
	    if (av[i] > x2)
		    cv[i] = 1.0;
	    else if (av[i] > x1)
		    cv[i] = pow(sigma,2.0)*(3.0-2.0*sigma);
	    else
		    cv[i] = 0;
	}
	return c;
}
nsl_vector NSLsigma(nsl_vector& a,nsl_data& x1,nsl_data& x2)
{
 	int s1 = a.get_imax();
	nsl_vector c(s1);
	num_type sigma;

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();
	num_type x1v = x1.get_data();
	num_type x2v = x2.get_data();

	for (int i = 0; i < s1; i++)
	{
            sigma = (av[i]-x1v)/(x2v-x1v);
	    if (av[i] > x2v)
		    cv[i] = 1.0;
	    else if (av[i] > x1v)
		    cv[i] = pow(sigma,2.0)*(3.0-2.0*sigma);
	    else
		    cv[i] = 0;
	}
	return c;
}
nsl_vector NSLsigma(nsl_vector& a,nsl_vector& x1,nsl_vector& x2,
	nsl_vector& y1,nsl_vector& y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();
	num_type* x1v = x1.get_vector();
	num_type* x2v = x2.get_vector();
	num_type* y1v = y1.get_vector();
	num_type* y2v = y2.get_vector();

	for (int i = 0; i < s1; i++)
	{
            num_type sigma = (av[i]-x1v[i])/(x2v[i]-x1v[i]);
	    if (av[i] > x2v[i])
		    cv[i] = y2v[i];
	    else if (av[i] > x1v[i])
		    cv[i] = (y2v[i]-y1v[i])*pow(sigma,2.0)*(3.0-2.0*sigma)
			 + y1v[i];
	    else
		    cv[i] = y1v[i];
	}
	return c;
}
nsl_vector NSLsigma(nsl_vector& a,nsl_vector& x1,nsl_vector& x2,
	num_type y1,num_type y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();
	num_type* x1v = x1.get_vector();
	num_type* x2v = x2.get_vector();

	for (int i = 0; i < s1; i++)
	{
            num_type sigma = (av[i]-x1v[i])/(x2v[i]-x1v[i]);
	    if (av[i] > x2v[i])
		    cv[i] = y2;
	    else if (av[i] > x1v[i])
		    cv[i] = (y2-y1)*pow(sigma,2.0)*(3.0-2.0*sigma) + y1;
	    else
		    cv[i] = y1;
	}
	return c;
}
nsl_vector NSLsigma(nsl_vector& a,nsl_vector& x1,nsl_vector& x2,
	nsl_data& y1,nsl_data& y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();
	num_type* x1v = x1.get_vector();
	num_type* x2v = x2.get_vector();
	num_type y1v = y1.get_data();
	num_type y2v = y2.get_data();

	for (int i = 0; i < s1; i++)
	{
            num_type sigma = (av[i]-x1v[i])/(x2v[i]-x1v[i]);
	    if (av[i] > x2v[i])
		    cv[i] = y2v;
	    else if (av[i] > x1v[i])
		    cv[i] = (y2v-y1v)*pow(sigma,2.0)*(3.0-2.0*sigma) + y1v;
	    else
		    cv[i] = y1v;
	}
	return c;
}
nsl_vector NSLsigma(nsl_vector& a,num_type x1,num_type x2,
	num_type y1,num_type y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
	{
            num_type sigma = (av[i]-x1)/(x2-x1);
	    if (av[i] > x2)
		    cv[i] = y2;
	    else if (av[i] > x1)
		    cv[i] = (y2-y1)*pow(sigma,2.0)*(3.0-2.0*sigma) + y1;
	    else
		    cv[i] = y1;
	}
	return c;
}
nsl_vector NSLsigma(nsl_vector& a,nsl_data& x1,nsl_data& x2,
	nsl_data& y1,nsl_data& y2)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();
	num_type x1v = x1.get_data();
	num_type x2v = x2.get_data();
	num_type y1v = y1.get_data();
	num_type y2v = y2.get_data();

	for (int i = 0; i < s1; i++)
	{
            num_type sigma = (av[i]-x1v)/(x2v-x1v);
	    if (av[i] > x2v)
		    cv[i] = y2v;
	    else if (av[i] > x1v)
		    cv[i] = (y2v-y1v)*pow(sigma,2.0)*(3.0-2.0*sigma) + y1v;
	    else
		    cv[i] = y1v;
	}
	return c;
}

// NSLmax

num_type NSLmax(nsl_vector& a) 
{
	int s1 = a.get_imax();

	num_type* av = a.get_vector();

	num_type c = 0;

	for (int i = 0; i < s1; i++)
		if (av[i] > c)
			c = av[i];

	return c;
}

// if a > b then a else b

nsl_vector NSLmax(nsl_vector& a,nsl_vector& b)
{
	if (a.get_imax() != b.get_imax())
		cmd_error_exit("bad vector sector");
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* bv = b.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > bv[i])
			cv[i] = av[i];
		else
			cv[i] = bv[i];
	return c;
}
nsl_vector NSLmax(nsl_vector& a,nsl_data& b)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type bv = b.get_data();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > bv)
			cv[i] = av[i];
		else
			cv[i] = bv;
	return c;
}
nsl_vector NSLmax(nsl_vector& a,num_type b)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] > b)
			cv[i] = av[i];
		else
			cv[i] = b;
	return c;
}

// NSLmin

num_type NSLmin(nsl_vector& a) 
{
	int s1 = a.get_imax();

	num_type* av = a.get_vector();

	num_type c = 0;

	for (int i = 0; i < s1; i++)
		if (av[i] < c)
			c = av[i];

	return c;
}

// if a < b then a else b

nsl_vector NSLmin(nsl_vector& a,nsl_vector& b)
{
	if (a.get_imax() != b.get_imax())
		cmd_error_exit("bad vector sector");
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* bv = b.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] < bv[i])
			cv[i] = av[i];
		else
			cv[i] = bv[i];
	return c;
}
nsl_vector NSLmin(nsl_vector& a,nsl_data& b)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type bv = b.get_data();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] < bv)
			cv[i] = av[i];
		else
			cv[i] = bv;
	return c;
}
nsl_vector NSLmin(nsl_vector& a,num_type b)
{
	int s1 = a.get_imax();
	nsl_vector c(s1);

	num_type* av = a.get_vector();
	num_type* cv = c.get_vector();

	for (int i = 0; i < s1; i++)
		if (av[i] < b)
			cv[i] = av[i];
		else
			cv[i] = b;
	return c;
}

// gaussian 

nsl_vector NSLgauss(nsl_data& sig,int xn)
{
	nsl_vector c(xn);
	float m = xn/2;

	for (int i=0; i < xn; i++)
	    c.elem(i) = (float) exp(-(i-m)*(i-m)/(2*sig.elem()*sig.elem()));

	return c;
}

// NSLeuler method

nsl_vector NSLeuler(nsl_vector& V,nsl_vector& S,nsl_data& tm)
{
	float dt;

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm.get_data() == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm.get_data());
		cmd_error_exit("Bad Integration Parameters");
	}

	return (nsl_vector) (V + dt/tm*S);
}
nsl_vector NSLeuler(nsl_vector& V,nsl_vector& S,num_type tm)
{
	float dt;

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm);
		cmd_error_exit("Bad Integration Parameters");
	}

	return (nsl_vector) (V + dt/tm*S);
}
nsl_vector NSLeuler(nsl_vector& V,nsl_data& S,nsl_data& tm)
{
	float dt;

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm.get_data() == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm.get_data());
		cmd_error_exit("Bad Integration Parameters");
	}

	return (nsl_vector) (V + dt/tm*S);
}
nsl_vector NSLeuler(nsl_vector& V,nsl_data& S,num_type tm)
{
	float dt;

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm);
		cmd_error_exit("Bad Integration Parameters");
	}

	return (nsl_vector) (V + dt/tm*S);
}
nsl_vector NSLeuler(nsl_vector& V,num_type S,nsl_data& tm)
{
	float dt;

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm.get_data() == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm.get_data());
		cmd_error_exit("Bad Integration Parameters");
	}

	return (nsl_vector) (V + dt/tm*S);
}
nsl_vector NSLeuler(nsl_vector& V,num_type S,num_type tm)
{
	float dt;

	if (NSL_MODEL != NULL)
		dt = NSL_MODEL->get_delta();
	else
	{
		cmd_error("*** NSLeuler ***");
		cmd_error_exit("Unknown Model");
	}

	if (dt == 0 || tm == 0)
	{
		cmd_error("*** NSLeuler ***");
		cmd_error("delta: ",dt);
		cmd_error("time constant: ",tm);
		cmd_error_exit("Bad Integration Parameters");
	}

	return (nsl_vector) (V + dt/tm*S);
}
