//
//	panel_item_object.c
//

# include "nsl.h"
# include "nsl_headers.h"
# include "nsl_winds.h"

extern Panel_item create_panel_text(Scrollable_panel,int,int,int,int,
	char*,int);
extern Panel_item create_panel_choice(Scrollable_panel,int,int,int,int,
	char*,int,int,Panel_setting (*)(Panel_item,Event*));
extern Panel_item create_panel_message(Scrollable_panel,int,int,char*);
extern Panel_item create_panel_button(Scrollable_panel,int,int,int,int,
	char*,void (*)(Panel_item));

extern void update_button_proc(Panel_item);

extern void set_panel_message(Panel_item,char*);
extern int set_panel_value(Panel_item,char*);
extern int set_panel_value(Panel_item,int);
extern char* get_panel_value(Panel_item);
extern int get_panel_choice_value(Panel_item);

extern Panel_setting panel_choice_handler(Panel_item,Event*);
extern void set_panel_choice_menu(Panel_item,int);
extern int get_menu_choice_index(int,char*);
extern char* get_menu_text(int,int);

// panel gen item obj
panel_item_obj::panel_item_obj(display_window* window,char* str,
	char* type_str,int index) : panel_object(window,str,type_str,index)
{
}
panel_item_obj::~panel_item_obj()
{
}
void panel_item_obj::print(ostream& out)
{
	panel_object::print(out);
}

// panel button item obj
panel_button_item_obj::panel_button_item_obj(display_window* window,
	char* str,int index) : (window,str,"button",index)
{
}
panel_button_item_obj::~panel_button_item_obj()
{
}

void panel_button_item_obj::init()
{
	Panel panel;
	if (panel_fg == 1)
		panel = main_panel;
	else
		panel = data_panel;

	if (able_fg == 1)
	   item = create_panel_button(panel,wx,wy,0,0,label,update_button_proc);
}


// panel data item obj
panel_data_item_obj::panel_data_item_obj(display_window* window,
	char* str,int index) : (window,str,"data",index)
{
}
panel_data_item_obj::~panel_data_item_obj()
{
}

void panel_data_item_obj::init()
{
	Panel panel;
	if (panel_fg == 1)
		panel = main_panel;
	else
		panel = data_panel;

	if (able_fg == 1)
	    item = create_panel_text(panel,wx,wy,0,0,label,10);
}

void panel_data_item_obj::write_to_database()
{
	char* str = get_panel_value(item);
	int ival;
       	float fval;
	int type = parent_window->get_general_data_type(name);

	switch (type)
	{
	case 1:
	       	ival = atoi(str);
		parent_window->set_general_data(name,ival);
		break;
	case 2:
	       	fval = atof(str);
		parent_window->set_general_data(name,fval);
		break;
	case 3:
		parent_window->set_general_data(name,str);
		break;
	default:
		cmd_error("*** panel_data_item_obj::write_to_database ***");
		cmd_error("Panel data type error: ",type);
	}
}
void panel_data_item_obj::read_from_database()
{
	int ival = 0;
	float fval = 0;
	string sval;

	int type = parent_window->get_general_data_type(name);

	switch (type)
	{
	case 1:
		parent_window->get_general_data(name,&ival);
		sprintf(text,"%d",ival);
		break;
	case 2:
		parent_window->get_general_data(name,&fval);
//		text << fval;
		sprintf(text,"%f",fval);
		break;
	case 3:
		strcpy(sval,"");
		parent_window->get_general_data(name,sval);
		sprintf(text,"%s",sval);
		break;
	default:
		cmd_error("*** panel_data_item_obj::read_from_database ***");
		cmd_error("Unknown data type: ",type);
	}
}
void panel_data_item_obj::display()
{
	if (able_fg == 1)
            set_panel_value(item,text);
}

// panel data menu item obj
panel_data_menu_item_obj::panel_data_menu_item_obj(display_window* window,
	char* str,int index) : (window,str,"menu",index)
{
}
panel_data_menu_item_obj::~panel_data_menu_item_obj()
{
}

void panel_data_menu_item_obj::init()
{
	Panel panel;
	if (panel_fg == 1)
		panel = main_panel;
	else
		panel = data_panel;

	int len = 20;
 	if (able_fg == 1)
	{
           item = create_panel_choice(panel,wx,wy,0,0,label,len,
		menu_index,panel_choice_handler);
	   set_panel_choice_menu(item,menu_index);
	}
}

void panel_data_menu_item_obj::read_from_database()
{
	int ival = 0;
	float fval = 0;
	string sval;
	char* str;

	int type = parent_window->get_general_data_type(name);

	switch (type)
	{
	case 1:
		parent_window->get_general_data(name,&ival);
		strcpy(text,get_menu_text(menu_index,ival));
		choice_index = ival;
		break;
	case 2:
		parent_window->get_general_data(name,&fval);
		sprintf(text,"%f",fval);
		break;
	case 3:
		strcpy(sval,"");
		parent_window->get_general_data(name,sval);
		sprintf(text,"%s",sval);
		choice_index = get_menu_choice_index(menu_index,text);
		break;
	default:
		cmd_error("*** panel_data_item_obj::read_from_database ***");
		cmd_error("Unknown data type: ",type);
	}
//	cmd_out("read type: ",type);
//	cmd_out("menu_index: ",menu_index);
//	cmd_out("text: ",text);
//	cmd_out("index: ",choice_index);
}
void panel_data_menu_item_obj::write_to_database()
{
	char* str;

	int type = parent_window->get_general_data_type(name);

	switch (type)
	{
	case 1:
		choice_index = get_panel_choice_value(item);
		str = get_menu_text(menu_index,choice_index);
		parent_window->set_general_data(name,choice_index);
		break;
	case 2:
//	       	float fval = atof(str);
//		parent_window->set_general_data(name,fval);
		break;
	case 3:
		choice_index = get_panel_choice_value(item);
		str = get_menu_text(menu_index,choice_index);
		parent_window->set_general_data(name,str,choice_index);
		break;
	default:
		cmd_error("*** panel_data_item_obj::write_to_database ***");
		cmd_error("Panel data type error: ",type);
	}
//	cmd_out("write type: ",type);
//	cmd_out("menu_index: ",menu_index);
//	cmd_out("text: ",str);
//	cmd_out("index: ",choice_index);
}
void panel_data_menu_item_obj::display()
{
 	if (able_fg == 1)
      	    set_panel_value(item,choice_index);
}

// panel message item obj
panel_message_item_obj::panel_message_item_obj(display_window* window,
	char* str,int index) : (window,str,"message",index)
{
}
panel_message_item_obj::~panel_message_item_obj()
{
}

void panel_message_item_obj::init()
{
	Panel panel;
	if (panel_fg == 1)
		panel = main_panel;
	else
		panel = data_panel;

 	if (able_fg == 1)
           item = create_panel_message(panel,wx,wy,label);
}

void panel_message_item_obj::read_from_database()
{
	int ival = 0;
	float fval = 0;
	string sval;

	int type = parent_window->get_general_data_type(name);

	switch (type)
	{
	case 1:
		parent_window->get_general_data(name,&ival);
		sprintf(text,"%s %d",label,ival);
		break;
	case 2:
		parent_window->get_general_data(name,&fval);
//		text << label << " " << fval;
		sprintf(text,"%s %f",label,fval);
		break;
	case 3:
		strcpy(sval,"");
		parent_window->get_general_data(name,sval);
		sprintf(text,"%s %s",label,sval);
		break;
	default:
		cmd_error("*** panel_data_item_obj::read_from_database ***");
		cmd_error("Unknown data type: ",type);
	}
}
void panel_message_item_obj::write_to_database()
{
}
void panel_message_item_obj::display()
{
	if (able_fg == 1)
             set_panel_message(item,text);
}




