/****************************************************************/
/*                                                            	*/
/*                       panel_winds.c                        	*/
/*                                                            	*/
/* by Alfredo Weitzenfeld				      	*/
/*                                                            	*/
/****************************************************************/

// static char SccsId[] = "%W%  %G%"; /* SCCS */

# include "nsl.h"
# include "nsl_headers.h"
# include "nsl_winds.h"
# include "nsl_system.h"

extern nsl_model* NSL_MODEL;
extern window_interface* NSL_INTERFACE;
extern nsl_system* NSL_SYSTEM;

Scrollable_panel create_scroll_panel(Frame frame,int x,int y,int w,int h)
{
	 return xv_create(frame,SCROLLABLE_PANEL,
		XV_X, x,
		XV_Y, y,
		XV_WIDTH, w,
		XV_HEIGHT, h,
		XV_SHOW,TRUE,
               	0);
}
void resize_panel(Panel panel,int x,int y,int w,int h)
{
	xv_set(panel,
		XV_X, x,
		XV_Y, y,
		XV_WIDTH, w,
		XV_HEIGHT, h,
           	XV_SHOW, TRUE,
		0);
}
void resize_scroll_panel(Scrollable_panel panel,int x,int y,int w,int h)
{
	xv_set(panel,
		XV_X, x,
		XV_Y, y,
		XV_WIDTH, w,
		XV_HEIGHT, h,
         	XV_SHOW, TRUE,
		0);
}
void set_panel_below(Panel bottom_panel,Panel top_panel)
{
	xv_set(bottom_panel,
		WIN_BELOW,top_panel,
		XV_SHOW,TRUE,
		0);
}
Panel get_cmd_panel(Frame frame)
{
	return (Panel) xv_get(frame,FRAME_CMD_PANEL);
}
void show_scroll_panel(Scrollable_panel panel)
{
	xv_set(panel,XV_SHOW,TRUE,0);
}
void show_panel(Panel panel)
{
	xv_set(panel,XV_SHOW,TRUE,0);
}
void init_scrollbar(Scrollable_panel panel)
{
	xv_create(panel,SCROLLBAR,
                SCROLLBAR_DIRECTION,  SCROLLBAR_VERTICAL,
//                XV_SHOW,      TRUE,
                0);
}

void scroll_panel_clear(Scrollable_panel panel)
{
	Panel_item item;

	PANEL_EACH_ITEM(panel,item)
		xv_destroy(item);
	PANEL_END_EACH
}
Panel_item create_panel_button(Scrollable_panel panel,int i,int j,
	int ni,int nj,char* str,void (*proc)(Panel_item))
{
	return xv_create(panel, PANEL_BUTTON,
		XV_X,	xv_col(panel,i)+ni,
		XV_Y,	xv_row(panel,j)+nj,
                PANEL_LABEL_STRING, str,
		PANEL_NOTIFY_PROC, proc,              
		0);
}
Panel_item create_panel_text(Scrollable_panel panel,int i,int j,
	int ni,int nj,char* str,int len)
{
	 return xv_create(panel, PANEL_TEXT,
		XV_X,	xv_col(panel,i)+ni,
		XV_Y,	xv_row(panel,j)+nj,
		PANEL_LABEL_STRING, str,
		PANEL_VALUE_DISPLAY_LENGTH, len,                
		0); 
}

Panel_item create_panel_choice(Scrollable_panel panel,int i,int j,
	int ni,int nj,char* str,int len,
	int index,Panel_setting (*proc)(Panel_item,Event*))
{
	return xv_create(panel, PANEL_CHOICE_STACK,
		XV_X,	xv_col(panel,i)+ni,
		XV_Y,	xv_row(panel,j)+nj,
		PANEL_LABEL_STRING, str,
		PANEL_VALUE_DISPLAY_LENGTH, len,
		PANEL_CLIENT_DATA,index,
		PANEL_NOTIFY_PROC,proc,
//		PANEL_CHOOSE_NONE, TRUE,
		0);		
}
Panel_item create_panel_message(Scrollable_panel panel,int i,int j,
	char *str)
{
	 return xv_create(panel, PANEL_MESSAGE,
		XV_X,	xv_col(panel,i),
		XV_Y,	xv_row(panel,j),
		PANEL_LABEL_STRING, str,
               	0);
}
void set_default_choice(Panel_item item,int index)
{
	xv_set(item,PANEL_DEFAULT_VALUE,index,0);
}
void set_panel_message(Panel_item item,char* text)
{
	xv_set(item,PANEL_LABEL_STRING,text,0);
}
int set_panel_value(Panel_item item,char* text)
{
	xv_set(item,PANEL_VALUE,text,0);
	return 1;
}
int set_panel_value(Panel_item item,int val)
{
	xv_set(item,PANEL_VALUE,val,0);
	return 1;
}
char* get_panel_value(Panel_item item)
{
 	return (char*) xv_get(item,PANEL_VALUE);
}
int get_panel_choice_value(Panel_item item)
{
 	return (int) xv_get(item,PANEL_VALUE);
}
int get_menu_choice_index(int menu_index,char* name)
{
	nsl_layer* layer;
	nsl_data* graph;

	if (menu_index == 1)
	{
	    if ((layer = NSL_MODEL->get_layer(name)) != NULL)
		return layer->get_index()-1; // index start from 0
	    else
	    {
		cmd_error("Unknown layer: ",name);
		return 0;
	    }
	}
	else if (menu_index == 2)
	{
	    if ((graph = NSL_SYSTEM->get_graph(name)) != NULL)
		return graph->get_index()-1;
	    else
	    {
		cmd_error("Unknown graph: ",name);
		return 0;
	    }
	}
	else if (menu_index == 3)
	{
	    if (strcmp("vertical",name) == 0)
		return 1;
	    else
		return 0;
	}
	else if (menu_index == 4)
	{
	    if (strcmp("ON",name) == 0)
		return 1;
	    else
		return 0;
	}
	else
		cmd_out("Unknown menu index: ",menu_index);

	return 0;
}
char* get_menu_text(int menu_index,int choice_index)
{
	nsl_layer* layer;
	nsl_data* graph;
	static char* str1 = "vertical";
	static char* str0 = "horizontal";
	static char* stra1 = "ON";
	static char* stra0 = "OFF";

	if (menu_index == 1)
	{
	    if ((layer = NSL_MODEL->get_layer(choice_index+1)) != NULL)
		return layer->get_name(); // index start from 1
	    else
	    {
		cmd_error("Unknown layer: ",choice_index+1);
		return 0;
	    }
	}
	else if (menu_index == 2)
	{
	    if ((graph = NSL_SYSTEM->get_graph(choice_index+1)) != NULL)
		return graph->get_name();
	    else
	    {
		cmd_error("Unknown graph: ",choice_index+1);
		return 0;
	    }
	}
	else if (menu_index == 3)
	{
	    if (choice_index == 1)
		return str1;
	    else
		return str0;	
	}
	else if (menu_index == 4)
	{
	    if (choice_index == 1)
		return stra1;
	    else
		return stra0;	
	}
	else
		cmd_out("Unknown menu index: ",menu_index);

	return 0;
}

void set_layer_choice_menu(Panel_item item) //(Menu menu)
{
        int i=0;
	char *name;

	layer_obj_list* list = NSL_MODEL->get_layer_list(); 
	
	layer_obj_iterator next(*list);
	nsl_layer* layer;
	while (layer = next()) 
	{
		name = layer->get_name();
		xv_set(item,PANEL_CHOICE_STRING,i++,name,0);
	}
}
char* get_layer_choice(Panel_item item)
{
	int j = 0;
	int i = (int) xv_get(item,PANEL_VALUE);

	layer_obj_list* list = NSL_MODEL->get_layer_list(); 
	
	layer_obj_iterator next(*list);
	nsl_layer* layer;
	while (layer = next())
	{
	    if (j == i)
	    {
		return layer->get_name();
	    }
	    j++;
	}

	return (char*) 0;
}

void set_graph_choice_menu(Panel_item item)
{
        int i=0;
	char *name;

	data_type_list* list = NSL_SYSTEM->get_graph_list(); 
	
	data_type_iterator next(*list);
	nsl_data* graph;
	while (graph = next()) 
	{
		name = graph->get_name();
		xv_set(item,PANEL_CHOICE_STRING,i++,name,0);
	}
}
char* get_graph_choice(Panel_item item)
{
	int j = 0;
	int i = (int) xv_get(item,PANEL_VALUE);

	data_type_list* list = NSL_SYSTEM->get_graph_list(); 
	
	data_type_iterator next(*list);
	nsl_data* graph;
	while (graph = next())
	{
	    if (j == i)
	    {
		return graph->get_name();
	    }
	    j++;
	}

	return (char*) 0;
}
void set_vec_type_choice_menu(Panel_item item)
{
	xv_set(item,PANEL_CHOICE_STRING,0,"horizontal",0); 
	xv_set(item,PANEL_CHOICE_STRING,1,"vertical",0);   
}
void set_on_off_choice_menu(Panel_item item)
{
	xv_set(item,PANEL_CHOICE_STRING,0,"OFF",0); 
	xv_set(item,PANEL_CHOICE_STRING,1,"ON",0);   
}
void set_panel_key_data(Panel panel,int key,int data)
{
	xv_set(panel,XV_KEY_DATA,key,data,0);
}
int get_panel_key_data(Panel panel,int key)
{
	return (int) xv_get(panel,XV_KEY_DATA,key);
}
void update_button_proc(Panel_item item)
{
 	display_frame* frame;    	
	display_window* window;

        Panel panel = xv_get(item,XV_OWNER);
	int fr_index = (int) xv_get(panel,XV_KEY_DATA,1); 
	int win_index = (int) xv_get(panel,XV_KEY_DATA,2); 
	if ((frame = NSL_INTERFACE->get_display_frame(fr_index)) != NULL)
	{
	    if ((window = frame->get_display_window(win_index)) != NULL)
	    	window->update_from_panel();
	    else
		cmd_error("Cannot update window: ",win_index);	
	}
	else
		cmd_error("Cannot update frame: ",fr_index);	
}
Panel_setting panel_choice_handler(Panel_item item,Event* event)
{
 	display_frame* frame;    	
	display_window* window;
	display_panel* panel;
	display_canvas* canvas;

	int index = (int) xv_get(item,PANEL_CLIENT_DATA);

	char* str;
	char* tmp;
	int reset_data_fg = 0;
	panel_object* obj;

	Panel panel_owner = xv_get(item,XV_OWNER);
	int fr_index = (int) xv_get(panel_owner,XV_KEY_DATA,1);
	int win_index = (int) xv_get(panel_owner,XV_KEY_DATA,2);

	if ((frame = NSL_INTERFACE->get_display_frame(fr_index)) != NULL)
	{
	    if ((window = frame->get_display_window(win_index)) != NULL &&
		(panel = window->get_control_panel()) != NULL &&
		(canvas = window->get_window_canvas()) != NULL)
	    {
		if (index == 1)
		{
			tmp = window->get_layer_name();
			obj = panel->get_panel_item("LAYER");
			str = get_layer_choice(item);
//			cmd_out("LAYER: ",str);
			if (strcmp(tmp,str) != 0)
			{
				window->update_window_layer(str);
				reset_data_fg = 1;
			}
		}
		else if (index == 2)
		{
			tmp = window->get_graph_name();
			obj = panel->get_panel_item("GRAPH");
			str = get_graph_choice(item);
//			cmd_out("GRAPH: ",str);
			if (strcmp(tmp,str) != 0)
			{
				window->update_window_graph(str);
				reset_data_fg = 1;
			}
		}
//		else if (index == 3)
//		else
//			cmd_error("unknown index: ",index);

		if (reset_data_fg == 1)
		{
//			window->reset_data(); // resets old values
			window->init_data();
			canvas->reset();
			canvas->init();
//			canvas->update();
			panel->reset_data();
			panel->init_data();
			panel->update();
		}
	    }
	    else
		cmd_error("Cannot update window: ",win_index);	
	}
	else
		cmd_error("Cannot update frame: ",fr_index);	

	return PANEL_NONE;
}
void set_panel_choice_menu(Panel_item item,int index)
{
	if (index == 1)
		set_layer_choice_menu(item);
	else if (index == 2)
		set_graph_choice_menu(item);
	else if (index == 3)
		set_vec_type_choice_menu(item);
	else if (index == 4)
		set_on_off_choice_menu(item);
	else
		cmd_error("unknown index: ",index);
}
