//
//	temporal_graph.c
//

# include "nsl.h"
# include "nsl_headers.h"
# include "nsl_winds.h"

extern nsl_model* NSL_MODEL;

extern void draw_temporal_graph_data(Canvas,num_type,num_type,
	int,int,int,int,
	float,float,float,float,float,float,int);

extern void draw_temporal_graph_vector(Canvas,num_type*,num_type*,
	int,int,int,int,int,int,
	float,float,float,float,float,float,int,int);

extern void draw_temporal_graph_matrix(Canvas,num_type**,num_type**,
	int,int,int,int,int,int,int,int,
	float,float,float,float,float,float,int);

extern void set_menu_canvas_events(Canvas);

// temporal graph
temporal_graph::temporal_graph(display_window* window) 
	: display_graph(window,"temporal_graph")
{
	set_menu_canvas_events(canvas);
	parent_canvas->set_clear_fg(0);
}
void temporal_graph::read_from_database()
{
	string str;

	display_graph::read_from_database();
	parent_window->get_general_data("t0",&t0);
	parent_window->get_general_data("t1",&t1);

	parent_window->get_general_data("pulsefg",str);

	if (strcmp(str,"ON") == 0)
		pulsefg = 1;
	else
		pulsefg = 0;

	if (t1 <= t0)
	{
		t0 = 0.0;
		t1 = 1.0;
		parent_window->set_general_data("t0",t0);
		parent_window->set_general_data("t1",t1);
	}
}
void temporal_graph::write_to_database()
{
	display_graph::write_to_database();
}
void temporal_graph::input(int x_pos,int y_pos,int fg)
{
}
void temporal_graph::output(int x_pos,int y_pos)
{
}
void temporal_graph::display()
{
}
void temporal_graph::print(ostream& out)
{
	canvas_object::print(out);
}

// par
temporal_graph_par::temporal_graph_par(display_window* window,nsl_data* m) 
	: temporal_graph(window)
{
	layer_ptr = m;
	layer0_ptr = new nsl_data;
}
void temporal_graph_par::read_from_database()
{
	parent_window->set_win_sector(0,0,0,0,1,1); // update win sector
	temporal_graph::read_from_database(); // read in new win sector
}
void temporal_graph_par::write_to_database()
{
	temporal_graph::write_to_database();
}
void temporal_graph_par::display()
{
	float ts = NSL_MODEL->get_time();
	float dt = NSL_MODEL->get_delta();

	num_type m0 = ((nsl_data*) layer0_ptr)->get_data();
	num_type m = ((nsl_data*) layer_ptr)->get_data();

	draw_temporal_graph_data(canvas,m,m0,
		wx0,wy0,wxs,wys,wymin,wymax,t0,t1,ts,dt,pulsefg);

	((nsl_data*) layer0_ptr)->set_value(m);
}
void temporal_graph_par::print(ostream& out)
{
	temporal_graph::print(out);
}

// vec
temporal_graph_vec::temporal_graph_vec(display_window* window,nsl_vector* m) 
	: temporal_graph(window)
{
	layer_ptr = m;
	imax = ((nsl_vector*) layer_ptr)->get_imax();
	layer0_ptr = new nsl_vector(imax);
}
void temporal_graph_vec::read_from_database()
{
	parent_window->get_general_data("i0",&i0);
	parent_window->get_general_data("i1",&i1);

	imax = ((nsl_vector*) layer_ptr)->get_imax();

	parent_window->get_general_data("vec_type",&vec_type); 

	if (i0 < 0) 
	{
		i0 = 0;
		parent_window->set_general_data("i0",i0);
	}
	if (i1 > imax-1) 
	{
		i1 = imax-1;
		parent_window->set_general_data("i1",i1);
	}
	if (i1 < i0)
	{
		i0 = 0;
		i1 = imax-1;
		parent_window->set_general_data("i0",i0);
		parent_window->set_general_data("i1",i1);
	}

	if (vec_type == 0)
	   parent_window->set_win_sector(0,i0,0,i1,1,imax); // update win sector
	else
	   parent_window->set_win_sector(i0,0,i1,0,imax,1); // update win sector
	temporal_graph::read_from_database(); // read in new win sector
}
void temporal_graph_vec::write_to_database()
{
	temporal_graph::write_to_database();
}
void temporal_graph_vec::display()
{
	float ts = NSL_MODEL->get_time();
	float dt = NSL_MODEL->get_delta();

	num_type* m0 = ((nsl_vector*) layer0_ptr)->get_vector();
	num_type* m = ((nsl_vector*) layer_ptr)->get_vector();

	draw_temporal_graph_vector(canvas,m,m0,
		wx0,wy0,wxs,wys,i0,i1,wymin,wymax,t0,t1,ts,dt,pulsefg,vec_type);

	for (int i = 0; i < imax; i++)
		m0[i] = m[i];
}
void temporal_graph_vec::print(ostream& out)
{
	temporal_graph::print(out);
}

// mat
temporal_graph_mat::temporal_graph_mat(display_window* window,nsl_matrix* m) 
	: temporal_graph(window)
{
	layer_ptr = m;

	imax = ((nsl_matrix*) layer_ptr)->get_imax();
	jmax = ((nsl_matrix*) layer_ptr)->get_jmax();

	layer0_ptr = new nsl_matrix(imax,jmax);
}
void temporal_graph_mat::read_from_database()
{
	parent_window->get_general_data("i0",&i0);
	parent_window->get_general_data("i1",&i1);
	parent_window->get_general_data("j0",&j0);
	parent_window->get_general_data("j1",&j1);

	imax = ((nsl_matrix*) layer_ptr)->get_imax();
	jmax = ((nsl_matrix*) layer_ptr)->get_jmax();

	if (i0 < 0) 
	{
		i0 = 0;
		parent_window->set_general_data("i0",i0);
	}
	if (i1 > imax-1) 
	{
		i1 = imax-1;
		parent_window->set_general_data("i1",i1);
	}
	if (i1 < i0)
	{
		i0 = 0;
		i1 = imax-1;
		parent_window->set_general_data("i0",i0);
		parent_window->set_general_data("i1",i1);
	}

	if (j0 < 0) 
	{
		j0 = 0;
		parent_window->set_general_data("j0",j0);
	}
	if (j1 > jmax-1) 
	{
		j1 = jmax-1;
		parent_window->set_general_data("j1",j1);
	}
	if (j1 < j0)
	{
		j0 = 0;
		j1 = jmax-1;
		parent_window->set_general_data("j0",j0);
		parent_window->set_general_data("j1",j1);
	}

	parent_window->set_win_sector(i0,j0,i1,j1,imax,jmax); // update win sector
	temporal_graph::read_from_database(); // read in new win sector
}
void temporal_graph_mat::write_to_database()
{
	temporal_graph::write_to_database();
}
void temporal_graph_mat::display()
{
	float ts = NSL_MODEL->get_time();
	float dt = NSL_MODEL->get_delta();

	num_type** m0 = ((nsl_matrix*) layer0_ptr)->get_matrix();
	num_type** m = ((nsl_matrix*) layer_ptr)->get_matrix();

	draw_temporal_graph_matrix(canvas,m,m0,
		wx0,wy0,wxs,wys,i0,i1,j0,j1,wymin,wymax,t0,t1,ts,dt,pulsefg);

	for (int i = 0; i < imax; i++)
	   for (int j = 0; j < jmax; j++)
		m0[i][j] = m[i][j];
}
void temporal_graph_mat::print(ostream& out)
{
	temporal_graph::print(out);
}
