/****************************************************************/
/*                                                           	*/
/*                       window_interface.c                     */
/*                                                            	*/
/* by Alfredo Weitzenfeld 					*/
/*                                                            	*/
/****************************************************************/

# include "nsl.h"
# include "nsl_headers.h"
# include "nsl_winds.h"

extern nsl_model* NSL_MODEL;

window_interface* NSL_INTERFACE;
display_frame* NSL_FRAME;
display_window* NSL_WINDOW;
display_canvas* NSL_CANVAS;
display_panel* NSL_PANEL;
nsl_data* NSL_WINDOW_ITEM;
canvas_object* NSL_CANVAS_ITEM;
panel_object* NSL_PANEL_ITEM;

extern "C" void init_win_interrupt_handler();

window_interface::window_interface(char* str)
{
	strcpy(model_name,str);

	NSL_INTERFACE = this;
	nsl_frame_list = new gen_window_list; 
	frame_total = 0;
}
window_interface::window_interface()
{
	NSL_INTERFACE = this;
	nsl_frame_list = new gen_window_list; 
	frame_total = 0;
}

window_interface::~window_interface()
{
	if (nsl_frame_list != NULL)
	{
	    gen_window_iterator next(*nsl_frame_list);
	    generic_window* win;
	    while (win = next())
	    {
//		cmd_out("DELETE FRAME : ",win->get_index());
		delete ((display_frame*) win);
	    }
	}

	frame_total = 0;
	nsl_frame_list = 0;
}

int window_interface::add_frame(display_frame* fr)
{
	++frame_total;

	fr->set_index(frame_total); // ++frame_total

	if (nsl_frame_list != NULL)
		nsl_frame_list->append(fr);
	else
		cmd_error_exit("NULL nsl_frame_list");

	return frame_total;
}

void window_interface::update()
{
	if (nsl_frame_list != NULL)
	{
	    gen_window_iterator next(*nsl_frame_list);
	    generic_window* win;
	    while (win = next()) 
		((display_frame*) win)->update();
	}
}
void window_interface::display()
{
	if (nsl_frame_list != NULL)
	{
	    gen_window_iterator next(*nsl_frame_list);
	    generic_window* win;
	    while (win = next()) 
		((display_frame*) win)->display();
	}
}
void window_interface::clear()
{
	if (nsl_frame_list != NULL)
	{
	    gen_window_iterator next(*nsl_frame_list);
	    generic_window* win;
	    while (win = next()) 
		((display_frame*) win)->clear();
	}
}

// obtain window from window list and window name
generic_window* get_window_obj(gen_window_list* list,char* name)
{
	if (list == NULL)
	{
		cmd_error("NULL list");
		return (generic_window*) 0;
	}

	char* str;

	if (list != NULL)
	{
	    gen_window_iterator next(*list);
	    generic_window* win;

	    while (win = next())
	    {
		str = win->get_name();
		if (strcmp(name,str) == 0)
			return win;
	    }
	}

	return (generic_window*) 0;
}

// obtain window from window list and window index
generic_window* get_window_obj(gen_window_list* list,int index)
{
	if (list == NULL)
	{
		cmd_error("NULL list");
		return (generic_window*) 0;
	}

	if (list != NULL)
	{
	    gen_window_iterator next(*list);
	    generic_window* win;

	    while (win = next())
	    {
		if (index == win->get_index())
			return win;
	    }
	}

	return (generic_window*) 0;
}
display_frame* window_interface::get_display_frame(char* str)
{
	return (display_frame*) get_window_obj(nsl_frame_list,str);
}
display_frame* window_interface::get_display_frame(int index)
{
	return (display_frame*) get_window_obj(nsl_frame_list,index);
}
display_window* window_interface::get_display_window(char* str)
{
	if (nsl_frame_list == NULL)
		return (display_window*) 0;

	if (nsl_frame_list != NULL)
	{
	    gen_window_list* win_list;
	    gen_window_iterator next(*nsl_frame_list);
	    generic_window* fr_win;
	    generic_window* win;

	    while (fr_win = next())
	    {
		win_list = ((display_frame*) fr_win)->get_window_list();
		if ((win = get_window_obj(win_list,str)) != NULL)
			return (display_window*) win;
	    }
	}

	return (display_window*) 0;
}
display_window* window_interface::get_display_window(int fr_index,int win_index)
{
	display_frame* fr = 0;
	display_window* win = 0;
	gen_window_list* win_list;

	if ((fr = (display_frame*) get_window_obj(nsl_frame_list,fr_index)) != NULL)
	{
		win_list = fr->get_window_list();
		win = (display_window*) get_window_obj(win_list,win_index);
	}

	return win;
}

void window_interface::print_status(ostream& out)
{
	out << "// frame total: " << frame_total << "\n";
	out.flush();
}
void window_interface::print_frame_status(ostream& out)
{
	if (nsl_frame_list != NULL)
	{
	    gen_window_iterator next(*nsl_frame_list);
	    generic_window* win;
	    while (win = next()) 
		((display_frame*) win)->print_status(out);
	}
}
void window_interface::print_frame_status(ostream& out,char* name)
{
	display_frame* win;
	if ((win = (display_frame*) get_display_frame(name)) != NULL)
		win->print_status(out);
	else
		cmd_error("Unknown Frame: ",name);
}
void window_interface::print_frame_status(ostream& out,int index)
{
	display_frame* win;
	if ((win = (display_frame*) get_display_frame(index)) != NULL)
		win->print_status(out);
	else
		cmd_error("Unknown Frame Index: ",index);
}
void window_interface::print_window_status(ostream& out)
{
	gen_window_list* list;

	if (nsl_frame_list != NULL)
	{
	    gen_window_iterator fr_next(*nsl_frame_list);
    	    generic_window* fr;
	    while (fr = fr_next())
	    { 
		if ((list = ((display_frame*) fr)->get_window_list()) != NULL)
		{
	    	    gen_window_iterator win_next(*list);
		    generic_window* win;
	    	    while (win = win_next())
			((display_window*) win)->print_status(out);
		}
	    }
	}
}
void window_interface::print_window_status(ostream& out,char* name)
{
	display_window* win;
	if ((win = (display_window*) get_display_window(name)) != NULL)
		win->print_status(out);
	else
		cmd_error("Unknown Window: ",name);
}
void window_interface::print_window_status(ostream& out,
	int fr_index,int win_index)
{
	display_window* win;
	if ((win = 
	   (display_window*) get_display_window(fr_index,win_index)) != NULL)
		win->print_status(out);
	else
		cmd_error("Unknown Window");
}

