#include "nsl_include.h"
#include "math.h"

//</PRE><A NAME="defs">Definitions:</A><PRE>
#define F1N 100
#define EYEMF 10
#define KNOWN 10
#define PROPRION 10
#define MF_DIV 6
#define BOUTONOF 180   // BOUTONOF = (EYEMF + KNOWN + PROPRION)*MF_DIV
#define PCNUMBEROF 10
#define DOGOF 9
 
#define COR_COR_MASKSIZE 20
#define THA_COR_MASKSIZE 5
#define SEED 1

extern  float ran1(int *);
extern 	nsl_matrix gaussm(int , int , nsl_data &, nsl_data &, nsl_data &);
extern 	nsl_vector gaussv(int , nsl_data &,  nsl_data &);

NETWORK(DART);

FILE * fopen(), *fpteta, *fpwf;

DATA(n);	// number of runs for the learning
DATA(n_cycles); // number of training cycles
DATA(prism_angle); // prism deflection angle
VECTOR(throw_direction,150); // Stores the resulting throw error
DATA(n_arm_proprio); // number of neurons coding arm position
DATA(n_know); // number of neurons coding prism knowledge
int rn;

int	 seed = SEED;	// random number initialization

DATA(wok);
DATA(birth);

//</PRE><A NAME="norm">Normalization</A><PRE>

DATA(ACT_MAX);
DATA(VP);
DATA(VM);
DATA(wgain_err);
DATA(errP);
DATA(errM);
VECTOR(ERRP, F1N);
VECTOR(ERRM, F1N);

//</PRE><A NAME="ccint">Cerebellar+cortical interaction</A><PRE>

DATA(eye_dir);
VECTOR(spindle,F1N);
VECTOR(spindleDOGged,F1N);
VECTOR(wspi_pre,COR_COR_MASKSIZE);
DATA(alpha_spi_pre);
DATA(sigma_spi_pre);
DATA(pos_height);
DATA(cst);

VECTOR(premotor,F1N);
VECTOR(PREMOTOR,F1N);
VECTOR(wpre_mot,COR_COR_MASKSIZE);

VECTOR(f1, F1N);
VECTOR(F1, F1N);
VECTOR(wF1,THA_COR_MASKSIZE);
VECTOR(wF1p,THA_COR_MASKSIZE);
VECTOR(wF1m,THA_COR_MASKSIZE);
DATA(alpha_F1p);
DATA(sigma_F1p);
DATA(alpha_F1m);
DATA(sigma_F1m);
DATA(gain);

VECTOR(tha_pre, F1N);
VECTOR(THA_PRE, F1N);

VECTOR(tha_mot, F1N);
VECTOR(THA_MOT, F1N);
VECTOR(wthap,THA_COR_MASKSIZE);
VECTOR(wtham,THA_COR_MASKSIZE);
VECTOR(wtha,THA_COR_MASKSIZE);
DATA(alpha_thap);
DATA(sigma_thap);
DATA(alpha_tham);
DATA(sigma_tham);

VECTOR(auxt,F1N);
VECTOR(wnu_tha, COR_COR_MASKSIZE);
VECTOR(wtha_pre, COR_COR_MASKSIZE);
DATA(alpha_nu_tha);
DATA(sigma_nu_tha);

DATA(f1_tau);
DATA(tha_tau);

DATA(Ps);
DATA(tetas);
DATA(P);
DATA(teta);

float deg;

//</PRE><A NAME="mfib" HREF="../doc/paper.html#d6">Mossy Fibers: Eye-pos, Arm Proprioception, Cortical Inputs</A><PRE>

DATA(deg_min);
DATA(pos_step);
VECTOR(cortico,KNOWN);
VECTOR(proprio,PROPRION);		
VECTOR(bouton, BOUTONOF);
VECTOR(boutonDOGged, BOUTONOF);
DATA(boutonthresh);
VECTOR(dogp,DOGOF);
VECTOR(dogm,DOGOF);
VECTOR(dog,DOGOF);
DATA(alpha_dogp);
DATA(sigma_dogp);
DATA(alpha_dogm);
DATA(sigma_dogm);

//</PRE><A NAME="gcs">Golgi Cells</A><PRE>

VECTOR(gc, 1000);
DATA(wmf_gc);
DATA(gcthresh);
DATA(gcmax);
VECTOR(gcinp, 1000);
VECTOR(GC, 1000);
VECTOR(GC1, 1000);	
DATA(gc1);		// step parameters for gc 
DATA(GCNUMBEROF);
DATA(cor);
DATA(corok);

//</PRE><A NAME="modwts">Modifiable Weights</A><PRE>
  
MATRIX(wltd, 1000, PCNUMBEROF);
DATA(w_ini_width);
DATA(w_ini_mean);
DATA(wthresh);
	
//</PRE><A NAME="pkc2">Synapse and pkc2 Messenger System</A><PRE>
		
//pkc2 messengers:two 1st order equa diffs make one 2 nd order

MATRIX(Hebb, 1000, PCNUMBEROF);
MATRIX(pkc1, 1000, PCNUMBEROF);
MATRIX(pkc2, 1000, PCNUMBEROF);
MATRIX(PKC, 1000, PCNUMBEROF);	//plus step

//</PRE><A NAME="pcs">Purkinje Cells</A><PRE>
  
VECTOR(auxp, PCNUMBEROF);
VECTOR(pc, PCNUMBEROF);
VECTOR(PC, PCNUMBEROF);		

//</PRE><A NAME="nuc">Nuclear Cells</A><PRE>
//nuclear noise and weights
	
DATA(B);
DATA(i_nu);

VECTOR(nuc, PCNUMBEROF);
VECTOR(NUC, PCNUMBEROF);
VECTOR(wpc_nu,5);
DATA(alpha_nu);
DATA(sigma_nu);
DATA(nucx);
DATA(nucy);
	
//</PRE><A NAME="ios">Inferior Olive</A><PRE>

VECTOR(CF,PCNUMBEROF);
DATA(io);
DATA(Prob);
DATA(cftime);
DATA(ang_span_cf);
DATA(errlimite);  // in degrees
DATA(firing_pres);
 
//</PRE><A NAME="tcs">Time Constants</A><PRE>

DATA(gc_tau);
DATA(pc_tau);
DATA(nuc_tau);
DATA(io_tau);
		
//</PRE><A NAME="elp">Eligibility Parameters</A><PRE>

DATA(tau2);
DATA(tau3);
DATA(K);
DATA(thresh);
DATA(val);

//</PRE><A NAME="lcfs">Learning Coefficients</A><PRE>

DATA(a);
DATA(b);

//</PRE><A NAME="sigspec">Sigmoid Specifications</A><PRE>

DATA(kx2);
DATA(ky2);

//</PRE><A NAME="rcg" HREF="../doc/paper.html#d2">The random connections generator</A><PRE>

MATRIX(connect,1000,4);


nsl_matrix synapse_add() {
    int p;
    nsl_matrix address(1000, 4);
    
    for (int i=0; i< GCNUMBEROF.elem(); i++) {
	for (int j=0; j< 4; j++) {
	    p = int(ran1(&seed)*BOUTONOF);
	    address.elem(i,j) = p;
	}					
    }
    return address;
}

	
// and this is the value assignment used each time gc is needed

nsl_vector gcint( nsl_matrix &connect, nsl_vector &bouton){
    nsl_vector gclocal(1000);
    float aux;
    
    for (int i=0; i< GCNUMBEROF.elem(); i++) {
	aux =0;
	for (int j=0; j< 4; j++) {
	    aux = aux + bouton.elem(int(connect.elem(i,j)));
	} 
	gclocal.elem(i) = aux;
    }
    return gclocal;
}

// Save modifiable weights
 
void WriteWeights(FILE * wf)
{
    float tf;

    for (int i = 0; i < GCNUMBEROF.elem(); i++) {
	for (int j = 0; j < PCNUMBEROF; j++) {
	    tf = wltd.elem(i, j);
	    fwrite((char *)&tf, sizeof(float), 1, wf); 
	}
    }
}

// Load saved weights
 
void ReadWeights(FILE * wf)
{
  float tf;

  for (int i = 0; i < GCNUMBEROF.elem(); i++) {
    for (int j = 0; j < PCNUMBEROF; j++) {
      fread((char *)&tf, sizeof(float), 1, wf); 
      wltd.elem(i, j) = tf;
    }
  }
}

//</PRE><A NAME="initm">Init Module</A><PRE>

INIT_MODULE(ini){

  f1 = 0;
  F1 = 0;
  premotor = 0;
  PREMOTOR = 0;
  tha_mot = 0;
  THA_MOT = 0;
  tha_pre = 0;
  THA_PRE = 0;
  
  gc = 0;
  GC = 0;
  pc = 0;
  PC = 0;
  nuc = 0;
  NUC = 0;
  pkc1 = 0;
  pkc2 = 0;
  PKC = 0;

}

RUN_MODULE(update_proprio){
    int   i;
    float tf, tf2;

    tf2 = teta.elem() + eye_dir.elem();
    // Use a Gaussian response for each cell around its preferred direction
    for(i=0;i<PROPRION;i++){
	// tf is cell's prefered direction
	tf = (-90.0 + 180.0 * (float) i / PROPRION);
	proprio.elem(i) = 50.0 * exp(-1.0 * (tf2-tf)*(tf2-tf)
				     /400.0);
	// Let's use a std-dev of 10 deg...
    }
}

//</PRE><A NAME="norm_pre" HREF="../doc/paper.html#d12">Normalization Premotor</A><PRE>

RUN_MODULE(normalization_pre){
 
    errP = THA_PRE.sum() - PREMOTOR.sum();

    if (errP.elem() > 0) {
	for (int i= 0; i< F1N; i++)
	    ERRP.elem(i) = errP.elem()/F1N;
    }
    else 
	ERRP = 0;
}

//</PRE><A NAME="thala_pre" HREF="../doc/paper.html#d7">Thalamus Premotor</A><PRE>

RUN_MODULE(thala_premotor) {

  auxt = 0;
  for (int i = 0; i < F1N; i++){
    auxt.elem(i)= NUC.elem(int((i*PCNUMBEROF)/F1N));	
  }
  	
  DIFF.eq(tha_pre,tha_tau) = -tha_pre + NSLconv_copy(wnu_tha,auxt) 
    + NSLconv_wrap(wF1,PREMOTOR) - wgain_err*ERRP;

  THA_PRE = NSLsaturation(tha_pre,0,kx2.elem(),0,ky2.elem());
}

//</PRE><A NAME="pre_cort" HREF="../doc/paper.html#d1">Premotor Cortex</A><PRE>

RUN_MODULE(premotor_cortex) {
  
  DIFF.eq(premotor,f1_tau) = -premotor + NSLconv_wrap(wtha,THA_PRE) 
    + wspi_pre*spindle;

  PREMOTOR = NSLsaturation(premotor,0,kx2.elem(),0,ky2.elem());
}

//</PRE><A NAME="thal_mot">Thalamus Motor</A><PRE>

RUN_MODULE(thala_motor) {
	
  DIFF.eq(tha_mot,f1_tau) = -tha_mot + NSLconv_wrap(wF1,F1) - wgain_err*ERRM;
  
  THA_MOT= NSLsaturation(tha_mot,0,kx2.elem(),0,ky2.elem());
}

//</PRE><A NAME="norm_mot" HREF="../doc/paper.html#d12">Normalization Motor</A><PRE>

RUN_MODULE(normalization_mot){

  errM = THA_MOT.sum() - F1.sum();

  if (errM.elem() > 0) {
    for (int i= 0; i< F1N; i++)
      ERRM.elem(i) = errM.elem()/F1N;
  }
  else 
    ERRM = 0;
}	

//</PRE><A NAME="motor">Motor</A><PRE>

RUN_MODULE(motor) {
	
  DIFF.eq(f1,f1_tau) = -f1 + wpre_mot*NSLconv_wrap(wpre_mot,PREMOTOR) +
    NSLconv_wrap(wtha,THA_MOT) ;

  F1 = NSLsaturation(f1,0,kx2.elem(),0,ky2.elem());
}

//</PRE><A NAME="cort_shft" HREF="../doc/paper.html#d7">Cortex Shift</A><PRE>

RUN_MODULE(cortex_shift){
  double	px = 0;
  double	py =0;

  for (int k = 0; k< F1N; k++) {		
    deg = (-180 + 3.6*k)* 3.1416/180;
    px = px + PREMOTOR.elem(k)*cos(deg);
    py = py + PREMOTOR.elem(k)*sin(deg);
  }
  P = sqrt(pow(px,2) + pow(py,2));
  teta = atan2(py, px) *180/3.1416 - eye_dir.elem(); // Jacob's hack
}

//</PRE><A NAME="dyn_gran" HREF="../doc/paper.html#d2">Dynamic Granule</A><PRE>

RUN_MODULE(dynamic_granule) {

  // granule cells are AND gates 

  nsl_vector eyemf(EYEMF);

  int count = 0;

  for (int n = 0; n< EYEMF; n++) {
    eyemf.elem(n) = NSLramp(spindle.elem(int(n*F1N/EYEMF)),(100*cst.elem()));
  }	
		
  for (int k =0; k< (EYEMF*MF_DIV); k++) {
    bouton.elem(k) = eyemf.elem(int(k/(MF_DIV)));
  }

  for (int l =(EYEMF*MF_DIV); l< (KNOWN*MF_DIV + EYEMF*MF_DIV); l++) {
    bouton.elem(l) = cortico.elem(int((l/MF_DIV) - EYEMF));
  }

  for (int m =(KNOWN*MF_DIV + EYEMF*MF_DIV);
       m< (KNOWN*MF_DIV + EYEMF*MF_DIV + PROPRION*MF_DIV); m++) {
    bouton.elem(m) = proprio.elem(int((m/MF_DIV) - KNOWN - EYEMF));
  }
  
  boutonDOGged = 
    NSLsaturation((dog*bouton),boutonthresh.elem(),kx2.elem(),0,ky2.elem());

  DIFF.eq(gc, gc_tau) = -gc + wmf_gc*gcint(connect, boutonDOGged);
			
  GC = NSLstep(gc,gcthresh.elem(),0,gcmax.elem());

}
		
//</PRE><A NAME="purk" HREF="../doc/paper.html#d4">Purkinje</A><PRE>

RUN_MODULE(Purkinje) {

  // Purkinje cell activity

  auxp = 0;
	
  for (int n = 0; n < PCNUMBEROF; n++){
    for (int m = 0; m < GCNUMBEROF.elem(); m++){
      auxp.elem(n) =  auxp.elem(n) + wltd.elem(m,n)*GC.elem(m);
    }
  }
		
  DIFF.eq(pc, pc_tau) = -pc + auxp;
  PC = NSLsaturation(pc,0,kx2.elem(),0,ky2.elem());
}			
				
//</PRE><A NAME="nucl" HREF="../doc/paper.html#d5">Nucleus</A><PRE>

RUN_MODULE(NUCLEUS) {

  DIFF.eq(nuc, nuc_tau) = -nuc - NSLconv_copy(wpc_nu,PC) + B;
  NUC = NSLsaturation(nuc,0,nucx.elem(),0,nucy.elem());
}

//</PRE><A NAME="eligib" HREF="../doc/paper.html#d9">Eligibility</A><PRE>

RUN_MODULE(eligibility) {

  //1st equation of 2nd messenger activated by coactivation of gc and PCs

  for (int n = 0; n < int(GCNUMBEROF.elem()); n++){
    for (int m = 0; m < int(PCNUMBEROF); m++){
      Hebb.elem(n,m) = GC.elem(n)*PC.elem(m);
    }
  }
		
  DIFF.eq(pkc1, tau2) = -pkc1 + K.elem() * Hebb;

  //2nd messenger 2nd equation

  DIFF.eq(pkc2, tau3) = -pkc2 + pkc1;
  PKC = NSLramp(pkc2, thresh.elem());
}

//</PRE><A NAME="io" HREF="d8">Inferior Olive</A><PRE>

RUN_MODULE(IO) {	
  float t = DART.get_time();
  float a;
	 
  if ((t > cftime.elem()) &&
      (teta.elem() > errlimite.elem())) {
    for (int i=0; i < int(PCNUMBEROF/2); i++) {
      a = teta.elem() + (ang_span_cf.elem()/(PCNUMBEROF)*i 
			 - (ang_span_cf.elem()/2));
				
      if (ran1(&seed) < (Prob.elem()* cos(firing_pres.elem()*a*3.1416/180))) {
	CF.elem(i) = 1;
      }
    }
    cout << "CF  CF" << CF << "\n";
    cout << P << "\t" << teta << "\n";
  }
  else if ((t > cftime.elem()) &&	
	   (teta.elem() < - errlimite.elem())) {
    // PCNUMBEROF+1 changed to PCNUMBER because of range prob 7/96 dj
    for (int i=int(PCNUMBEROF/2)  ; i< int(PCNUMBEROF ); i++) {
      a = teta.elem() + (ang_span_cf.elem()/(PCNUMBEROF)*i 
			 - (ang_span_cf.elem()/2));
      if (ran1(&seed) < (Prob.elem()* cos(firing_pres.elem()*a*3.1416/180))) {
	CF.elem(i) = 1;
      }
    }
    cout << "CF  " << CF << "\n";
    cout << P << "\t" << teta << "\n";
  }
  else  {
    CF = 0;
  }
}

//</PRE><A NAME="lrl" HREF="../doc/paper.html#d10">Learning Rule</A><PRE>

RUN_MODULE(learning_rule) {	

  for (int k = 0; k < PCNUMBEROF; k++){
    for (int m = 0; m < GCNUMBEROF.elem(); m++){
      wltd.elem(m,k) = wltd.elem(m,k) - a.elem()*CF.elem(k)*PKC.elem(m,k)
	+ b.elem()*(1 - CF.elem(k))*PKC.elem(m,k);
    }
  }
  wltd = NSLsaturation(wltd, 0 , wthresh.elem(), 0, wthresh.elem());

}

//</PRE><A NAME="dart">Dart</A><PRE>

FUNC_MODULE(dart){

    int exit_flag = 0;

    // from a run to another: reinitialize the weights and connections or not?
    
    if (birth.elem() > .5){
	connect = synapse_add();
    }
    
    if (wok.elem() > .5 ){
	for (int i = 0; i < GCNUMBEROF.elem(); i++) {
	    for (int j = 0; j < PCNUMBEROF; j++) {
		wltd.elem(i, j) = w_ini_width.elem() * (ran1(&seed) - .5) 
		    + w_ini_mean.elem();
	    }
	}
    }
  
    if (cor.elem() > .5)
	GC1 = GC;
    
    // generation of the different masks
    
    wF1p =  gaussv(THA_COR_MASKSIZE, alpha_F1p, sigma_F1p);
    wF1m =  gaussv(THA_COR_MASKSIZE, alpha_F1m, sigma_F1m);
    wF1 = wF1p - wF1m;
    
    wthap = gaussv(THA_COR_MASKSIZE, alpha_thap, sigma_thap);
    wtham = gaussv(THA_COR_MASKSIZE, alpha_tham, sigma_tham);
    wtha = wthap - wtham;
    
    wpc_nu = gaussv(5, alpha_nu, sigma_nu);
    wnu_tha = gaussv(COR_COR_MASKSIZE, alpha_nu_tha, sigma_nu_tha);
    wspi_pre = gaussv(COR_COR_MASKSIZE, alpha_spi_pre, sigma_spi_pre);
    
    wpre_mot = gain*wspi_pre;
    
    dogp =  gaussv(DOGOF, alpha_dogp, sigma_dogp);
    dogm =  gaussv(DOGOF, alpha_dogm, sigma_dogm);
    dog = dogp - dogm;
    
    // run the simulation n times
    
//    fpteta = fopen("teta.m", "w");
//    fpwf = fopen("weights.dat", "w");
    
    float flag = 1;
    
    for(int i =0; i < 8; i++)
	cortico.elem(i) = 40.0;

    for (int m = 0; (m < n_cycles.elem())&&(!exit_flag); m++) {
	flag = flag*(-1);
	if (flag > 0){
	    eye_dir = prism_angle;
	    for(int i = 8;i < KNOWN;i++)
		if(i < KNOWN/2 + 4)
		    cortico.elem(i) = 40.0;
		else
		    cortico.elem(i) = 0.0;
	}
	else{
	    eye_dir = 0.0;
	    for(int i = 8;i < KNOWN;i++)
		if(i < KNOWN/2 + 4)
		    cortico.elem(i) = 0.0;
		else
		    cortico.elem(i) = 40.0;
	}
	cout << "m  " << m << " Body direction    " << eye_dir.elem() << "\n";
	
	// response of spindle to muscle contraction:
	//    cosine ditribution (generalization)
	
	for (int k = 0; k< F1N; k++) {			
	    
	    // response of spindle to muscle contraction: 
	    // gaussian ditribution (generalization)
	    
	    deg = -180 + 3.6*k; // prefered cell direction
	    
	    spindle.elem(k) =  // Change this!
		NSLsaturation((pos_height.elem()*(cos((0.0 * eye_dir.elem() 
						       - deg)*3.1416/180))),
			      0,kx2.elem(),0,ky2.elem());
	}
	
	// calcul of teta for population
	
	double	pxs = 0;
	double	pys = 0;
	
	for (int j = 0; j < F1N; j++) {	
	    deg = (-90 + 1.8*j)* 3.1416/180;
	    
	    pxs = pxs + spindle.elem(j)*cos(deg);
	    pys = pys + spindle.elem(j)*sin(deg);
	}
	
	Ps = sqrt(pow(pxs,2) + pow(pys,2));
	tetas = atan2(pys, pxs) *180/3.1416;
	
	cout << Ps << "\t" << tetas << "\n";
	
	rn = 0;

	while((rn < n.elem())&&(!exit_flag)) {
	    cout << "n   " << rn << "\n";
	    
	    NSL << "run" << ENDL;
	    
            if(DART.get_time() < DART.get_end_time() - DART.get_delta()){
              exit_flag = 1;
            }

//	    fprintf(fpteta,"%7.4f\t%7.4f\n", n.elem(), teta.elem());
	    
	    throw_direction.elem(rn + m*(int)n.elem()) = teta.elem();
	    rn = rn + 1;
	    
	    float gcsum;
	    
	    gcsum = GC.sum();
	    
	    //		cout<<gcsum<<"\n";
	    
	    cout << P << " Teta: " << teta << "\n";      
	}
	
	if (corok.elem() > .5) {
	    cor = NSLprod(GC,GC1);
	    cout << cor << "\n";
	}
//	WriteWeights(fpwf); 
    }
//    fclose(fpteta);
//    fclose(fpwf);
}

FUNC_MODULE(dart2){ // Over- under-hand experiment
    int i,j;
    int exit_flag = 0;

    // from a run to another: reinitialize the weights and connections or not?
    
    if (birth.elem() > .5){
	connect = synapse_add();
    }
    
    if (wok.elem() > .5 ){
	for (i = 0; i < GCNUMBEROF.elem(); i++) {
	    for (j = 0; j < PCNUMBEROF; j++) {
		wltd.elem(i, j) = w_ini_width.elem() * (ran1(&seed) - .5) 
		    + w_ini_mean.elem();
	    }
	}
    }
  
    if (cor.elem() > .5)
	GC1 = GC;
    
    // generation of the different masks
    
    wF1p =  gaussv(THA_COR_MASKSIZE, alpha_F1p, sigma_F1p);
    wF1m =  gaussv(THA_COR_MASKSIZE, alpha_F1m, sigma_F1m);
    wF1 = wF1p - wF1m;
    
    wthap = gaussv(THA_COR_MASKSIZE, alpha_thap, sigma_thap);
    wtham = gaussv(THA_COR_MASKSIZE, alpha_tham, sigma_tham);
    wtha = wthap - wtham;
    
    wpc_nu = gaussv(5, alpha_nu, sigma_nu);
    wnu_tha = gaussv(COR_COR_MASKSIZE, alpha_nu_tha, sigma_nu_tha);
    wspi_pre = gaussv(COR_COR_MASKSIZE, alpha_spi_pre, sigma_spi_pre);
    
    wpre_mot = gain*wspi_pre;
    
    dogp =  gaussv(DOGOF, alpha_dogp, sigma_dogp);
    dogm =  gaussv(DOGOF, alpha_dogm, sigma_dogm);
    dog = dogp - dogm;
    
    // run the simulation n times
    
//    fpteta = fopen("teta.m", "w");
//    fpwf = fopen("weights.dat", "w");
    
    for (int m = 0; (m < 5)&&(!exit_flag); m++) {
	// m=0: Throw without prisms - overhand
	// m=1: Throw without prisms - underhand
	// m=2: Throw with prisms - overhand
	// m=3: Throw without prisms - underhand
	// m=4: Throw without prisms - overhand

	// Take care of eye_dir
	if(m == 2)
	    eye_dir = prism_angle;
	else
	    eye_dir = 0;

	// Take care of inputs relating to over/under
	if(!(m%2))
	    for(i = 0; i < (int)n_arm_proprio.elem(); i++)
		cortico.elem(i) = 40.0;
	else
	    for(i = 0; i < (int)n_arm_proprio.elem(); i++)
		cortico.elem(i) = 0.0;

	// Do the in-between stuff
	for(i = (int)n_arm_proprio.elem(); i < KNOWN - (int)n_know.elem(); i++)
	    cortico.elem(i) = 40.0;

	// Take care of "prism knowledge"
	if(m == 2)
	    for(i = KNOWN - (int)n_know.elem(); i < KNOWN; i++)
		cortico.elem(i) = 40.0;
	else
	    for(i = KNOWN - (int)n_know.elem(); i < KNOWN; i++)
		cortico.elem(i) = 0.0;


	cout << "m  " << m << " Body direction    " << eye_dir.elem() << "\n";
	
	// response of spindle to muscle contraction:
	//    cosine ditribution (generalization)
	
	for (int k = 0; k< F1N; k++) {			
	    
	    // response of spindle to muscle contraction: 
	    // gaussian ditribution (generalization)
	    
	    deg = -180 + 3.6*k; // prefered cell direction
	    
	    spindle.elem(k) =  // Change this!
		NSLsaturation((pos_height.elem()*(cos((0.0 * eye_dir.elem() 
						       - deg)*3.1416/180))),
			      0,kx2.elem(),0,ky2.elem());
	}
	
	// calcul of teta for population
	
	double	pxs = 0;
	double	pys = 0;
	
	for (int j = 0; j < F1N; j++) {	
	    deg = (-90 + 1.8*j)* 3.1416/180;
	    
	    pxs = pxs + spindle.elem(j)*cos(deg);
	    pys = pys + spindle.elem(j)*sin(deg);
	}
	
	Ps = sqrt(pow(pxs,2) + pow(pys,2));
	tetas = atan2(pys, pxs) *180/3.1416;
	
	cout << Ps << "\t" << tetas << "\n";
	
	rn = 0;

	while((rn < n.elem())&&(!exit_flag)) {
	    cout<<"n   " << rn << "\n";
	    
	    NSL << "run" << ENDL;
	    
            if(DART.get_time() < DART.get_end_time() - DART.get_delta()){
              exit_flag = 1;
            }

//	    fprintf(fpteta,"%7.4f\t%7.4f\n", n.elem(), teta.elem());
	    
	    throw_direction.elem(rn + m*(int)n.elem()) = teta.elem();
	    rn = rn + 1;
	    
	    float gcsum;
	    
	    gcsum = GC.sum();
	    
	    //		cout<<gcsum<<"\n";
	    
	    cout << P << " Teta: " << teta << "\n";      
	}
	
	if (corok.elem() > .5) {
	    cor = NSLprod(GC,GC1);
	    cout << cor << "\n";
	}
//	WriteWeights(fpwf); 
    }
//    fclose(fpteta);
//    fclose(fpwf);
}

