# include "nsl_include.h" 
# include "math.h"

// ******************** Random number generator ****************

// The following is taken from "Numerical Recipes in C".

#define M1 259200
#define IA1 7141
#define IC1 54773
#define RM1 (1.0/M1)
#define M2 134456
#define IA2 8121
#define IC2 28411
#define RM2 (1.0/M2)
#define M3 243000
#define IA3 4561
#define IC3 51349

float ran1(int* idum)
{
  static long ix1,ix2,ix3;
  static float r[98];
  float temp;
  static int iff=0;
  int j;
  //	void nrerror();

  if (*idum < 0 || iff == 0) {
    iff=1;
    ix1=(IC1-(*idum)) % M1;
    ix1=(IA1*ix1+IC1) % M1;
    ix2=ix1 % M2;
    ix1=(IA1*ix1+IC1) % M1;
    ix3=ix1 % M3;
    for (j=1;j<=97;j++) {
      ix1=(IA1*ix1+IC1) % M1;
      ix2=(IA2*ix2+IC2) % M2;
      r[j]=(ix1+ix2*RM2)*RM1;
    }
    *idum=1;
  }
  ix1=(IA1*ix1+IC1) % M1;
  ix2=(IA2*ix2+IC2) % M2;
  ix3=(IA3*ix3+IC3) % M3;
  j= (int)(1 + ((97*ix3)/M3));
  //	if (j > 97 || j < 1) nrerror("RAN1: This cannot happen.");
  temp=r[j];
  r[j]=(ix1+ix2*RM2)*RM1;
  return temp;
}

#undef M1
#undef IA1
#undef IC1
#undef RM1
#undef M2
#undef IA2
#undef IC2
#undef RM2
#undef M3
#undef IA3
#undef IC3

// Gaussian random number generator

// This generates a gaussian R.V. with zero mean, unit variance

float gasdev(int* idum)
{
  static int iset=0;
  static float gset;
  float fac,r,v1,v2;
  //	float ran1();
  
  if  (iset == 0) {
    do {
      v1=2.0*ran1(idum)-1.0;
      v2=2.0*ran1(idum)-1.0;
      r=v1*v1+v2*v2;
    } while (r >= 1.0);
    fac=sqrt(-2.0*log(r)/r);
    gset=v1*fac;
    iset=1;
    return v2*fac;
  } 
  else {
    iset=0;
    return gset;
  }
}

//**************************** delay line ******************************/

nsl_data delayedOutput(nsl_data& inputValue,  nsl_vector& delayArray, 
			 nsl_data& ndelay)
{
  int i,n;
  nsl_data result;
	
  n = (int) ndelay.elem();
  delayArray.elem(n-1) = inputValue.elem();
	
  for (i=0; i<=n-2; i++)
    delayArray.elem(i) = delayArray.elem(i+1);
  
  result.elem() = delayArray.elem(0);
  return result;
}

//*************************************************************
// creation of a gaussian mask matrix */

nsl_matrix gaussm(int imax, int jmax, nsl_data& alpha, nsl_data& sig_x, 
                  nsl_data& sig_y)
{
  int      i, j;
  num_type r, tmp1, tmp2, tmp3, sum = 0;
  
  nsl_matrix Q(imax,jmax);

  int imax2 = imax/2;
  int jmax2 = jmax/2;

  for (i=0; i<imax; i++)
    for (j=0; j<jmax; j++){
      tmp1 = (num_type) pow((num_type)(i-imax2),2.0)/
	(num_type) pow(sig_x.elem(),2.0);

      tmp2 = (num_type) pow((num_type)(j-jmax2),2.0)/
	(num_type) pow(sig_y.elem(),2.0);

      tmp3 = alpha.elem() / (2 * sig_x.elem() * sig_y.elem());

      r = tmp3 * exp(-tmp1 - tmp2);

      Q.elem(i,j) = r;
      sum = sum + Q.elem(i,j);
    }
  
  if (sum == 0.0)
    Q = 0;

  return Q;
}


// creation of a gaussian mask vector

nsl_vector gaussv(int imax,  nsl_data& alpha, nsl_data& sig_x)
{
  int i;
  num_type r, tmp1, tmp2, sum = 0;
  
  nsl_vector Q(imax);
  
  int imax2 = imax/2;
  
  for (i=0; i<imax; i++){
    tmp1 = (num_type) pow((num_type)(i-imax2),2.0)/
      (num_type) pow(sig_x.elem(),2.0);
    
    tmp2 = alpha.elem();
    
    r = tmp2 * exp(-tmp1);
    
    Q.elem(i) = r;
    sum = sum + Q.elem(i);
  }
  
  if (sum == 0.0)
    Q = 0;
  
  return Q;
}

nsl_vector newconv(nsl_vector& w,nsl_vector& b)

// w is the Mask and b is the input layer, c the output layer.
{
  nsl_vector c_aux (35); 
  nsl_vector c (25); 
      
  for (int l = 0; l < 35; l++)
    c_aux.elem(l)=0;
  
  for (int i = 0 ; i < 25  ; i= i+ 5 ) {     
    // i < cmax - wmax and i = i + # inputs

      for (int m = 0; m < 10 ; m++) 
	c_aux.elem(m+i) =  c_aux.elem(m+i) + w.elem(m) * b.elem((int) (i/5));
  }
  
  for (int k =0; k < 25; k++)
    c.elem(k) = c_aux.elem(k+5);
  
  return c;
}


