////////////////////////////////////////////////////////////////////////////
// Movement generation simulation for hand transport and preshape
// Bruce Hoff
// October 22, 1991
// orient control update: 3/92
// filename:  fb_reach.c
////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include "nsl_include.h"

// dimension of transport simulation 
// (eg. 1 or 2 dim. hand movement)
#define TDIM 2

// dimension of preshape simulation
#define PDIM 1

// dimension of state
#define STATEDIM 3
#define MAXDELAY 1000
#define NormRand ((float)2*drand48()-(float)1)
#define MIN(x,y) ((x)<(y)?(x):(y))
#define MAX(x,y) ((x)>(y)?(x):(y))

#define noiseWindow 10
#define MAXXYSTEPS 1500
#define FIRSTXYPTR 4
#define GRAPHICMARGIN 0.10

// flags for perturbation experiments
#define T_NOPTRB 0
#define T_LEFTPTRB 1
#define T_RGHTPTRB 2

#define P_SMALLTARG 0
#define P_LARGETARG 1

#define P_NOPTRB 0
#define P_LRGEPTRB 1
#define P_SMLLPTRB 2

NETWORK(fb_reach);

// Simulation variables.  Throughout, the prefix
// T - indicates variables relating to simulation
// of transport; P - indicates preshape
VECTOR(Tpos, TDIM);	    
VECTOR(Tpos0, TDIM);	
VECTOR(Tvel, TDIM);	    
VECTOR(Tvel0, TDIM);	
VECTOR(Taccel, TDIM);	    
VECTOR(Taccel0, TDIM);
VECTOR(TTARGET, TDIM);
VECTOR(LeftTarget, TDIM);
VECTOR(RightTarget, TDIM);
VECTOR(Ttarget, TDIM);
VECTOR(delTtarget, TDIM);   
VECTOR(oldTTarg, TDIM);
VECTOR(origin, TDIM);
VECTOR(delTMTest, 1);
VECTOR(delPMTnETest, 1);
VECTOR(Tzero, TDIM);

MATRIX (po, STATEDIM, TDIM);
MATRIX (xo, STATEDIM, TDIM);

VECTOR(Ppos, PDIM);   //  <A HREF="fbr_params.html#Ppos"> Ppos is output graphically. </A>
VECTOR(Pvel, PDIM);   //  <A HREF="fbr_params.html#Pvel"> Pvel is output graphically. </A>
VECTOR(Paccel, PDIM);

VECTOR(maxPvel, PDIM);
VECTOR(Pinitial, PDIM);
VECTOR(Ptarget, PDIM);
VECTOR(delPtarget, PDIM);

VECTOR(SmallObjectSize, PDIM);
VECTOR(LargeObjectSize, PDIM);
VECTOR(A, PDIM);
VECTOR(B, PDIM);
VECTOR(C, PDIM);
VECTOR(Pzero, PDIM);

VECTOR(objSize, PDIM);
VECTOR(delObjSize, PDIM);

MATRIX(TtargetDA, MAXDELAY, TDIM);
MATRIX(TMTestDA, MAXDELAY, 1);
MATRIX(PMTnETestDA, MAXDELAY, 1);
MATRIX(PtargetDA, MAXDELAY, PDIM);

MATRIX(XYarray, MAXXYSTEPS, TDIM);

DATA(TposMag);  //    <A HREF="fbr_params.html#TposMag"> TposMag is output graphically. </A>
DATA(TvelMag);  //    <A HREF="fbr_params.html#TvelMag"> TvelMag is output graphically. </A>
DATA(TaccelMag);

DATA(appAngl);

DATA(PposMag);
DATA(PvelMag);

VECTOR(TMTest, 1);
VECTOR(PMTest, 1);
VECTOR(PMTnETest, 1);
VECTOR(TMVMT_TIME, 1);
VECTOR(PMVMT_TIME, 1);
VECTOR(ET, 1);

DATA(dMT_ML);
DATA(dMT_MR);
DATA(dMT_LS);
DATA(dMT_SL);
DATA(ptrbTime);
DATA(locDelay);
DATA(LPDelay);
DATA(TPTRBFLAG);

DATA(sizDelay);
DATA(STDelay);
DATA(PPTRBFLAG);	
DATA(PTARGFLAG);
DATA(Pwt);
DATA(maxPvelTime);

DATA(ta_m);	DATA(ta_b);
DATA(BATCH_FLAG);
DATA(fileTag);
DATA(termAccel);
DATA(transVers);

float TMT, PMT;
float oldPMT, oldTMT, oldEMT, oldPTarg, oldETarg;
float MT_increase;
float Pcoeff[4], Ecoeff[4];
float PinitTime, TinitTime, EinitTime;
float TINC;
float approachAngle;

int locStepDelay;
int sizStepDelay;
int LPStepDelay;
int STStepDelay;
int xyPtr;
int TptrbFlag, PptrbFlag;
int enclose_mode;

FILE *fout;


//////////////////////  UTILITIES  /////////////////////

void MatMult(nsl_matrix x, nsl_matrix y, nsl_matrix& z, int xr, int xc, int yc)
{
	int i, j, k;

	for (i=0; i < xr; i++)
		for (j=0; j < yc; j++)
			for (k=0, z.elem(i,j)=0; k < xc; k++)
				z.elem(i,j)+=x.elem(i,k)*y.elem(k,j);

	return;
}

float arrayMin(nsl_vector& v, int dim)
{
	int i;
	float extremum;

	extremum = v.elem(0);
	for (i=1; i < dim; i++)
		if (extremum > v.elem(i)) extremum = v.elem(i);
	return extremum;
}

float arrayMax(nsl_vector& v, int dim)
{
	int i;
	float extremum;

	extremum = v.elem(0);
	for (i=1; i < dim; i++)
		if (extremum < v.elem(i)) extremum = v.elem(i);
	return extremum;
}

void TstateCalcInit()
{
	int i;
	float ti, tf, L, tf2, cp, sp;
	nsl_matrix A(1, STATEDIM), phiMat(TDIM, 1);
	nsl_matrix phi11(STATEDIM, STATEDIM);
	nsl_matrix phi12Inv(STATEDIM, STATEDIM);
	nsl_matrix xf(STATEDIM, TDIM);
	nsl_matrix temp1(STATEDIM, 1), temp2(1,1), temp3(STATEDIM, TDIM);

	// printf("in TstateCalcInit\n");
	for (i=0; i < TDIM; i++) {
		xo.elem(2,i) = Taccel.elem(i); 
		xo.elem(1,i) = Tvel.elem(i);
		xo.elem(0,i) = Tpos.elem(i);
	} 
	ti = TinitTime = fb_reach.get_time()-TINC;
	tf = TMT;
	
	phiMat.elem(0, 0) = cp = cos(approachAngle);
	phiMat.elem(1, 0) = sp = sin(approachAngle);
	tf = tf-ti;
	tf2 = tf * tf;
	A.elem(0, 0) = -20/3/tf2;
	A.elem(0, 1) = -8/3/tf;
	A.elem(0, 2) = -1/3;
	MatMult(xo, phiMat, temp1, STATEDIM, TDIM, 1);
	MatMult(A, temp1, temp2, 1, STATEDIM, 1);
	
	if (transVers.elem()) {
		L = termAccel.elem()*temp2.elem(0,0)
			+(20/3/tf*(delTtarget.elem(0)*cp+delTtarget.elem(1)*sp));
	} else {
		L = -termAccel.elem();
	}

	// Now we can calculate target accel values, fill in "xf"
	xf = 0;
	// final pos is given
	xf.elem(0, 0) = delTtarget.elem(0); 
	xf.elem(0, 1) = delTtarget.elem(1);
	// final velocity is zero
	// final accel is calculated here
	xf.elem(2, 0) = L * cp; xf.elem(2, 1) = L * sp;

	// Now use xf to find po, which is used at run-time
	phi11.elem(0, 0) = 1; phi11.elem(0, 1) = tf; phi11.elem(0, 2) = .5*tf2;
	phi11.elem(1, 0) = 0; phi11.elem(1, 1) = 1; phi11.elem(1, 2) = tf;
	phi11.elem(2, 0) = 0; phi11.elem(2, 1) = 0; phi11.elem(2, 2) = 1;

	phi12Inv.elem(0, 0) = 240; phi12Inv.elem(0, 1) = -120*tf; 
	phi12Inv.elem(0, 2) = 20*tf2;

	phi12Inv.elem(1, 0) = 120*tf; phi12Inv.elem(1, 1) = -56*tf2; 
	phi12Inv.elem(1, 2) = 8*tf*tf2;

	phi12Inv.elem(2, 0) = 20*tf2; phi12Inv.elem(2, 1) = -8*tf*tf2; 
	phi12Inv.elem(2, 2) = tf2*tf2;

	phi12Inv = (3/tf/tf2/tf2) ^ phi12Inv;


	MatMult(phi11, xo, temp3, STATEDIM, STATEDIM, TDIM);
	MatMult(phi12Inv, (xf-temp3), po, STATEDIM, STATEDIM, TDIM);

	return;
}

void TstateCalc(nsl_vector& accel, nsl_vector& vel, nsl_vector& pos,
		float ti, float tf, float tc)
{
	int i;
	float t, t2;
	nsl_matrix phi12(STATEDIM, STATEDIM);
	nsl_matrix phi11(STATEDIM, STATEDIM);
	nsl_matrix x(STATEDIM, TDIM);
	nsl_matrix temp1(STATEDIM, TDIM), temp2(STATEDIM, TDIM); 

	// printf("in TstateCalc\n");
	if (tc < ti || tc > tf) {
		accel = 0;
		vel = 0;
		return;
	}
	
	t = tc - ti;
	t2 = t*t;
	
	phi11.elem(0, 0) = 1; phi11.elem(0, 1) = t; phi11.elem(0, 2) = .5*t2;
	phi11.elem(1, 0) = 0; phi11.elem(1, 1) = 1; phi11.elem(1, 2) = t;
	phi11.elem(2, 0) = 0; phi11.elem(2, 1) = 0; phi11.elem(2, 2) = 1;

	phi12.elem(0, 0) = t2*t2; phi12.elem(0, 1) = -5*t*t2; phi12.elem(0, 2) = 20*t2; 
	phi12.elem(1, 0) = 5*t*t2; phi12.elem(1, 1) = -20*t2; phi12.elem(1, 2) = 60*t; 
	phi12.elem(2, 0) = 20*t2; phi12.elem(2, 1) = -60*t; phi12.elem(2, 2) = 120; 

	phi12 = (t/120) ^ phi12;

	// x = NSLprod(phi11, xo) + NSLprod(phi12, po);
	MatMult(phi11, xo, temp1, STATEDIM, STATEDIM, TDIM);
	MatMult(phi12, po, temp2, STATEDIM, STATEDIM, TDIM);
	x = temp1 + temp2;

	for (i=0; i < TDIM; i++) {
		accel.elem(i) = x.elem(2,i); 
		vel.elem(i) = x.elem(1,i);
		pos.elem(i) = x.elem(0,i);
	}
	return;
}


void PFFCalcInit(float* initTime, float endTime,
		float xf, float vf, float xo, float vo, float* coeff)
{
	float time, TPval, expVal, sinVal, cosVal;
	float det, a, b, c, d, aSq, bSq, cSq, dSq;
	float m[4][4], bndry[4];
	int i, j;

	*initTime = fb_reach.get_time()-TINC;
	time = endTime - *initTime;
	TPval = MAX(time, TINC) / Pwt.elem();
	expVal = exp(TPval);
	sinVal = sin(TPval);
	cosVal = cos(TPval);
	a = cosVal*expVal; aSq = a*a;
	b = sinVal*expVal; bSq = b*b;
	c = cosVal/expVal; cSq = c*c;
	d = sinVal/expVal; dSq = d*d;
	det = aSq + bSq + cSq + dSq - 2*a*c - 6*b*d;
	if (fabs(det) < 1.0e-10) {
		printf("PFFCalcInit:  parameters undefined\n");
	}
	m[0][0] = a+b-c+d;
	m[0][1] = -b+d;
	m[0][2] = -a*c-b*c+c*c-a*d-3*b*d+d*d;
	m[0][3] = b*c-a*d-2*b*d;
	m[1][0] = -a+b+c-3*d;
	m[1][1] = a-c-2*d;
	m[1][2] = a*c-b*c-c*c+3*a*d-b*d-d*d;
	m[1][3] = -a*c+c*c+2*a*d-b*d+d*d;
	m[2][0] = -a-b+c-d;
	m[2][1] = b-d;
	m[2][2] = a*a+b*b-a*c+b*c+a*d-3*b*d;
	m[2][3] = -b*c+a*d+2*b*d;
	m[3][0] = -a-3*b+c+d;
	m[3][1] = -a+2*b+c;
	m[3][2] = a*a+b*b-a*c+3*b*c-a*d+b*d;
	m[3][3] = a*a+b*b-a*c-2*b*c-b*d;

	//  *** Note coordinate shift in x so that
	//  *** position cost is relative to relaxed 
	//  *** position, not x=0.
	bndry[0] = (xf-Pinitial.elem(0));
	bndry[1] = vf;
	bndry[2] = (xo-Pinitial.elem(0));
	bndry[3] = vo * Pwt.elem();

	for (i=0; i < 4; i++) {
		coeff[i] = 0;
		for (j=0; j < 4; j++) 
			coeff[i] += m[i][j]*bndry[j];
		coeff[i] /= det;
	}


	return;
}

// returns acceleration 'drive signal' 
void PFFDriveCalc(nsl_vector& accel, nsl_vector& vel, nsl_vector& pos,
	float ti, float tf, float tc, float* coeff)
{
	float DPwt, PwtSq, expVal, sinVal, cosVal;
	float a, b, c, d;

	if (tc < ti || tc > tf) {
		accel = 0;
		vel = 0;
		return;
	}
	
	// ***don't*** servo if movement is done
	// if () {
	// 	drive = (-pff_B ^ vel) - (pff_K ^ (pos-delPtarget));
	// 	Ppos = delPtarget;
	// 	Pvel = 0;
	// 	return drive;
	// }

	DPwt = (tc-ti) / Pwt.elem();
	PwtSq = Pwt.elem() * Pwt.elem();
	expVal = exp(DPwt);
	sinVal = sin(DPwt);
	cosVal = cos(DPwt);
	a = cosVal*expVal;
	b = sinVal*expVal;
	c = cosVal/expVal;
	d = sinVal/expVal;

	accel = ((float)2/PwtSq) *
		(-coeff[0]*b+coeff[1]*a+coeff[2]*d-coeff[3]*c);

	pos = Pinitial.elem(0) +
		coeff[0]*a+coeff[1]*b+coeff[2]*c+coeff[3]*d;

	vel =  ((float)1/Pwt.elem()) *
		(( coeff[0]+coeff[1])*a+
		 ( coeff[1]-coeff[0])*b+
		 (-coeff[2]+coeff[3])*c+
		 (-coeff[2]-coeff[3])*d);

	return;
}

nsl_vector delayedOutput(nsl_vector& inputValue, 
		nsl_matrix& delayArray, int n, int dim)
{
	int i, j;
	nsl_vector result(dim);

	n = MAX(n, 1);

	for (j=0; j < dim; j++) {
	 	result.elem(j) = delayArray.elem(0, j);
		for (i=0; i < n-1; i++)
			delayArray.elem(i, j) = delayArray.elem(i+1, j);
		delayArray.elem(n-1, j) = inputValue.elem(j);
	} 

	return result;
}

void initDA(nsl_matrix& delayArray, nsl_vector& val)
{
	int i, j, n, m;
	
	n = delayArray.get_imax();
	m = delayArray.get_jmax();
	for (i=0; i < n; i++)
		for (j=0; j < m; j++) 
			delayArray.elem(i, j) = val.elem(j);

	return;
}

float TtoMA(float T)
{
	return ta_m.elem() * T + ta_b.elem();
}

//////////////////// END OF UTILITIES /////////////////////////

INIT_MODULE(output_init)
{
	fileTag.set_data_type(3);
	fout = 0;
}

//<A NAME="TINCinit">
INIT_MODULE(TINCinit)
{
	TINC = fb_reach.get_delta();
}

//<A NAME="perturb_init">
// Depending on the selected perturbation experiment, set up
// the appropriate extended movement time.

INIT_MODULE(perturb_init)
{
	if (TPTRBFLAG.elem()==T_LEFTPTRB)
		MT_increase = dMT_ML.elem();
	if (TPTRBFLAG.elem()==T_RGHTPTRB)
		MT_increase = dMT_MR.elem();
	if (PPTRBFLAG.elem()==P_SMLLPTRB)
		MT_increase = dMT_LS.elem();
	if (PPTRBFLAG.elem()==P_LRGEPTRB)
		MT_increase = dMT_SL.elem();
}

//<A NAME="transport_init">
INIT_MODULE(transport_init)
{
	Tzero = 0;
	Ttarget = TTARGET;
	TptrbFlag = (int)TPTRBFLAG.elem();

	Taccel = 0;
	Tvel = 0;
	Tpos = 0;

	TposMag = 0;
	TvelMag = 0;
	TaccelMag = 0;

	oldTMT = 0; oldTTarg = 0;

	approachAngle = (3.14)/180 * appAngl.elem();

}

//<A NAME="preshape_init">
INIT_MODULE(preshape_init)
{
	Pzero = 0;
	objSize.elem(0) = (PTARGFLAG.elem()==P_SMALLTARG ?
		SmallObjectSize.elem(0) : LargeObjectSize.elem(0));
	Ptarget = TtoMA( objSize.elem(0) );
	PptrbFlag = (int)PPTRBFLAG.elem();

	Pvel = 0;
	Ppos = Pinitial;

	maxPvel = Pvel;
	maxPvelTime = fb_reach.get_time();

	oldPMT = 0;  oldPTarg  = 0; 
	PMT = fb_reach.get_end_time();

	enclose_mode = 0;
	oldEMT = 0; oldETarg = 0;
}

//<A NAME="delayInit"> 
INIT_MODULE(delayInit)
{
	locStepDelay = (int) (locDelay.elem()/TINC);
	if (locStepDelay > MAXDELAY)
		printf("*** Error: locDelay too long ***\n");
	initDA(TtargetDA, Ttarget);

	LPStepDelay = (int) (LPDelay.elem()/TINC);
	if (LPStepDelay > MAXDELAY)
		printf("*** Error:  LPDelay too long ***\n");
	initDA(TMTestDA, TMVMT_TIME);

	sizStepDelay = (int) (sizDelay.elem()/TINC);
	if (sizStepDelay > MAXDELAY)
		printf("*** Error: sizDelay too long ***\n");

	if (PTARGFLAG.elem()==P_SMALLTARG)
		objSize = SmallObjectSize;
	else  // PTARGFLAG==P_LARGETARG
		objSize = LargeObjectSize;

	initDA(PtargetDA, objSize);

	STStepDelay = (int) (STDelay.elem()/TINC);
	if (STStepDelay > MAXDELAY)
		printf("*** Error:  STDelay too long ***\n");
	initDA(PMTnETestDA, PMVMT_TIME);
}

// With proper X11 graphics codes, it's possible to display the trajectory 
// of the hand on the X-Y plane during the simulation. No such graphics
// codes are provided here at the moment.

INIT_MODULE(XYgraphInit)
{
	float low, high, temp;

	// set up X-Y graph array
	origin = 0;
	XYarray = 0;
	XYarray = XYarray + XYarray.set_row(origin, 0);
	XYarray = XYarray + XYarray.set_row(TTARGET, 1);
	XYarray = XYarray + XYarray.set_row(LeftTarget, 2);
	XYarray = XYarray + XYarray.set_row(RightTarget, 3);
	xyPtr = FIRSTXYPTR;

	low = arrayMin(origin, TDIM);
	temp = arrayMin(TTARGET, TDIM);
	low = MIN(low, temp);
	temp = arrayMin(LeftTarget, TDIM);
	low = MIN(low, temp);
	temp = arrayMin(RightTarget, TDIM);
	low = (1.0+GRAPHICMARGIN) * MIN(low, temp);

	high = arrayMax(origin, TDIM);
	temp = arrayMax(TTARGET, TDIM);
	high = MAX(high, temp);
	temp = arrayMax(LeftTarget, TDIM);
	high = MAX(high, temp);
	temp = arrayMax(RightTarget, TDIM);
	high = (1.0+GRAPHICMARGIN) * MAX(high, temp);
}

//<A NAME="location">
RUN_MODULE(location)
{
	float currTime = fb_reach.get_time();

	Ttarget=TTARGET;
	if (ptrbTime.elem() < currTime) {
		if (TptrbFlag==T_LEFTPTRB)
			Ttarget = LeftTarget;
		if (TptrbFlag==T_RGHTPTRB)
			Ttarget = RightTarget;
	}

	delTtarget = delayedOutput(Ttarget, TtargetDA, locStepDelay, TDIM);
}

//<A NAME="size">
RUN_MODULE(size)
{
	float currTime = fb_reach.get_time();

	if (PTARGFLAG.elem()==P_SMALLTARG)
		objSize = SmallObjectSize;
	else  // PTARGFLAG==P_LARGETARG
		objSize = LargeObjectSize;

	if (ptrbTime.elem() < currTime) {
		if (PptrbFlag==P_SMLLPTRB)
			objSize = SmallObjectSize;
		if (PptrbFlag==P_LRGEPTRB)
			objSize = LargeObjectSize;
	}

	delObjSize = delayedOutput(objSize, PtargetDA, sizStepDelay, PDIM);

}

//<A NAME="TargToMaxaperture">
RUN_MODULE(TargToMaxaperture)
{

	delPtarget.elem(0) = TtoMA(delObjSize.elem(0));
}

//<A NAME="TtimeNeeded">
RUN_MODULE(TtimeNeeded)
{
	float currTime = fb_reach.get_time();
	TMTest =  TMVMT_TIME;
	if ( TptrbFlag &&
		(ptrbTime.elem()+locDelay.elem() < currTime) ) {
		TMTest = TMTest + MT_increase;
	}
	delTMTest = delayedOutput(TMTest, TMTestDA, LPStepDelay, (int)1);
}

//<A NAME="PtimeNeeded">
RUN_MODULE(PtimeNeeded)
{
	float currTime = fb_reach.get_time();

	PMTest =  PMVMT_TIME;
	if ( PptrbFlag && (ptrbTime.elem()+sizDelay.elem() < currTime) )
			PMTest = PMTest + MT_increase;
	PMTnETest = PMTest + ET;
	delPMTnETest = delayedOutput(PMTnETest, PMTnETestDA, STStepDelay, (int)1);
}

//<A NAME="TMAX">
RUN_MODULE(TMAX)
{
	TMT = MAX(TMTest.elem(0), delPMTnETest.elem(0));
}

//<A NAME="PMAX">
RUN_MODULE(PMAX)
{
	PMT = MAX(delTMTest.elem(0), PMTnETest.elem(0))-ET.elem(0);
}

int floatEqual(float a, float b)
{
	return((int)(fabs(a-b) < .00001));
}

int vecEqual(nsl_vector& a, nsl_vector& b, int size)
{
	int equal = -1; // TRUE

	for (int i=0; (i < size) && equal; i++)
		equal = floatEqual(a.elem(i), b.elem(i));
	return equal;
}


RUN_MODULE(TtrajGen)
{
	if (!floatEqual(oldTMT, TMT) || !vecEqual(oldTTarg, delTtarget, TDIM)) {
		TstateCalcInit();
		oldTMT = TMT;  oldTTarg = delTtarget;
	}

	TstateCalc(Taccel, Tvel, Tpos, TinitTime, TMT, fb_reach.get_time());
}


RUN_MODULE(PreshapeTrajGen)
{
	float t = fb_reach.get_time();

	if (fb_reach.get_time() < PMT) {
		if (!floatEqual(oldPMT, PMT) || !floatEqual(oldPTarg, delPtarget.elem(0))) {
			PFFCalcInit(&PinitTime, PMT, delPtarget.elem(0), 
				(float)0, Ppos.elem(0), Pvel.elem(0), Pcoeff);
			oldPMT = PMT;  oldPTarg = delPtarget.elem(0);
		}

		PFFDriveCalc(Paccel, Pvel, Ppos, PinitTime, PMT, t, Pcoeff);
	} else {
		enclose_mode = 1;
	}
}


RUN_MODULE(EncloseTrajGen)
{
	float t = fb_reach.get_time();

	if (enclose_mode) {
		if (!floatEqual(oldEMT, PMT+ET.elem(0)) || !floatEqual(oldETarg, delObjSize.elem(0))) {

			// PFFCalcInit(&EinitTime, PMT+ET.elem(0), delObjSize.elem(0), 
			// 	(float)0, Ppos.elem(0), Pvel.elem(0), Ecoeff);

			// Make hand "head toward" complete closure
			PFFCalcInit(&EinitTime, PMT+ET.elem(0), Pinitial.elem(0), 
				(float)0, Ppos.elem(0), Pvel.elem(0), Ecoeff);

			oldEMT = PMT+ET.elem(0);  oldETarg = delObjSize.elem(0);
		}

		PFFDriveCalc(Paccel, Pvel, Ppos, EinitTime, PMT+ET.elem(0), t, Ecoeff);
	}
}

float dist(nsl_vector& x, nsl_vector& y, int size)
{
	int i;
	float d, temp;

	for (i=0, d=0; i < size; i++) {
		temp = x.elem(i)-y.elem(i);
		d += temp * temp;
	}
	return sqrt(d);
}


RUN_MODULE(dowelContact)
{
	// if hand is enclosing AND is within a dowel diameter 
	// of its target AND is smaller in aperture than the dowel
	// then apply the dowel size as a limit on the aperture
	if (enclose_mode && Ppos.elem(0) <= objSize.elem(0)
			&& dist(Tpos, Ttarget, TDIM) <= objSize.elem(0)) {
		Ppos = objSize;
		Pvel = 0;
	}
}

// This routine calculates the magnitudes 
// of the pos, vel, accel vectors
RUN_MODULE(magCalc)
{
	int i;

	TposMag = 0;
	TvelMag = 0;
	TaccelMag = 0;

	for (i=0; i < TDIM; i++) {
		TposMag.elem() += Tpos.elem(i)*Tpos.elem(i);
		TvelMag.elem() += Tvel.elem(i)*Tvel.elem(i);
		TaccelMag.elem() += Taccel.elem(i)*Taccel.elem(i);		
	}

	TposMag.elem() = sqrt(TposMag.elem());
	TvelMag.elem() = sqrt(TvelMag.elem());
	TaccelMag.elem() = sqrt(TaccelMag.elem());
}

RUN_MODULE(dataOut)
{ 
	float t = fb_reach.get_time();

	if (fout) {

		fprintf(fout, "%.5f\t%.5f\t%.5f\t%.5f\t%.5f\t%.5f\t%.5f\t%.5f\t%.5f\n", 
			t,
			Tpos.elem(0), Tpos.elem(1),
			TvelMag.elem(), TposMag.elem(), 
			Taccel.elem(0), Taccel.elem(1),
			Ppos.elem(0), Pvel.elem(0));

		if (floatEqual(t, fb_reach.get_end_time())) {
			// output the three targets
			fprintf(fout, "%.5f\t%.5f\t%.5f\n", t, 
				TTARGET.elem(0), TTARGET.elem(1));
			fprintf(fout, "%.5f\t%.5f\t%.5f\n", t, 
				LeftTarget.elem(0), LeftTarget.elem(1));
			fprintf(fout, "%.5f\t%.5f\t%.5f\n", t, 
				RightTarget.elem(0), RightTarget.elem(1));
			fclose(fout);
		}

	} else if (*fileTag.get_str_value() != '$') {
		fout = fopen(fileTag.get_str_value(), "w");
		if (!fout)
			printf("can't open output file\n");
		else
			fprintf(fout, 
				"*\nt (s)\tx (cm)\ty (cm)\tHand Speed (cm/s)\tHand Displacement (cm)\tX Accel (cm/s^2)\tY Accel (cm/s^2)\tAperture (cm)\tAperture Speed (cm/s)\n");
	}
}

// end of cycle "house cleaning"
RUN_MODULE(endOfCycle)
{
	// This chooses the array to be displayed
	// as an X-Y graph
	XYarray = XYarray + XYarray.set_row(Tpos, xyPtr++);

	// this keeps track of the time and value of 
	// the maximum preshape velocity value
	if (Pvel.elem(0) > maxPvel.elem(0)) {
		maxPvel = Pvel;
		maxPvelTime = fb_reach.get_time();
	}

	if (BATCH_FLAG.elem())
		printf("Pwt: %.3f, timestep: %.3f\n", 
			Pwt.elem(), fb_reach.get_time());
}





